@@ -25,13 +25,19 @@ import (
 
 	"github.com/apache/skywalking-banyandb/api/common"
 	"github.com/apache/skywalking-banyandb/api/data"
+	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/banyand/dquery"
 	"github.com/apache/skywalking-banyandb/banyand/liaison/grpc"
 	"github.com/apache/skywalking-banyandb/banyand/liaison/http"
+	"github.com/apache/skywalking-banyandb/banyand/measure"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/observability"
+	"github.com/apache/skywalking-banyandb/banyand/protector"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/queue/pub"
+	"github.com/apache/skywalking-banyandb/banyand/queue/sub"
+	"github.com/apache/skywalking-banyandb/banyand/stream"
+	"github.com/apache/skywalking-banyandb/banyand/trace"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/node"
 	"github.com/apache/skywalking-banyandb/pkg/run"
@@ -45,34 +51,69 @@ func newLiaisonCmd(runners ...run.Unit) *cobra.Command {
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate metadata service")
 	}
-	pipeline := pub.New(metaSvc)
+	tire1Client := pub.New(metaSvc, databasev1.Role_ROLE_LIAISON)
+	tire2Client := pub.New(metaSvc, databasev1.Role_ROLE_DATA)
 	localPipeline := queue.Local()
-	measureNodeSel := node.NewRoundRobinSelector(data.TopicMeasureWrite.String(), metaSvc)
-	measureNodeRegistry := grpc.NewClusterNodeRegistry(data.TopicMeasureWrite, pipeline, measureNodeSel)
-	metricSvc := observability.NewMetricService(metaSvc, pipeline, "liaison", measureNodeRegistry)
-	streamNodeSel := node.NewRoundRobinSelector(data.TopicStreamWrite.String(), metaSvc)
+
+	measureLiaisonNodeSel := node.NewRoundRobinSelector(data.TopicMeasureWrite.String(), metaSvc)
+	measureLiaisonNodeRegistry := grpc.NewClusterNodeRegistry(data.TopicMeasureWrite, tire1Client, measureLiaisonNodeSel)
+	measureDataNodeSel := node.NewRoundRobinSelector(data.TopicMeasureWrite.String(), metaSvc)
+	metricSvc := observability.NewMetricService(metaSvc, tire1Client, "liaison", measureLiaisonNodeRegistry)
+	pm := protector.NewMemory(metricSvc)
+	internalPipeline := sub.NewServerWithPorts(metricSvc, "liaison-server", 18912, 18913)
+	measureSVC, err := measure.NewLiaison(metaSvc, internalPipeline, metricSvc, pm, measureDataNodeSel, tire2Client)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate measure liaison service")
+	}
+
+	streamLiaisonNodeSel := node.NewRoundRobinSelector(data.TopicStreamWrite.String(), metaSvc)
+	streamDataNodeSel := node.NewRoundRobinSelector(data.TopicStreamPartSync.String(), metaSvc)
+	streamSVC, err := stream.NewLiaison(metaSvc, internalPipeline, metricSvc, pm, streamDataNodeSel, tire2Client)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate stream liaison service")
+	}
+
+	traceLiaisonNodeSel := node.NewRoundRobinSelector(data.TopicTraceWrite.String(), metaSvc)
+	traceDataNodeSel := node.NewRoundRobinSelector(data.TopicTracePartSync.String(), metaSvc)
+	traceSVC, err := trace.NewLiaison(metaSvc, internalPipeline, metricSvc, pm, traceDataNodeSel, tire2Client)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate trace liaison service")
+	}
+
 	propertyNodeSel := node.NewRoundRobinSelector(data.TopicPropertyUpdate.String(), metaSvc)
-	grpcServer := grpc.NewServer(ctx, pipeline, localPipeline, metaSvc, grpc.NodeRegistries{
-		MeasureNodeRegistry:  measureNodeRegistry,
-		StreamNodeRegistry:   grpc.NewClusterNodeRegistry(data.TopicStreamWrite, pipeline, streamNodeSel),
-		PropertyNodeRegistry: grpc.NewClusterNodeRegistry(data.TopicPropertyUpdate, pipeline, propertyNodeSel),
-	}, metricSvc)
-	profSvc := observability.NewProfService()
-	httpServer := http.NewServer()
-	dQuery, err := dquery.NewService(metaSvc, localPipeline, pipeline, metricSvc)
+
+	dQuery, err := dquery.NewService(metaSvc, localPipeline, tire2Client, metricSvc, streamSVC, measureSVC, traceSVC)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate distributed query service")
 	}
+
+	grpcServer := grpc.NewServer(ctx, tire1Client, tire2Client, localPipeline, metaSvc, grpc.NodeRegistries{
+		MeasureLiaisonNodeRegistry: measureLiaisonNodeRegistry,
+		StreamLiaisonNodeRegistry:  grpc.NewClusterNodeRegistry(data.TopicStreamWrite, tire1Client, streamLiaisonNodeSel),
+		PropertyNodeRegistry:       grpc.NewClusterNodeRegistry(data.TopicPropertyUpdate, tire2Client, propertyNodeSel),
+		TraceLiaisonNodeRegistry:   grpc.NewClusterNodeRegistry(data.TopicTraceWrite, tire1Client, traceLiaisonNodeSel),
+	}, metricSvc, pm)
+	profSvc := observability.NewProfService()
+	httpServer := http.NewServer(grpcServer.GetAuthReloader())
 	var units []run.Unit
 	units = append(units, runners...)
 	units = append(units,
 		metaSvc,
-		localPipeline,
-		pipeline,
-		measureNodeSel,
-		streamNodeSel,
-		propertyNodeSel,
 		metricSvc,
+		localPipeline,
+		internalPipeline,
+		tire1Client,
+		tire2Client,
+		measureLiaisonNodeSel,
+		measureDataNodeSel,
+		streamLiaisonNodeSel,
+		streamDataNodeSel,
+		traceLiaisonNodeSel,
+		traceDataNodeSel,
+		propertyNodeSel,
+		streamSVC,
+		measureSVC,
+		traceSVC,
 		dQuery,
 		grpcServer,
 		httpServer,
@@ -86,23 +127,26 @@ func newLiaisonCmd(runners ...run.Unit) *cobra.Command {
 		Version: version.Build(),
 		Short:   "Run as the liaison server",
 		RunE: func(_ *cobra.Command, _ []string) (err error) {
+			ctx := context.Background()
 			if nodeSelector != "" {
+				ctx = context.WithValue(ctx, common.ContextNodeSelectorKey, nodeSelector)
 				var ls *pub.LabelSelector
 				ls, err = pub.ParseLabelSelector(nodeSelector)
 				if err != nil {
 					return err
 				}
-				for _, sel := range []node.Selector{measureNodeSel, streamNodeSel, propertyNodeSel} {
+				for _, sel := range []node.Selector{measureDataNodeSel, streamDataNodeSel, propertyNodeSel, traceDataNodeSel} {
 					sel.SetNodeSelector(ls)
 				}
 			}
-			node, err := common.GenerateNode(grpcServer.GetPort(), httpServer.GetPort())
+			node, err := common.GenerateNode(internalPipeline.GetPort(), httpServer.GetPort(), nil)
 			if err != nil {
 				return err
 			}
 			logger.GetLogger().Info().Msg("starting as a liaison server")
+			ctx = context.WithValue(ctx, common.ContextNodeKey, node)
 			// Spawn our go routines and wait for shutdown.
-			if err := liaisonGroup.Run(context.WithValue(context.Background(), common.ContextNodeKey, node)); err != nil {
+			if err := liaisonGroup.Run(ctx); err != nil {
 				logger.GetLogger().Error().Err(err).Stack().Str("name", liaisonGroup.Name()).Msg("Exit")
 				os.Exit(-1)
 			}
