@@ -35,6 +35,7 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/query"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/stream"
+	"github.com/apache/skywalking-banyandb/banyand/trace"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
 	"github.com/apache/skywalking-banyandb/pkg/version"
@@ -43,51 +44,57 @@ import (
 func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 	l := logger.GetLogger("bootstrap")
 	ctx := context.Background()
-	pipeline := queue.Local()
+	dataPipeline := queue.Local()
 	metaSvc, err := embeddedserver.NewService(ctx)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate metadata service")
 	}
-	metricSvc := observability.NewMetricService(metaSvc, pipeline, "standalone", nil)
+	metricSvc := observability.NewMetricService(metaSvc, dataPipeline, "standalone", nil)
 	pm := protector.NewMemory(metricSvc)
-	propertySvc, err := property.NewService(metaSvc, pipeline, metricSvc)
+	propertySvc, err := property.NewService(metaSvc, dataPipeline, nil, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate property service")
 	}
-	streamSvc, err := stream.NewService(metaSvc, pipeline, metricSvc, pm)
+	streamSvc, err := stream.NewService(metaSvc, dataPipeline, metricSvc, pm, nil)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate stream service")
 	}
+	traceSvc, err := trace.NewService(metaSvc, dataPipeline, metricSvc, pm)
+	if err != nil {
+		l.Fatal().Err(err).Msg("failed to initiate trace service")
+	}
 	var srvMetrics *grpcprom.ServerMetrics
 	srvMetrics.UnaryServerInterceptor()
 	srvMetrics.UnaryServerInterceptor()
-	measureSvc, err := measure.NewService(metaSvc, pipeline, nil, metricSvc, pm)
+	measureSvc, err := measure.NewStandalone(metaSvc, dataPipeline, nil, metricSvc, pm)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate measure service")
 	}
-	q, err := query.NewService(ctx, streamSvc, measureSvc, metaSvc, pipeline)
+	q, err := query.NewService(ctx, streamSvc, measureSvc, traceSvc, metaSvc, dataPipeline)
 	if err != nil {
 		l.Fatal().Err(err).Msg("failed to initiate query processor")
 	}
 	nr := grpc.NewLocalNodeRegistry()
-	grpcServer := grpc.NewServer(ctx, pipeline, pipeline, metaSvc, grpc.NodeRegistries{
-		MeasureNodeRegistry:  nr,
-		StreamNodeRegistry:   nr,
-		PropertyNodeRegistry: nr,
-	}, metricSvc)
+	grpcServer := grpc.NewServer(ctx, dataPipeline, dataPipeline, dataPipeline, metaSvc, grpc.NodeRegistries{
+		MeasureLiaisonNodeRegistry: nr,
+		StreamLiaisonNodeRegistry:  nr,
+		PropertyNodeRegistry:       nr,
+		TraceLiaisonNodeRegistry:   nr,
+	}, metricSvc, pm)
 	profSvc := observability.NewProfService()
-	httpServer := http.NewServer()
+	httpServer := http.NewServer(grpcServer.GetAuthReloader())
 
 	var units []run.Unit
 	units = append(units, runners...)
 	units = append(units,
-		pipeline,
+		dataPipeline,
 		metaSvc,
 		metricSvc,
 		pm,
 		propertySvc,
 		measureSvc,
 		streamSvc,
+		traceSvc,
 		q,
 		grpcServer,
 		httpServer,
@@ -102,7 +109,7 @@ func newStandaloneCmd(runners ...run.Unit) *cobra.Command {
 		Version: version.Build(),
 		Short:   "Run as the standalone server",
 		RunE: func(_ *cobra.Command, _ []string) (err error) {
-			nodeID, err := common.GenerateNode(grpcServer.GetPort(), httpServer.GetPort())
+			nodeID, err := common.GenerateNode(grpcServer.GetPort(), httpServer.GetPort(), nil)
 			if err != nil {
 				return err
 			}
