@@ -15,6 +15,7 @@
 // specific language governing permissions and limitations
 // under the License.
 
+//revive:disable:var-naming // "common" is a valid package name for shared API types
 package common
 
 import (
@@ -152,6 +153,8 @@ type Node struct {
 	NodeID      string
 	GrpcAddress string
 	HTTPAddress string
+
+	PropertyGossipGrpcAddress string
 }
 
 var (
@@ -193,7 +196,7 @@ func ParseNodeHostProvider(s string) (NodeHostProvider, error) {
 }
 
 // GenerateNode generates a node id.
-func GenerateNode(grpcPort, httpPort *uint32) (node Node, err error) {
+func GenerateNode(grpcPort, httpPort, propertyGrpcPort *uint32) (node Node, err error) {
 	port := grpcPort
 	if port == nil {
 		port = httpPort
@@ -227,6 +230,9 @@ func GenerateNode(grpcPort, httpPort *uint32) (node Node, err error) {
 	if httpPort != nil {
 		node.HTTPAddress = net.JoinHostPort(nodeHost, strconv.FormatUint(uint64(*httpPort), 10))
 	}
+	if propertyGrpcPort != nil {
+		node.PropertyGossipGrpcAddress = net.JoinHostPort(nodeHost, strconv.FormatUint(uint64(*propertyGrpcPort), 10))
+	}
 	node.Labels = ParseNodeFlags()
 	return node, nil
 }
@@ -253,3 +259,8 @@ type contextNodeKey struct{}
 var ContextNodeRolesKey = contextNodeRolesKey{}
 
 type contextNodeRolesKey struct{}
+
+// ContextNodeSelectorKey is a context key to store the node selector.
+var ContextNodeSelectorKey = contextNodeSelectorKey{}
+
+type contextNodeSelectorKey struct{}
