@@ -51,6 +51,44 @@ func Uint32ToBytes(u uint32) []byte {
 	return bs
 }
 
+// Int32ToBytes converts int32 to bytes.
+func Int32ToBytes(i int32) []byte {
+	abs := i
+	if i < 0 {
+		abs = -abs
+	}
+	u := uint32(abs)
+	if i >= 0 {
+		u |= 1 << 31
+	} else {
+		u = (1 << 31) - u
+	}
+	return Uint32ToBytes(u)
+}
+
+// Int16ToBytes converts int16 to bytes using BigEndian.
+func Int16ToBytes(i int16) []byte {
+	bs := make([]byte, 2)
+	binary.BigEndian.PutUint16(bs, uint16(i))
+	return bs
+}
+
+// BytesToInt16 converts a 2-byte slice to int16 using BigEndian.
+func BytesToInt16(b []byte) int16 {
+	if len(b) < 2 {
+		panic("BytesToInt16: byte slice too short")
+	}
+	return int16(binary.BigEndian.Uint16(b))
+}
+
+// BoolToBytes converts bool to bytes.
+func BoolToBytes(b bool) []byte {
+	if b {
+		return []byte{1}
+	}
+	return []byte{0}
+}
+
 // BytesToInt64 converts bytes to int64.
 func BytesToInt64(b []byte) int64 {
 	u := binary.BigEndian.Uint64(b)
@@ -66,6 +104,16 @@ func BytesToInt64(b []byte) int64 {
 	return abs
 }
 
+// BytesToInt32 converts bytes to int32 based on the encoding used in Int32ToBytes.
+func BytesToInt32(b []byte) int32 {
+	u := binary.BigEndian.Uint32(b)
+	if (u >> 31) == 1 {
+		// Positive number
+		return int32(u &^ (1 << 31)) // clear the sign bit
+	}
+	return -int32((1 << 31) - u)
+}
+
 // BytesToUint64 converts bytes to uint64.
 func BytesToUint64(b []byte) uint64 {
 	return binary.BigEndian.Uint64(b)
@@ -87,3 +135,11 @@ func Float64ToBytes(f float64) []byte {
 func BytesToFloat64(b []byte) float64 {
 	return math.Float64frombits(binary.BigEndian.Uint64(b))
 }
+
+// BytesToBool converts bytes to bool.
+func BytesToBool(b []byte) bool {
+	if len(b) == 0 {
+		return false
+	}
+	return b[0] != 0
+}
