@@ -25,6 +25,7 @@ import (
 	g "github.com/onsi/ginkgo/v2"
 	"github.com/onsi/gomega"
 
+	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/flow"
 	"github.com/apache/skywalking-banyandb/pkg/test/flags"
 	flowTest "github.com/apache/skywalking-banyandb/pkg/test/flow"
@@ -61,7 +62,7 @@ var _ = g.Describe("Streaming", func() {
 
 		g.JustBeforeEach(func() {
 			snk = newSlice()
-			f = New(input).
+			f = New("test", input).
 				Filter(filter).
 				To(snk)
 			errCh = f.Open()
@@ -98,7 +99,7 @@ var _ = g.Describe("Streaming", func() {
 
 		g.JustBeforeEach(func() {
 			snk = newSlice()
-			f = New(input).
+			f = New("test", input).
 				Map(mapper).
 				To(snk)
 			errCh = f.Open()
@@ -143,17 +144,20 @@ var _ = g.Describe("Streaming", func() {
 		g.JustBeforeEach(func() {
 			snk = newSlice()
 
-			f = New(flowTest.NewSlice(input)).
+			f = New("test", flowTest.NewSlice(input)).
 				Map(flow.UnaryFunc[any](func(_ context.Context, item interface{}) interface{} {
 					// groupBy
-					return flow.Data{item.(*record).service, int64(item.(*record).value)}
-				})).
-				Window(NewTumblingTimeWindows(15*time.Second)).
-				TopN(3, WithSortKeyExtractor(func(record flow.StreamRecord) int64 {
-					return record.Data().(flow.Data)[1].(int64)
-				}), OrderBy(ASC), WithGroupKeyExtractor(func(record flow.StreamRecord) string {
-					return record.Data().(flow.Data)[0].(string)
+					return flow.Data{item.(*record).service, int64(item.(*record).value), item.(*record).service + item.(*record).instance}
 				})).
+				Window(NewTumblingTimeWindows(15*time.Second, 15*time.Second)).
+				TopN(3, WithKeyExtractor(func(record flow.StreamRecord) uint64 {
+					return convert.HashStr(record.Data().(flow.Data)[2].(string))
+				}),
+					WithSortKeyExtractor(func(record flow.StreamRecord) int64 {
+						return record.Data().(flow.Data)[1].(int64)
+					}), OrderBy(ASC), WithGroupKeyExtractor(func(record flow.StreamRecord) string {
+						return record.Data().(flow.Data)[0].(string)
+					})).
 				To(snk)
 
 			errCh = f.Open()
@@ -179,15 +183,15 @@ var _ = g.Describe("Streaming", func() {
 					g.Expect(len(snk.Value())).Should(gomega.BeNumerically(">=", 1))
 					// e2e-service-consumer Group
 					g.Expect(snk.Value()[0].(flow.StreamRecord).Data().(map[string][]*Tuple2)["e2e-service-consumer"]).Should(gomega.BeEquivalentTo([]*Tuple2{
-						{int64(9500), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9500)}, 7000)},
-						{int64(9600), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9600)}, 6000)},
-						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9700)}, 4000)},
+						{int64(9500), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9500), "e2e-service-consumerinstance-001"}, 7000)},
+						{int64(9600), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9600), "e2e-service-consumerinstance-004"}, 6000)},
+						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9700), "e2e-service-consumerinstance-002"}, 4000)},
 					}))
 					// e2e-service-provider Group
 					g.Expect(snk.Value()[0].(flow.StreamRecord).Data().(map[string][]*Tuple2)["e2e-service-provider"]).Should(gomega.BeEquivalentTo([]*Tuple2{
-						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9700)}, 5000)},
-						{int64(9800), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9800)}, 3000)},
-						{int64(10000), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(10000)}, 1000)},
+						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9700), "e2e-service-providerinstance-003"}, 5000)},
+						{int64(9800), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9800), "e2e-service-providerinstance-002"}, 3000)},
+						{int64(10000), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(10000), "e2e-service-providerinstance-001"}, 1000)},
 					}))
 				}).WithTimeout(flags.EventuallyTimeout).Should(gomega.Succeed())
 			})
@@ -206,13 +210,15 @@ var _ = g.Describe("Streaming", func() {
 		g.JustBeforeEach(func() {
 			snk = newSlice()
 
-			f = New(flowTest.NewSlice(input)).
+			f = New("test", flowTest.NewSlice(input)).
 				Map(flow.UnaryFunc[any](func(_ context.Context, item interface{}) interface{} {
 					// groupBy
-					return flow.Data{item.(*record).service, int64(item.(*record).value)}
+					return flow.Data{item.(*record).service, int64(item.(*record).value), item.(*record).service + item.(*record).instance}
 				})).
-				Window(NewTumblingTimeWindows(15*time.Second)).
-				TopN(3, WithSortKeyExtractor(func(record flow.StreamRecord) int64 {
+				Window(NewTumblingTimeWindows(15*time.Second, 15*time.Second)).
+				TopN(3, WithKeyExtractor(func(record flow.StreamRecord) uint64 {
+					return convert.HashStr(record.Data().(flow.Data)[2].(string))
+				}), WithSortKeyExtractor(func(record flow.StreamRecord) int64 {
 					return record.Data().(flow.Data)[1].(int64)
 				}), WithGroupKeyExtractor(func(record flow.StreamRecord) string {
 					return record.Data().(flow.Data)[0].(string)
@@ -242,15 +248,15 @@ var _ = g.Describe("Streaming", func() {
 					g.Expect(len(snk.Value())).Should(gomega.BeNumerically(">=", 1))
 					// e2e-service-consumer Group
 					g.Expect(snk.Value()[0].(flow.StreamRecord).Data().(map[string][]*Tuple2)["e2e-service-consumer"]).Should(gomega.BeEquivalentTo([]*Tuple2{
-						{int64(9900), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9900)}, 2000)},
-						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9700)}, 4000)},
-						{int64(9600), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9600)}, 6000)},
+						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9700), "e2e-service-consumerinstance-002"}, 4000)},
+						{int64(9600), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9600), "e2e-service-consumerinstance-004"}, 6000)},
+						{int64(9500), flow.NewStreamRecord(flow.Data{"e2e-service-consumer", int64(9500), "e2e-service-consumerinstance-001"}, 7000)},
 					}))
 					// e2e-service-provider Group
 					g.Expect(snk.Value()[0].(flow.StreamRecord).Data().(map[string][]*Tuple2)["e2e-service-provider"]).Should(gomega.BeEquivalentTo([]*Tuple2{
-						{int64(10000), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(10000)}, 1000)},
-						{int64(9800), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9800)}, 3000)},
-						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9700)}, 5000)},
+						{int64(10000), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(10000), "e2e-service-providerinstance-001"}, 1000)},
+						{int64(9800), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9800), "e2e-service-providerinstance-002"}, 3000)},
+						{int64(9700), flow.NewStreamRecord(flow.Data{"e2e-service-provider", int64(9700), "e2e-service-providerinstance-003"}, 5000)},
 					}))
 				}).WithTimeout(flags.EventuallyTimeout).Should(gomega.Succeed())
 			})
