@@ -33,22 +33,24 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/pool"
 )
 
-const defaultIOSize = 256 * 1024
-
 // localFileSystem implements the File System interface.
 type localFileSystem struct {
 	logger *logger.Logger
+	ioSize int
 }
 
 // LocalFile implements the File interface.
 type LocalFile struct {
-	file *os.File
+	file   *os.File
+	ioSize int
+	cached bool // Caching decision from upper layer
 }
 
 // NewLocalFileSystem is used to create the Local File system.
 func NewLocalFileSystem() FileSystem {
 	return &localFileSystem{
 		logger: logger.GetLogger(moduleName),
+		ioSize: 0,
 	}
 }
 
@@ -56,9 +58,26 @@ func NewLocalFileSystem() FileSystem {
 func NewLocalFileSystemWithLogger(parent *logger.Logger) FileSystem {
 	return &localFileSystem{
 		logger: parent.Named(moduleName),
+		ioSize: 0,
 	}
 }
 
+// NewLocalFileSystemWithLoggerAndLimit is used to create the Local File system with logger and limit,
+// limit is used to set the IO size.
+func NewLocalFileSystemWithLoggerAndLimit(parent *logger.Logger, limit uint64) FileSystem {
+	ioSize := limit2IOSize(limit)
+	return &localFileSystem{
+		logger: parent.Named(moduleName),
+		ioSize: ioSize,
+	}
+}
+
+// Limit2IOSize is used to transfer protector memory limit to IO size.
+func limit2IOSize(limit uint64) int {
+	ioSize := limit / 1024 / 10
+	return max(4*1024, min(256*1024, int(ioSize)))
+}
+
 func readErrorHandle(operation string, err error, name string, size int) (int, error) {
 	switch {
 	case errors.Is(err, io.EOF):
@@ -131,7 +150,8 @@ func (fs *localFileSystem) CreateFile(name string, permission Mode) (File, error
 	switch {
 	case err == nil:
 		return &LocalFile{
-			file: file,
+			file:   file,
+			ioSize: fs.ioSize,
 		}, nil
 	case os.IsExist(err):
 		return nil, &FileSystemError{
@@ -156,7 +176,8 @@ func (fs *localFileSystem) OpenFile(name string) (File, error) {
 	switch {
 	case err == nil:
 		return &LocalFile{
-			file: file,
+			file:   file,
+			ioSize: fs.ioSize,
 		}, nil
 	case os.IsNotExist(err):
 		return nil, &FileSystemError{
@@ -280,6 +301,14 @@ func (fs *localFileSystem) MustGetFreeSpace(path string) uint64 {
 	return usage.Free
 }
 
+func (fs *localFileSystem) MustGetTotalSpace(path string) uint64 {
+	usage, err := disk.Usage(path)
+	if err != nil {
+		fs.logger.Panic().Str("path", path).Err(err).Msg("failed to get disk usage")
+	}
+	return usage.Total
+}
+
 func (fs *localFileSystem) CreateHardLink(srcPath, destPath string, filter func(string) bool) error {
 	fi, err := os.Stat(srcPath)
 	if err != nil {
@@ -431,8 +460,22 @@ func (file *LocalFile) Writev(iov *[][]byte) (int, error) {
 
 // SequentialWrite supports appending consecutive buffers to the end of the file.
 func (file *LocalFile) SequentialWrite() SeqWriter {
-	writer := generateWriter(file.file)
-	return &seqWriter{writer: writer, fileName: file.file.Name()}
+	writer := generateWriter(file.file, file.ioSize)
+	return &seqWriter{writer: writer, file: file.file, fileName: file.file.Name(), skipFadvise: file.cached}
+}
+
+// SequentialRead is used to read the entire file using streaming read.
+// If cached is true, fadvise will not be applied.
+func (file *LocalFile) SequentialRead() SeqReader {
+	// Use the cached parameter from upper layer instead of local file's cached field
+	// This allows override when needed (e.g. metadata always cached)
+
+	// Reset file position to beginning to ensure each SeqReader starts from the beginning
+	// If seek fails, we'll still try to create the reader and the error will be handled when reading
+	_, _ = file.file.Seek(0, 0)
+
+	reader := generateReader(file.file, file.ioSize)
+	return &seqReader{reader: reader, file: file.file, fileName: file.file.Name(), length: 0, skipFadvise: file.cached}
 }
 
 // Read is used to read a specified location of file.
@@ -448,24 +491,19 @@ func (file *LocalFile) Read(offset int64, buffer []byte) (int, error) {
 // Readv is used to read contiguous regions of a file and disperse them into discontinuous buffers.
 func (file *LocalFile) Readv(offset int64, iov *[][]byte) (int, error) {
 	var size int
+	currentOffset := offset
 	for _, buffer := range *iov {
-		rsize, err := file.file.ReadAt(buffer, offset)
+		rsize, err := file.file.ReadAt(buffer, currentOffset)
 		if err != nil {
 			return readErrorHandle("Readv operation", err, file.file.Name(), rsize)
 		}
 		size += rsize
-		offset += int64(rsize)
+		currentOffset += int64(rsize)
 	}
 
 	return size, nil
 }
 
-// SequentialRead is used to read the entire file using streaming read.
-func (file *LocalFile) SequentialRead() SeqReader {
-	reader := generateReader(file.file)
-	return &seqReader{reader: reader, fileName: file.file.Name()}
-}
-
 // Size is used to get the file written data's size and return an error if the file does not exist. The unit of file size is Byte.
 func (file *LocalFile) Size() (int64, error) {
 	fileInfo, err := os.Stat(file.file.Name())
@@ -510,15 +548,27 @@ func (file *LocalFile) Close() error {
 }
 
 type seqReader struct {
-	reader   *bufio.Reader
-	fileName string
+	reader      *bufio.Reader
+	file        *os.File
+	fileName    string
+	length      int64
+	skipFadvise bool
 }
 
 func (i *seqReader) Read(p []byte) (int, error) {
 	rsize, err := i.reader.Read(p)
+	if rsize > 0 && i.file != nil && !i.skipFadvise {
+		offset := i.length
+		// Apply fadvise directly without threshold checking since decision was made at upper layer
+		_ = applyFadviseToFD(i.file.Fd(), offset, int64(rsize))
+		i.length += int64(rsize)
+	} else if rsize > 0 {
+		i.length += int64(rsize)
+	}
 	if err != nil {
 		return readErrorHandle("ReadStream operation", err, i.fileName, rsize)
 	}
+
 	return rsize, nil
 }
 
@@ -527,17 +577,48 @@ func (i *seqReader) Path() string {
 }
 
 func (i *seqReader) Close() error {
+	if i.file != nil {
+		if !i.skipFadvise {
+			_ = applyFadviseToFD(i.file.Fd(), 0, 0)
+		}
+	}
 	releaseReader(i.reader)
 	return nil
 }
 
 type seqWriter struct {
-	writer   *bufio.Writer
-	fileName string
+	writer      *bufio.Writer
+	file        *os.File
+	fileName    string
+	length      int64
+	skipFadvise bool
 }
 
 func (w *seqWriter) Write(p []byte) (n int, err error) {
-	return w.writer.Write(p)
+	n, err = w.writer.Write(p)
+	if n > 0 && w.file != nil && !w.skipFadvise {
+		if flushErr := w.writer.Flush(); flushErr != nil {
+			return n, &FileSystemError{
+				Code:    flushError,
+				Message: fmt.Sprintf("Flush File error, directory: %s, error: %s", w.fileName, flushErr),
+			}
+		}
+
+		offset := w.length
+		// Apply fadvise directly without threshold checking since decision was made at upper layer
+		_ = applyFadviseToFD(w.file.Fd(), offset, int64(n))
+		w.length += int64(n)
+	} else if n > 0 {
+		w.length += int64(n)
+	}
+
+	if err != nil {
+		return n, &FileSystemError{
+			Code:    writeError,
+			Message: fmt.Sprintf("Write File error, directory: %s, error: %s", w.fileName, err),
+		}
+	}
+	return n, nil
 }
 
 func (w *seqWriter) Path() string {
@@ -552,13 +633,17 @@ func (w *seqWriter) Close() error {
 			Message: fmt.Sprintf("Flush File error, directory name: %s, error message: %s", w.fileName, err),
 		}
 	}
+
+	if w.file != nil && !w.skipFadvise {
+		_ = SyncAndDropCache(w.file.Fd(), 0, 0)
+	}
 	return nil
 }
 
-func generateReader(f *os.File) *bufio.Reader {
+func generateReader(f *os.File, ioSize int) *bufio.Reader {
 	v := bufReaderPool.Get()
 	if v == nil {
-		return bufio.NewReaderSize(f, defaultIOSize)
+		return bufio.NewReaderSize(f, ioSize)
 	}
 	br := v
 	br.Reset(f)
@@ -572,10 +657,10 @@ func releaseReader(br *bufio.Reader) {
 
 var bufReaderPool = pool.Register[*bufio.Reader]("fs-bufReader")
 
-func generateWriter(f *os.File) *bufio.Writer {
+func generateWriter(f *os.File, ioSize int) *bufio.Writer {
 	v := bufWriterPool.Get()
 	if v == nil {
-		return bufio.NewWriterSize(f, defaultIOSize)
+		return bufio.NewWriterSize(f, ioSize)
 	}
 	bw := v
 	bw.Reset(f)
