@@ -44,18 +44,12 @@ func RunServiceName(t *testing.T, store SimpleStore) {
 	}{
 		{
 			name: "match gateway",
-			arg: index.Field{
-				Key:  serviceName,
-				Term: []byte("gateway"),
-			},
+			arg:  index.NewStringField(serviceName, "gateway"),
 			want: roaring.NewRange(0, 50),
 		},
 		{
 			name: "match webpage",
-			arg: index.Field{
-				Key:  serviceName,
-				Term: []byte("webpage"),
-			},
+			arg:  index.NewStringField(serviceName, "webpage"),
 			want: roaring.NewRange(50, 100),
 		},
 		{
@@ -64,16 +58,13 @@ func RunServiceName(t *testing.T, store SimpleStore) {
 		},
 		{
 			name: "unknown term",
-			arg: index.Field{
-				Key:  serviceName,
-				Term: []byte("unknown"),
-			},
+			arg:  index.NewStringField(serviceName, "unknown"),
 			want: roaring.DummyPostingList,
 		},
 	}
 	for _, tt := range tests {
 		t.Run(tt.name, func(_ *testing.T) {
-			list, err := store.MatchTerms(tt.arg)
+			list, _, err := store.MatchTerms(tt.arg)
 			if tt.wantErr {
 				tester.Error(err)
 				return
@@ -87,17 +78,23 @@ func RunServiceName(t *testing.T, store SimpleStore) {
 
 // SetUp initializes a index repository.
 func SetUp(t *assert.Assertions, store SimpleStore) {
+	var batch index.Batch
 	for i := 0; i < 100; i++ {
 		if i < 100/2 {
-			t.NoError(store.Write([]index.Field{{
-				Key:  serviceName,
-				Term: []byte("gateway"),
-			}}, uint64(i)))
+			batch.Documents = append(batch.Documents, index.Document{
+				Fields: []index.Field{
+					index.NewStringField(serviceName, "gateway"),
+				},
+				DocID: uint64(i),
+			})
 		} else {
-			t.NoError(store.Write([]index.Field{{
-				Key:  serviceName,
-				Term: []byte("webpage"),
-			}}, uint64(i)))
+			batch.Documents = append(batch.Documents, index.Document{
+				Fields: []index.Field{
+					index.NewStringField(serviceName, "webpage"),
+				},
+				DocID: uint64(i),
+			})
 		}
 	}
+	t.NoError(store.Batch(batch))
 }
