@@ -117,8 +117,10 @@ tag_families:
     tags:
       - name: id
         type: TAG_TYPE_STRING
+      - name: tag1
+        type: TAG_TYPE_STRING
 entity:
-  tagNames: ["tag1"]`))
+  tagNames: ["id"]`))
 		out := capturer.CaptureStdout(func() {
 			err := rootCmd.Execute()
 			Expect(err).NotTo(HaveOccurred())
@@ -133,7 +135,7 @@ entity:
 		helpers.UnmarshalYAML([]byte(out), resp)
 		Expect(resp.Measure.Metadata.Group).To(Equal("group1"))
 		Expect(resp.Measure.Metadata.Name).To(Equal("name1"))
-		Expect(resp.Measure.Entity.TagNames[0]).To(Equal("tag1"))
+		Expect(resp.Measure.TagFamilies[0].Tags[1].GetName()).To(Equal("tag1"))
 	})
 
 	It("delete measure schema", func() {
@@ -177,7 +179,8 @@ entity:
 		})
 		resp := new(databasev1.MeasureRegistryServiceListResponse)
 		helpers.UnmarshalYAML([]byte(out), resp)
-		Expect(resp.Measure).To(HaveLen(2))
+		// There is a _topn_result measure created by default
+		Expect(resp.Measure).To(HaveLen(3))
 	})
 
 	AfterEach(func() {
@@ -193,7 +196,9 @@ var _ = Describe("Measure Data Query", func() {
 	var startStr, endStr string
 	var interval time.Duration
 	BeforeEach(func() {
-		now = timestamp.NowMilli()
+		var err error
+		now, err = time.ParseInLocation("2006-01-02T15:04:05", "2021-09-01T23:30:00", time.Local)
+		Expect(err).NotTo(HaveOccurred())
 		startStr = now.Add(-20 * time.Minute).Format(time.RFC3339)
 		interval = 1 * time.Millisecond
 		endStr = now.Add(5 * time.Minute).Format(time.RFC3339)
@@ -204,7 +209,7 @@ var _ = Describe("Measure Data Query", func() {
 	})
 
 	It("query all measure data", func() {
-		conn, err := grpclib.Dial(
+		conn, err := grpclib.NewClient(
 			grpcAddr,
 			grpclib.WithTransportCredentials(insecure.NewCredentials()),
 		)
@@ -240,7 +245,7 @@ tagProjection:
 	})
 
 	DescribeTable("query measure data with time range flags", func(timeArgs ...string) {
-		conn, err := grpclib.Dial(
+		conn, err := grpclib.NewClient(
 			grpcAddr,
 			grpclib.WithTransportCredentials(insecure.NewCredentials()),
 		)
