@@ -26,13 +26,13 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/version"
 )
 
-func newUserCmd() *cobra.Command {
+func newUseCmd() *cobra.Command {
 	return &cobra.Command{
 		Use:     "use group",
 		Version: version.Build(),
 		Short:   "Select a group",
 		Args:    cobra.ExactArgs(1),
-		RunE: func(cmd *cobra.Command, args []string) (err error) {
+		RunE: func(_ *cobra.Command, args []string) (err error) {
 			viper.Set("group", args[0])
 			err = viper.WriteConfig()
 			if err != nil {
