@@ -37,29 +37,42 @@ import (
 var errNotExist = errors.New("the object doesn't exist")
 
 type discoveryService struct {
-	metadataRepo metadata.Repo
-	nodeRegistry NodeRegistry
-	groupRepo    *groupRepo
-	entityRepo   *entityRepo
-	log          *logger.Logger
-	kind         schema.Kind
+	metadataRepo    metadata.Repo
+	nodeRegistry    NodeRegistry
+	groupRepo       *groupRepo
+	entityRepo      *entityRepo
+	shardingKeyRepo *shardingKeyRepo
+	log             *logger.Logger
+	kind            schema.Kind
 }
 
-func newDiscoveryService(kind schema.Kind, metadataRepo metadata.Repo, nodeRegistry NodeRegistry) *discoveryService {
-	sr := &groupRepo{resourceOpts: make(map[string]*commonv1.ResourceOpts)}
-	er := &entityRepo{entitiesMap: make(map[identity]partition.EntityLocator)}
+func newDiscoveryService(kind schema.Kind, metadataRepo metadata.Repo, nodeRegistry NodeRegistry, gr *groupRepo) *discoveryService {
+	er := &entityRepo{
+		entitiesMap:     make(map[identity]partition.Locator),
+		measureMap:      make(map[identity]*databasev1.Measure),
+		traceMap:        make(map[identity]*databasev1.Trace),
+		traceIDIndexMap: make(map[identity]int),
+	}
+	return newDiscoveryServiceWithEntityRepo(kind, metadataRepo, nodeRegistry, gr, er)
+}
+
+func newDiscoveryServiceWithEntityRepo(kind schema.Kind, metadataRepo metadata.Repo, nodeRegistry NodeRegistry, gr *groupRepo, er *entityRepo) *discoveryService {
+	sr := &shardingKeyRepo{shardingKeysMap: make(map[identity]partition.Locator)}
 	return &discoveryService{
-		groupRepo:    sr,
-		entityRepo:   er,
-		kind:         kind,
-		metadataRepo: metadataRepo,
-		nodeRegistry: nodeRegistry,
+		groupRepo:       gr,
+		entityRepo:      er,
+		shardingKeyRepo: sr,
+		kind:            kind,
+		metadataRepo:    metadataRepo,
+		nodeRegistry:    nodeRegistry,
 	}
 }
 
 func (ds *discoveryService) initialize() error {
-	ds.metadataRepo.RegisterHandler("liaison", schema.KindGroup, ds.groupRepo)
 	ds.metadataRepo.RegisterHandler("liaison", ds.kind, ds.entityRepo)
+	if ds.kind == schema.KindMeasure {
+		ds.metadataRepo.RegisterHandler("liaison", ds.kind, ds.shardingKeyRepo)
+	}
 	return nil
 }
 
@@ -67,18 +80,32 @@ func (ds *discoveryService) SetLogger(log *logger.Logger) {
 	ds.log = log
 	ds.groupRepo.log = log
 	ds.entityRepo.log = log
+	ds.shardingKeyRepo.log = log
 }
 
-func (ds *discoveryService) navigate(metadata *commonv1.Metadata, tagFamilies []*modelv1.TagFamilyForWrite) (pbv1.Entity, pbv1.EntityValues, common.ShardID, error) {
+func (ds *discoveryService) navigate(metadata *commonv1.Metadata, tagFamilies []*modelv1.TagFamilyForWrite) (pbv1.EntityValues, common.ShardID, error) {
 	shardNum, existed := ds.groupRepo.shardNum(metadata.Group)
 	if !existed {
-		return nil, nil, common.ShardID(0), errors.Wrapf(errNotExist, "finding the shard num by: %v", metadata)
+		return nil, common.ShardID(0), errors.Wrapf(errNotExist, "finding the shard num by: %v", metadata)
 	}
-	locator, existed := ds.entityRepo.getLocator(getID(metadata))
+	id := getID(metadata)
+	entityLocator, existed := ds.entityRepo.getLocator(id)
 	if !existed {
-		return nil, nil, common.ShardID(0), errors.Wrapf(errNotExist, "finding the locator by: %v", metadata)
+		return nil, common.ShardID(0), errors.Wrapf(errNotExist, "finding the entity locator by: %v", metadata)
 	}
-	return locator.Locate(metadata.Name, tagFamilies, shardNum)
+	entityValues, shardID, err := entityLocator.Locate(metadata.Name, tagFamilies, shardNum)
+	if err != nil {
+		return nil, common.ShardID(0), err
+	}
+	shardingKeyLocator, existed := ds.shardingKeyRepo.getLocator(id)
+	if !existed {
+		return entityValues, shardID, nil
+	}
+	_, shardID, err = shardingKeyLocator.Locate(metadata.Name, tagFamilies, shardNum)
+	if err != nil {
+		return nil, common.ShardID(0), err
+	}
+	return entityValues, shardID, nil
 }
 
 type identity struct {
@@ -124,7 +151,7 @@ func (s *groupRepo) OnDelete(schemaMetadata schema.Metadata) {
 		return
 	}
 	if le := s.log.Debug(); le.Enabled() {
-		le.Stringer("id", group.Metadata).Msg("shard deleted")
+		le.Stringer("id", group.Metadata).Msg("shard deletedTime")
 	}
 	s.RWMutex.Lock()
 	defer s.RWMutex.Unlock()
@@ -141,14 +168,14 @@ func (s *groupRepo) shardNum(groupName string) (uint32, bool) {
 	return r.ShardNum, true
 }
 
-func (s *groupRepo) getNodeSelector(groupName string) (string, bool) {
+func (s *groupRepo) copies(groupName string) (uint32, bool) {
 	s.RWMutex.RLock()
 	defer s.RWMutex.RUnlock()
 	r, ok := s.resourceOpts[groupName]
 	if !ok {
-		return "", false
+		return 0, false
 	}
-	return r.DefaultNodeSelector, true
+	return r.Replicas + 1, true
 }
 
 func getID(metadata *commonv1.Metadata) identity {
@@ -162,30 +189,19 @@ var _ schema.EventHandler = (*entityRepo)(nil)
 
 type entityRepo struct {
 	schema.UnimplementedOnInitHandler
-	log         *logger.Logger
-	entitiesMap map[identity]partition.EntityLocator
+	log             *logger.Logger
+	entitiesMap     map[identity]partition.Locator
+	measureMap      map[identity]*databasev1.Measure
+	traceMap        map[identity]*databasev1.Trace
+	traceIDIndexMap map[identity]int // Cache trace ID tag index
 	sync.RWMutex
 }
 
 // OnAddOrUpdate implements schema.EventHandler.
 func (e *entityRepo) OnAddOrUpdate(schemaMetadata schema.Metadata) {
-	var el partition.EntityLocator
+	var l partition.Locator
 	var id identity
 	var modRevision int64
-	switch schemaMetadata.Kind {
-	case schema.KindMeasure:
-		measure := schemaMetadata.Spec.(*databasev1.Measure)
-		modRevision = measure.GetMetadata().GetModRevision()
-		el = partition.NewEntityLocator(measure.TagFamilies, measure.Entity, modRevision)
-		id = getID(measure.GetMetadata())
-	case schema.KindStream:
-		stream := schemaMetadata.Spec.(*databasev1.Stream)
-		modRevision = stream.GetMetadata().GetModRevision()
-		el = partition.NewEntityLocator(stream.TagFamilies, stream.Entity, modRevision)
-		id = getID(stream.GetMetadata())
-	default:
-		return
-	}
 	if le := e.log.Debug(); le.Enabled() {
 		var kind string
 		switch schemaMetadata.Kind {
@@ -193,6 +209,8 @@ func (e *entityRepo) OnAddOrUpdate(schemaMetadata schema.Metadata) {
 			kind = "measure"
 		case schema.KindStream:
 			kind = "stream"
+		case schema.KindTrace:
+			kind = "trace"
 		default:
 			kind = "unknown"
 		}
@@ -202,16 +220,47 @@ func (e *entityRepo) OnAddOrUpdate(schemaMetadata schema.Metadata) {
 			Str("kind", kind).
 			Msg("entity added or updated")
 	}
-	en := make([]partition.TagLocator, 0, len(el.TagLocators))
-	for _, l := range el.TagLocators {
-		en = append(en, partition.TagLocator{
-			FamilyOffset: l.FamilyOffset,
-			TagOffset:    l.TagOffset,
-		})
+	switch schemaMetadata.Kind {
+	case schema.KindMeasure:
+		measure := schemaMetadata.Spec.(*databasev1.Measure)
+		modRevision = measure.GetMetadata().GetModRevision()
+		l = partition.NewEntityLocator(measure.TagFamilies, measure.Entity, modRevision)
+		id = getID(measure.GetMetadata())
+	case schema.KindStream:
+		stream := schemaMetadata.Spec.(*databasev1.Stream)
+		modRevision = stream.GetMetadata().GetModRevision()
+		l = partition.NewEntityLocator(stream.TagFamilies, stream.Entity, modRevision)
+		id = getID(stream.GetMetadata())
+	case schema.KindTrace:
+		trace := schemaMetadata.Spec.(*databasev1.Trace)
+		id = getID(trace.GetMetadata())
+		// Pre-compute trace ID tag index
+		traceIDTagName := trace.GetTraceIdTagName()
+		traceIDIndex := -1
+		for i, tagSpec := range trace.GetTags() {
+			if tagSpec.GetName() == traceIDTagName {
+				traceIDIndex = i
+				break
+			}
+		}
+		e.RWMutex.Lock()
+		e.traceMap[id] = trace
+		e.traceIDIndexMap[id] = traceIDIndex
+		e.RWMutex.Unlock()
+		return
+	default:
+		return
 	}
+
 	e.RWMutex.Lock()
 	defer e.RWMutex.Unlock()
-	e.entitiesMap[id] = partition.EntityLocator{TagLocators: en, ModRevision: modRevision}
+	e.entitiesMap[id] = partition.Locator{TagLocators: l.TagLocators, ModRevision: modRevision}
+	if schemaMetadata.Kind == schema.KindMeasure {
+		measure := schemaMetadata.Spec.(*databasev1.Measure)
+		e.measureMap[id] = measure
+	} else {
+		delete(e.measureMap, id) // Ensure measure is not stored for streams
+	}
 }
 
 // OnDelete implements schema.EventHandler.
@@ -224,6 +273,9 @@ func (e *entityRepo) OnDelete(schemaMetadata schema.Metadata) {
 	case schema.KindStream:
 		stream := schemaMetadata.Spec.(*databasev1.Stream)
 		id = getID(stream.GetMetadata())
+	case schema.KindTrace:
+		trace := schemaMetadata.Spec.(*databasev1.Trace)
+		id = getID(trace.GetMetadata())
 	default:
 		return
 	}
@@ -234,6 +286,8 @@ func (e *entityRepo) OnDelete(schemaMetadata schema.Metadata) {
 			kind = "measure"
 		case schema.KindStream:
 			kind = "stream"
+		case schema.KindTrace:
+			kind = "trace"
 		default:
 			kind = "unknown"
 		}
@@ -241,19 +295,108 @@ func (e *entityRepo) OnDelete(schemaMetadata schema.Metadata) {
 			Str("action", "delete").
 			Stringer("subject", id).
 			Str("kind", kind).
-			Msg("entity deleted")
+			Msg("entity deletedTime")
 	}
 	e.RWMutex.Lock()
 	defer e.RWMutex.Unlock()
 	delete(e.entitiesMap, id)
+	delete(e.measureMap, id)      // Clean up measure
+	delete(e.traceMap, id)        // Clean up trace
+	delete(e.traceIDIndexMap, id) // Clean up trace ID index
 }
 
-func (e *entityRepo) getLocator(id identity) (partition.EntityLocator, bool) {
+func (e *entityRepo) getLocator(id identity) (partition.Locator, bool) {
 	e.RWMutex.RLock()
 	defer e.RWMutex.RUnlock()
 	el, ok := e.entitiesMap[id]
 	if !ok {
-		return partition.EntityLocator{}, false
+		return partition.Locator{}, false
 	}
 	return el, true
 }
+
+func (e *entityRepo) getTrace(id identity) (*databasev1.Trace, bool) {
+	e.RWMutex.RLock()
+	defer e.RWMutex.RUnlock()
+	trace, ok := e.traceMap[id]
+	if !ok {
+		return nil, false
+	}
+	return trace, true
+}
+
+func (e *entityRepo) getTraceIDIndex(id identity) (int, bool) {
+	e.RWMutex.RLock()
+	defer e.RWMutex.RUnlock()
+	index, ok := e.traceIDIndexMap[id]
+	if !ok {
+		return -1, false
+	}
+	return index, true
+}
+
+var _ schema.EventHandler = (*shardingKeyRepo)(nil)
+
+type shardingKeyRepo struct {
+	schema.UnimplementedOnInitHandler
+	log             *logger.Logger
+	shardingKeysMap map[identity]partition.Locator
+	sync.RWMutex
+}
+
+// OnAddOrUpdate implements schema.EventHandler.
+func (s *shardingKeyRepo) OnAddOrUpdate(schemaMetadata schema.Metadata) {
+	if schemaMetadata.Kind != schema.KindMeasure {
+		return
+	}
+	measure := schemaMetadata.Spec.(*databasev1.Measure)
+	shardingKey := measure.GetShardingKey()
+	if shardingKey == nil || len(shardingKey.GetTagNames()) == 0 {
+		return
+	}
+	l := partition.NewShardingKeyLocator(measure.TagFamilies, measure.ShardingKey)
+	id := getID(measure.GetMetadata())
+	if le := s.log.Debug(); le.Enabled() {
+		le.
+			Str("action", "add_or_update").
+			Stringer("subject", id).
+			Str("kind", "measure").
+			Msg("sharding key added or updated")
+	}
+	s.RWMutex.Lock()
+	defer s.RWMutex.Unlock()
+	s.shardingKeysMap[id] = partition.Locator{TagLocators: l.TagLocators}
+}
+
+// OnDelete implements schema.EventHandler.
+func (s *shardingKeyRepo) OnDelete(schemaMetadata schema.Metadata) {
+	if schemaMetadata.Kind != schema.KindMeasure {
+		return
+	}
+	measure := schemaMetadata.Spec.(*databasev1.Measure)
+	shardingKey := measure.GetShardingKey()
+	if shardingKey == nil || len(shardingKey.GetTagNames()) == 0 {
+		return
+	}
+	id := getID(measure.GetMetadata())
+	if le := s.log.Debug(); le.Enabled() {
+		le.
+			Str("action", "delete").
+			Stringer("subject", id).
+			Str("kind", "measure").
+			Msg("sharding key deletedTime")
+	}
+	s.RWMutex.Lock()
+	defer s.RWMutex.Unlock()
+	delete(s.shardingKeysMap, id)
+}
+
+func (s *shardingKeyRepo) getLocator(id identity) (partition.Locator, bool) {
+	s.RWMutex.RLock()
+	defer s.RWMutex.RUnlock()
+	sl, ok := s.shardingKeysMap[id]
+	if !ok {
+		return partition.Locator{}, false
+	}
+	return sl, true
+}
