@@ -19,16 +19,14 @@ package grpc_test
 
 import (
 	"testing"
-	"time"
 
 	. "github.com/onsi/ginkgo/v2"
 	. "github.com/onsi/gomega"
 
 	"github.com/apache/skywalking-banyandb/pkg/logger"
+	"github.com/apache/skywalking-banyandb/pkg/test/flags"
 )
 
-var defaultEventallyTimeout = 30 * time.Second
-
 func TestGrpc(t *testing.T) {
 	RegisterFailHandler(Fail)
 	RunSpecs(t, "Grpc Suite")
@@ -37,6 +35,6 @@ func TestGrpc(t *testing.T) {
 var _ = BeforeSuite(func() {
 	Expect(logger.Init(logger.Logging{
 		Env:   "dev",
-		Level: "warn",
+		Level: flags.LogLevel,
 	})).Should(Succeed())
 })
