@@ -75,6 +75,7 @@ func generateFlusherIntroduction() *flusherIntroduction {
 }
 
 func releaseFlusherIntroduction(i *flusherIntroduction) {
+	i.reset()
 	flusherIntroductionPool.Put(i)
 }
 
@@ -118,20 +119,60 @@ func (tst *tsTable) introducerLoop(flushCh chan *flusherIntroduction, mergeCh ch
 		case <-tst.loopCloser.CloseNotify():
 			return
 		case next := <-tst.introductions:
-			tst.incTotalIntroduceLoopStarted(1, "mem")
+			tst.incTotalIntroduceLoopStarted("mem")
 			tst.introduceMemPart(next, epoch)
-			tst.incTotalIntroduceLoopFinished(1, "mem")
+			tst.incTotalIntroduceLoopFinished("mem")
 			epoch++
 		case next := <-flushCh:
-			tst.incTotalIntroduceLoopStarted(1, "flush")
+			tst.incTotalIntroduceLoopStarted("flush")
 			tst.introduceFlushed(next, epoch)
-			tst.incTotalIntroduceLoopFinished(1, "flush")
+			tst.incTotalIntroduceLoopFinished("flush")
 			tst.gc.clean()
 			epoch++
 		case next := <-mergeCh:
-			tst.incTotalIntroduceLoopStarted(1, "merge")
+			tst.incTotalIntroduceLoopStarted("merge")
 			tst.introduceMerged(next, epoch)
-			tst.incTotalIntroduceLoopFinished(1, "merge")
+			tst.incTotalIntroduceLoopFinished("merge")
+			tst.gc.clean()
+			epoch++
+		case epochWatcher := <-watcherCh:
+			introducerWatchers.Add(epochWatcher)
+		}
+		curEpoch := tst.currentEpoch()
+		introducerWatchers.Notify(curEpoch)
+	}
+}
+
+func (tst *tsTable) introducerLoopWithSync(flushCh chan *flusherIntroduction, mergeCh chan *mergerIntroduction,
+	syncCh chan *syncIntroduction, watcherCh watcher.Channel, epoch uint64,
+) {
+	var introducerWatchers watcher.Epochs
+	defer tst.loopCloser.Done()
+	for {
+		select {
+		case <-tst.loopCloser.CloseNotify():
+			return
+		case next := <-tst.introductions:
+			tst.incTotalIntroduceLoopStarted("mem")
+			tst.introduceMemPart(next, epoch)
+			tst.incTotalIntroduceLoopFinished("mem")
+			epoch++
+		case next := <-flushCh:
+			tst.incTotalIntroduceLoopStarted("flush")
+			tst.introduceFlushed(next, epoch)
+			tst.incTotalIntroduceLoopFinished("flush")
+			tst.gc.clean()
+			epoch++
+		case next := <-mergeCh:
+			tst.incTotalIntroduceLoopStarted("merge")
+			tst.introduceMerged(next, epoch)
+			tst.incTotalIntroduceLoopFinished("merge")
+			tst.gc.clean()
+			epoch++
+		case next := <-syncCh:
+			tst.incTotalIntroduceLoopStarted("sync")
+			tst.introduceSync(next, epoch)
+			tst.incTotalIntroduceLoopFinished("sync")
 			tst.gc.clean()
 			epoch++
 		case epochWatcher := <-watcherCh:
@@ -190,6 +231,21 @@ func (tst *tsTable) introduceMerged(nextIntroduction *mergerIntroduction, epoch
 	}
 }
 
+func (tst *tsTable) introduceSync(nextIntroduction *syncIntroduction, epoch uint64) {
+	cur := tst.currentSnapshot()
+	if cur == nil {
+		tst.l.Panic().Msg("current snapshot is nil")
+		return
+	}
+	defer cur.decRef()
+	nextSnp := cur.remove(epoch, nextIntroduction.synced)
+	nextSnp.creator = snapshotCreatorSyncer
+	tst.replaceSnapshot(&nextSnp, true)
+	if nextIntroduction.applied != nil {
+		close(nextIntroduction.applied)
+	}
+}
+
 func (tst *tsTable) replaceSnapshot(next *snapshot, persisted bool) {
 	tst.Lock()
 	defer tst.Unlock()
