@@ -20,8 +20,8 @@ package measure
 import (
 	"context"
 	"fmt"
-	"io"
 	"path"
+	"strings"
 	"sync"
 	"time"
 
@@ -34,6 +34,8 @@ import (
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/api/validate"
 	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
+	"github.com/apache/skywalking-banyandb/banyand/internal/wqueue"
+	"github.com/apache/skywalking-banyandb/banyand/liaison/grpc"
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/banyand/observability"
@@ -41,6 +43,7 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/queue/pub"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
+	"github.com/apache/skywalking-banyandb/pkg/meter"
 	resourceSchema "github.com/apache/skywalking-banyandb/pkg/schema"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
@@ -64,7 +67,7 @@ var (
 		CompressionMethod: databasev1.CompressionMethod_COMPRESSION_METHOD_ZSTD,
 	}}
 	// TopNTagNames is the tag names of the topN result measure.
-	TopNTagNames = []string{"name", "direction", "group"}
+	TopNTagNames = []string{"name", "direction", "group", "source"}
 )
 
 // SchemaService allows querying schema information.
@@ -75,18 +78,23 @@ type SchemaService interface {
 type schemaRepo struct {
 	resourceSchema.Repository
 	metadata         metadata.Repo
-	pipeline         queue.Queue
+	pipeline         queue.Client
 	l                *logger.Logger
+	closingGroups    map[string]struct{}
 	topNProcessorMap sync.Map
+	nodeID           string
 	path             string
+	closingGroupsMu  sync.RWMutex
 }
 
-func newSchemaRepo(path string, svc *service, nodeLabels map[string]string) *schemaRepo {
+func newSchemaRepo(path string, svc *standalone, nodeLabels map[string]string, nodeID string) *schemaRepo {
 	sr := &schemaRepo{
-		path:     path,
-		l:        svc.l,
-		metadata: svc.metadata,
-		pipeline: svc.localPipeline,
+		path:          path,
+		l:             svc.l,
+		metadata:      svc.metadata,
+		pipeline:      svc.localPipeline,
+		nodeID:        nodeID,
+		closingGroups: make(map[string]struct{}),
 	}
 	sr.Repository = resourceSchema.NewRepository(
 		svc.metadata,
@@ -98,20 +106,47 @@ func newSchemaRepo(path string, svc *service, nodeLabels map[string]string) *sch
 	return sr
 }
 
-// NewPortableRepository creates a new portable repository.
-func NewPortableRepository(metadata metadata.Repo, l *logger.Logger, metrics *resourceSchema.Metrics) SchemaService {
-	r := &schemaRepo{
-		l:        l,
-		metadata: metadata,
-		Repository: resourceSchema.NewPortableRepository(
-			metadata,
-			l,
-			newPortableSupplier(metadata, l),
-			metrics,
-		),
+func newLiaisonSchemaRepo(path string, svc *liaison, measureDataNodeRegistry grpc.NodeRegistry, pipeline queue.Client) *schemaRepo {
+	sr := &schemaRepo{
+		path:          path,
+		l:             svc.l,
+		metadata:      svc.metadata,
+		pipeline:      pipeline,
+		closingGroups: make(map[string]struct{}),
 	}
-	r.start()
-	return r
+	sr.Repository = resourceSchema.NewRepository(
+		svc.metadata,
+		svc.l,
+		newQueueSupplier(path, svc, measureDataNodeRegistry),
+		resourceSchema.NewMetrics(svc.omr.With(metadataScope)),
+	)
+	sr.start()
+	return sr
+}
+
+// markGroupClosing marks a group as closing to prevent new processors from being created.
+func (sr *schemaRepo) markGroupClosing(group string) {
+	sr.closingGroupsMu.Lock()
+	if sr.closingGroups == nil {
+		sr.closingGroups = make(map[string]struct{})
+	}
+	sr.closingGroups[group] = struct{}{}
+	sr.closingGroupsMu.Unlock()
+}
+
+// unmarkGroupClosing removes the closing mark of a group.
+func (sr *schemaRepo) unmarkGroupClosing(group string) {
+	sr.closingGroupsMu.Lock()
+	delete(sr.closingGroups, group)
+	sr.closingGroupsMu.Unlock()
+}
+
+// isGroupClosing returns true if the group is currently being closed.
+func (sr *schemaRepo) isGroupClosing(group string) bool {
+	sr.closingGroupsMu.RLock()
+	_, ok := sr.closingGroups[group]
+	sr.closingGroupsMu.RUnlock()
+	return ok
 }
 
 func (sr *schemaRepo) start() {
@@ -160,7 +195,7 @@ func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
 		if g.Catalog != commonv1.Catalog_CATALOG_MEASURE {
 			return
 		}
-		if err := validate.GroupForStreamOrMeasure(g); err != nil {
+		if err := validate.GroupForNonProperty(g); err != nil {
 			sr.l.Warn().Err(err).Msg("group is ignored")
 			return
 		}
@@ -208,12 +243,26 @@ func (sr *schemaRepo) OnAddOrUpdate(metadata schema.Metadata) {
 			})
 		}
 	case schema.KindTopNAggregation:
+		if sr.pipeline == nil {
+			return
+		}
 		topNSchema := metadata.Spec.(*databasev1.TopNAggregation)
+		// Skip creating/registering processors if the group is closing
+		if sr.isGroupClosing(topNSchema.GetMetadata().GetGroup()) || (topNSchema.SourceMeasure != nil && sr.isGroupClosing(topNSchema.SourceMeasure.GetGroup())) {
+			sr.l.Debug().Str("group", topNSchema.GetMetadata().GetGroup()).
+				Str("topN", topNSchema.GetMetadata().GetName()).
+				Msg("skip TopNAggregation registration: group is closing")
+			return
+		}
 		if err := validate.TopNAggregation(topNSchema); err != nil {
 			sr.l.Warn().Err(err).Msg("topNAggregation is ignored")
 			return
 		}
 		manager := sr.getSteamingManager(topNSchema.SourceMeasure, sr.pipeline)
+		if manager == nil {
+			// group is closing; skip registering
+			return
+		}
 		manager.register(topNSchema)
 	default:
 	}
@@ -226,11 +275,16 @@ func (sr *schemaRepo) OnDelete(metadata schema.Metadata) {
 		if g.Catalog != commonv1.Catalog_CATALOG_MEASURE {
 			return
 		}
+		// Mark group as closing to prevent new processors from being created during deletion
+		sr.markGroupClosing(g.Metadata.Name)
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
 			Typ:      resourceSchema.EventDelete,
 			Kind:     resourceSchema.EventKindGroup,
 			Metadata: g,
 		})
+		sr.stopAllProcessorsWithGroupPrefix(g.Metadata.Name)
+		// Deletion completed; allow future re-creation
+		sr.unmarkGroupClosing(g.Metadata.Name)
 	case schema.KindMeasure:
 		m := metadata.Spec.(*databasev1.Measure)
 		sr.SendMetadataEvent(resourceSchema.MetadataEvent{
@@ -259,8 +313,10 @@ func (sr *schemaRepo) OnDelete(metadata schema.Metadata) {
 			})
 		}
 	case schema.KindTopNAggregation:
-		topNAggregation := metadata.Spec.(*databasev1.TopNAggregation)
-		sr.stopSteamingManager(topNAggregation.SourceMeasure)
+		if sr.pipeline != nil {
+			topNAggregation := metadata.Spec.(*databasev1.TopNAggregation)
+			sr.stopSteamingManager(topNAggregation.SourceMeasure)
+		}
 	default:
 	}
 }
@@ -302,6 +358,18 @@ func (sr *schemaRepo) loadTSDB(groupName string) (storage.TSDB[*tsTable, option]
 	return db.(storage.TSDB[*tsTable, option]), nil
 }
 
+func (sr *schemaRepo) loadQueue(groupName string) (*wqueue.Queue[*tsTable, option], error) {
+	g, ok := sr.LoadGroup(groupName)
+	if !ok {
+		return nil, fmt.Errorf("group %s not found", groupName)
+	}
+	db := g.SupplyTSDB()
+	if db == nil {
+		return nil, fmt.Errorf("queue for group %s not found", groupName)
+	}
+	return db.(*wqueue.Queue[*tsTable, option]), nil
+}
+
 func (sr *schemaRepo) createTopNResultMeasure(ctx context.Context, measureSchemaRegistry schema.Measure, group string) {
 	md := GetTopNSchemaMetadata(group)
 	operation := func() error {
@@ -323,7 +391,7 @@ func (sr *schemaRepo) createTopNResultMeasure(ctx context.Context, measureSchema
 	}
 
 	backoffStrategy := backoff.NewExponentialBackOff()
-	backoffStrategy.MaxElapsedTime = 2 * time.Minute
+	backoffStrategy.MaxElapsedTime = 0 // never stop until topN measure has been created
 
 	err := backoff.Retry(operation, backoffStrategy)
 	if err != nil {
@@ -331,25 +399,66 @@ func (sr *schemaRepo) createTopNResultMeasure(ctx context.Context, measureSchema
 	}
 }
 
+func (sr *schemaRepo) stopAllProcessorsWithGroupPrefix(groupName string) {
+	groupPrefix := groupName + "/"
+	totalClosed := 0
+	for {
+		closedThisRound := 0
+		sr.topNProcessorMap.Range(func(key, val any) bool {
+			keyStr := key.(string)
+			if strings.HasPrefix(keyStr, groupPrefix) {
+				manager := val.(*topNProcessorManager)
+				if err := manager.Close(); err != nil {
+					sr.l.Error().Err(err).Str("key", keyStr).Msg("failed to close topN processor manager")
+				} else {
+					sr.topNProcessorMap.Delete(keyStr)
+					closedThisRound++
+				}
+			}
+			return true
+		})
+		if closedThisRound == 0 {
+			break
+		}
+		totalClosed += closedThisRound
+		// continue to ensure no late-created managers remain
+	}
+	if totalClosed > 0 {
+		sr.l.Info().Str("groupName", groupName).Int("count", totalClosed).Msg("stopped topN processors for group")
+	}
+}
+
 var _ resourceSchema.ResourceSupplier = (*supplier)(nil)
 
 type supplier struct {
 	metadata   metadata.Repo
 	omr        observability.MetricsRegistry
+	c          storage.Cache
+	pm         protector.Memory
 	l          *logger.Logger
-	pm         *protector.Memory
 	schemaRepo *schemaRepo
 	nodeLabels map[string]string
 	path       string
 	option     option
 }
 
-func newSupplier(path string, svc *service, sr *schemaRepo, nodeLabels map[string]string) *supplier {
+func newSupplier(path string, svc *standalone, sr *schemaRepo, nodeLabels map[string]string) *supplier {
+	if svc.pm == nil {
+		svc.l.Panic().Msg("CRITICAL: svc.pm is nil in newSupplier")
+	}
+	opt := svc.option
+	opt.protector = svc.pm
+
+	if opt.protector == nil {
+		svc.l.Panic().Msg("CRITICAL: opt.protector is still nil after assignment")
+	}
+
 	return &supplier{
 		path:       path,
 		metadata:   svc.metadata,
 		l:          svc.l,
-		option:     svc.option,
+		c:          svc.c,
+		option:     opt,
 		omr:        svc.omr,
 		pm:         svc.pm,
 		schemaRepo: sr,
@@ -361,7 +470,7 @@ func (s *supplier) OpenResource(spec resourceSchema.Resource) (resourceSchema.In
 	measureSchema := spec.Schema().(*databasev1.Measure)
 	return openMeasure(measureSpec{
 		schema: measureSchema,
-	}, s.l, s.pm, s.schemaRepo)
+	}, s.l, s.c, s.pm, s.schemaRepo)
 }
 
 func (s *supplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.ResourceSchema, error) {
@@ -384,8 +493,8 @@ func (s *supplier) OpenDB(groupSchema *commonv1.Group) (resourceSchema.DB, error
 	shardNum := ro.ShardNum
 	ttl := ro.Ttl
 	segInterval := ro.SegmentInterval
+	segmentIdleTimeout := time.Duration(0)
 	if len(ro.Stages) > 0 && len(s.nodeLabels) > 0 {
-		matched := false
 		var ttlNum uint32
 		for _, st := range ro.Stages {
 			if st.Ttl.Unit != ro.Ttl.Unit {
@@ -399,18 +508,19 @@ func (s *supplier) OpenDB(groupSchema *commonv1.Group) (resourceSchema.DB, error
 			if !selector.Matches(s.nodeLabels) {
 				continue
 			}
-			matched = true
+			ttl.Num += ttlNum
 			shardNum = st.ShardNum
 			segInterval = st.SegmentInterval
+			if st.Close {
+				segmentIdleTimeout = 5 * time.Minute
+			}
 			break
 		}
-		if matched {
-			ttl.Num += ttlNum
-		}
 	}
+	group := groupSchema.Metadata.Name
 	opts := storage.TSDBOpts[*tsTable, option]{
 		ShardNum:                       shardNum,
-		Location:                       path.Join(s.path, groupSchema.Metadata.Name),
+		Location:                       path.Join(s.path, group),
 		TSTableCreator:                 newTSTable,
 		TableMetrics:                   metrics,
 		SegmentInterval:                storage.MustToIntervalRule(segInterval),
@@ -419,42 +529,146 @@ func (s *supplier) OpenDB(groupSchema *commonv1.Group) (resourceSchema.DB, error
 		SeriesIndexFlushTimeoutSeconds: s.option.flushTimeout.Nanoseconds() / int64(time.Second),
 		SeriesIndexCacheMaxBytes:       int(s.option.seriesCacheMaxSize),
 		StorageMetricsFactory:          factory,
-		SegmentBoundaryUpdateFn:        s.metadata.UpdateSegmentsBoundary,
+		SegmentIdleTimeout:             segmentIdleTimeout,
+		MemoryLimit:                    s.pm.GetLimit(),
 	}
 	return storage.OpenTSDB(
 		common.SetPosition(context.Background(), func(_ common.Position) common.Position {
 			return p
 		}),
-		opts)
+		opts, s.c, group)
 }
 
-type portableSupplier struct {
-	metadata metadata.Repo
-	l        *logger.Logger
+type queueSupplier struct {
+	metadata                metadata.Repo
+	omr                     observability.MetricsRegistry
+	pm                      protector.Memory
+	measureDataNodeRegistry grpc.NodeRegistry
+	l                       *logger.Logger
+	schemaRepo              *schemaRepo
+	path                    string
+	option                  option
 }
 
-func newPortableSupplier(metadata metadata.Repo, l *logger.Logger) *portableSupplier {
-	return &portableSupplier{
-		metadata: metadata,
-		l:        l,
+func newQueueSupplier(path string, svc *liaison, measureDataNodeRegistry grpc.NodeRegistry) *queueSupplier {
+	opt := svc.option
+	opt.protector = svc.pm
+
+	if opt.protector == nil {
+		svc.l.Panic().Msg("CRITICAL: opt.protector is still nil after assignment")
+	}
+	return &queueSupplier{
+		metadata:                svc.metadata,
+		omr:                     svc.omr,
+		pm:                      svc.pm,
+		measureDataNodeRegistry: measureDataNodeRegistry,
+		l:                       svc.l,
+		path:                    path,
+		option:                  opt,
 	}
 }
 
-func (s *portableSupplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.ResourceSchema, error) {
+func (s *queueSupplier) OpenResource(spec resourceSchema.Resource) (resourceSchema.IndexListener, error) {
+	measureSchema := spec.Schema().(*databasev1.Measure)
+	return openMeasure(measureSpec{
+		schema: measureSchema,
+	}, s.l, nil, s.pm, s.schemaRepo)
+}
+
+func (s *queueSupplier) ResourceSchema(md *commonv1.Metadata) (resourceSchema.ResourceSchema, error) {
 	ctx, cancel := context.WithTimeout(context.Background(), 5*time.Second)
 	defer cancel()
 	return s.metadata.MeasureRegistry().GetMeasure(ctx, md)
 }
 
-func (*portableSupplier) OpenDB(_ *commonv1.Group) (io.Closer, error) {
-	panic("do not support open db")
+func (s *queueSupplier) OpenDB(groupSchema *commonv1.Group) (resourceSchema.DB, error) {
+	name := groupSchema.Metadata.Name
+	p := common.Position{
+		Module:   "measure",
+		Database: name,
+	}
+	ro := groupSchema.ResourceOpts
+	if ro == nil {
+		return nil, fmt.Errorf("no resource opts in group %s", name)
+	}
+	shardNum := ro.ShardNum
+	group := groupSchema.Metadata.Name
+	opts := wqueue.Opts[*tsTable, option]{
+		Group:           group,
+		ShardNum:        shardNum,
+		SegmentInterval: storage.MustToIntervalRule(ro.SegmentInterval),
+		Location:        path.Join(s.path, group),
+		Option:          s.option,
+		Metrics:         s.newMetrics(p),
+		SubQueueCreator: newWriteQueue,
+		GetNodes: func(shardID common.ShardID) []string {
+			copies := ro.Replicas + 1
+			nodeSet := make(map[string]struct{}, copies)
+			for i := uint32(0); i < copies; i++ {
+				nodeID, err := s.measureDataNodeRegistry.Locate(group, "", uint32(shardID), i)
+				if err != nil {
+					s.l.Error().Err(err).Str("group", group).Uint32("shard", uint32(shardID)).Uint32("copy", i).Msg("failed to locate node")
+					return nil
+				}
+				nodeSet[nodeID] = struct{}{}
+			}
+			nodes := make([]string, 0, len(nodeSet))
+			for nodeID := range nodeSet {
+				nodes = append(nodes, nodeID)
+			}
+			return nodes
+		},
+	}
+	return wqueue.Open(
+		common.SetPosition(context.Background(), func(_ common.Position) common.Position {
+			return p
+		}),
+		opts, group,
+	)
 }
 
-func (s *portableSupplier) OpenResource(spec resourceSchema.Resource) (resourceSchema.IndexListener, error) {
-	measureSchema := spec.Schema().(*databasev1.Measure)
-	return openMeasure(measureSpec{
-		schema: measureSchema,
-	}, s.l, nil, nil)
+func (s *queueSupplier) newMetrics(p common.Position) storage.Metrics {
+	factory := s.omr.With(measureScope.ConstLabels(meter.ToLabelPairs(common.DBLabelNames(), p.DBLabelValues())))
+	return &metrics{
+		totalWritten:               factory.NewCounter("total_written"),
+		totalBatch:                 factory.NewCounter("total_batch"),
+		totalBatchIntroLatency:     factory.NewCounter("total_batch_intro_time"),
+		totalIntroduceLoopStarted:  factory.NewCounter("total_introduce_loop_started", "phase"),
+		totalIntroduceLoopFinished: factory.NewCounter("total_introduce_loop_finished", "phase"),
+		totalFlushLoopStarted:      factory.NewCounter("total_flush_loop_started"),
+		totalFlushLoopFinished:     factory.NewCounter("total_flush_loop_finished"),
+		totalFlushLoopErr:          factory.NewCounter("total_flush_loop_err"),
+		totalMergeLoopStarted:      factory.NewCounter("total_merge_loop_started"),
+		totalMergeLoopFinished:     factory.NewCounter("total_merge_loop_finished"),
+		totalMergeLoopErr:          factory.NewCounter("total_merge_loop_err"),
+		totalSyncLoopStarted:       factory.NewCounter("total_sync_loop_started"),
+		totalSyncLoopFinished:      factory.NewCounter("total_sync_loop_finished"),
+		totalSyncLoopErr:           factory.NewCounter("total_sync_loop_err"),
+		totalSyncLoopLatency:       factory.NewCounter("total_sync_loop_latency"),
+		totalSyncLoopBytes:         factory.NewCounter("total_sync_loop_bytes"),
+		totalFlushLoopProgress:     factory.NewCounter("total_flush_loop_progress"),
+		totalFlushed:               factory.NewCounter("total_flushed"),
+		totalFlushedMemParts:       factory.NewCounter("total_flushed_mem_parts"),
+		totalFlushPauseCompleted:   factory.NewCounter("total_flush_pause_completed"),
+		totalFlushPauseBreak:       factory.NewCounter("total_flush_pause_break"),
+		totalFlushIntroLatency:     factory.NewCounter("total_flush_intro_latency"),
+		totalFlushLatency:          factory.NewCounter("total_flush_latency"),
+		totalMergedParts:           factory.NewCounter("total_merged_parts", "type"),
+		totalMergeLatency:          factory.NewCounter("total_merge_latency", "type"),
+		totalMerged:                factory.NewCounter("total_merged", "type"),
+		tbMetrics: tbMetrics{
+			totalMemParts:                  factory.NewGauge("total_mem_part", common.ShardLabelNames()...),
+			totalMemElements:               factory.NewGauge("total_mem_elements", common.ShardLabelNames()...),
+			totalMemBlocks:                 factory.NewGauge("total_mem_blocks", common.ShardLabelNames()...),
+			totalMemPartBytes:              factory.NewGauge("total_mem_part_bytes", common.ShardLabelNames()...),
+			totalMemPartUncompressedBytes:  factory.NewGauge("total_mem_part_uncompressed_bytes", common.ShardLabelNames()...),
+			totalFileParts:                 factory.NewGauge("total_file_parts", common.ShardLabelNames()...),
+			totalFileElements:              factory.NewGauge("total_file_elements", common.ShardLabelNames()...),
+			totalFileBlocks:                factory.NewGauge("total_file_blocks", common.ShardLabelNames()...),
+			totalFilePartBytes:             factory.NewGauge("total_file_part_bytes", common.ShardLabelNames()...),
+			totalFilePartUncompressedBytes: factory.NewGauge("total_file_part_uncompressed_bytes", common.ShardLabelNames()...),
+		},
+	}
 }
 
 // GetTopNSchema returns the schema of the topN result measure.
@@ -468,6 +682,7 @@ func GetTopNSchema(md *commonv1.Metadata) *databasev1.Measure {
 					{Name: TopNTagNames[0], Type: databasev1.TagType_TAG_TYPE_STRING},
 					{Name: TopNTagNames[1], Type: databasev1.TagType_TAG_TYPE_INT},
 					{Name: TopNTagNames[2], Type: databasev1.TagType_TAG_TYPE_STRING},
+					{Name: TopNTagNames[3], Type: databasev1.TagType_TAG_TYPE_STRING},
 				},
 			},
 		},
