@@ -50,6 +50,12 @@ type metrics struct {
 	totalMergeLoopFinished meter.Counter
 	totalMergeLoopErr      meter.Counter
 
+	totalSyncLoopStarted  meter.Counter
+	totalSyncLoopFinished meter.Counter
+	totalSyncLoopErr      meter.Counter
+	totalSyncLoopLatency  meter.Counter
+	totalSyncLoopBytes    meter.Counter
+
 	totalFlushLoopProgress   meter.Counter
 	totalFlushed             meter.Counter
 	totalFlushedMemParts     meter.Counter
@@ -84,18 +90,18 @@ func (tst *tsTable) incTotalBatchIntroLatency(delta float64) {
 	tst.metrics.totalBatchIntroLatency.Inc(delta)
 }
 
-func (tst *tsTable) incTotalIntroduceLoopStarted(delta int, phase string) {
+func (tst *tsTable) incTotalIntroduceLoopStarted(phase string) {
 	if tst == nil || tst.metrics == nil {
 		return
 	}
-	tst.metrics.totalIntroduceLoopStarted.Inc(float64(delta), phase)
+	tst.metrics.totalIntroduceLoopStarted.Inc(1, phase)
 }
 
-func (tst *tsTable) incTotalIntroduceLoopFinished(delta int, phase string) {
+func (tst *tsTable) incTotalIntroduceLoopFinished(phase string) {
 	if tst == nil || tst.metrics == nil {
 		return
 	}
-	tst.metrics.totalIntroduceLoopFinished.Inc(float64(delta), phase)
+	tst.metrics.totalIntroduceLoopFinished.Inc(1, phase)
 }
 
 func (tst *tsTable) incTotalFlushLoopStarted(delta int) {
@@ -140,6 +146,41 @@ func (tst *tsTable) incTotalMergeLoopErr(delta int) {
 	tst.metrics.totalMergeLoopErr.Inc(float64(delta))
 }
 
+func (tst *tsTable) incTotalSyncLoopStarted(delta int) {
+	if tst == nil || tst.metrics == nil {
+		return
+	}
+	tst.metrics.totalSyncLoopStarted.Inc(float64(delta))
+}
+
+func (tst *tsTable) incTotalSyncLoopFinished(delta int) {
+	if tst == nil || tst.metrics == nil {
+		return
+	}
+	tst.metrics.totalSyncLoopFinished.Inc(float64(delta))
+}
+
+func (tst *tsTable) incTotalSyncLoopErr(delta int) {
+	if tst == nil || tst.metrics == nil {
+		return
+	}
+	tst.metrics.totalSyncLoopErr.Inc(float64(delta))
+}
+
+func (tst *tsTable) incTotalSyncLoopLatency(delta float64) {
+	if tst == nil || tst.metrics == nil {
+		return
+	}
+	tst.metrics.totalSyncLoopLatency.Inc(delta)
+}
+
+func (tst *tsTable) incTotalSyncLoopBytes(delta uint64) {
+	if tst == nil || tst.metrics == nil {
+		return
+	}
+	tst.metrics.totalSyncLoopBytes.Inc(float64(delta))
+}
+
 func (tst *tsTable) incTotalFlushLoopProgress(delta int) {
 	if tst == nil || tst.metrics == nil {
 		return
@@ -233,6 +274,12 @@ func (m *metrics) DeleteAll() {
 	m.totalMergeLoopFinished.Delete()
 	m.totalMergeLoopErr.Delete()
 
+	m.totalSyncLoopStarted.Delete()
+	m.totalSyncLoopFinished.Delete()
+	m.totalSyncLoopErr.Delete()
+	m.totalSyncLoopLatency.Delete()
+	m.totalSyncLoopBytes.Delete()
+
 	m.totalFlushLoopProgress.Delete()
 	m.totalFlushed.Delete()
 	m.totalFlushedMemParts.Delete()
@@ -262,6 +309,11 @@ func (s *supplier) newMetrics(p common.Position) (storage.Metrics, *observabilit
 		totalMergeLoopStarted:      factory.NewCounter("total_merge_loop_started"),
 		totalMergeLoopFinished:     factory.NewCounter("total_merge_loop_finished"),
 		totalMergeLoopErr:          factory.NewCounter("total_merge_loop_err"),
+		totalSyncLoopStarted:       factory.NewCounter("total_sync_loop_started"),
+		totalSyncLoopFinished:      factory.NewCounter("total_sync_loop_finished"),
+		totalSyncLoopErr:           factory.NewCounter("total_sync_loop_err"),
+		totalSyncLoopLatency:       factory.NewCounter("total_sync_loop_latency"),
+		totalSyncLoopBytes:         factory.NewCounter("total_sync_loop_bytes"),
 		totalFlushLoopProgress:     factory.NewCounter("total_flush_loop_progress"),
 		totalFlushed:               factory.NewCounter("total_flushed"),
 		totalFlushedMemParts:       factory.NewCounter("total_flushed_mem_parts"),
@@ -357,7 +409,7 @@ type tbMetrics struct {
 	totalFilePartUncompressedBytes meter.Gauge
 }
 
-func (s *service) createNativeObservabilityGroup(ctx context.Context) error {
+func (s *standalone) createNativeObservabilityGroup(ctx context.Context) error {
 	if !s.omr.NativeEnabled() {
 		return nil
 	}
@@ -384,3 +436,26 @@ func (s *service) createNativeObservabilityGroup(ctx context.Context) error {
 	}
 	return nil
 }
+
+type cacheMetrics struct {
+	requests meter.Gauge
+	misses   meter.Gauge
+	hitRatio meter.Gauge
+	entries  meter.Gauge
+	size     meter.Gauge
+}
+
+func newCacheMetrics(omr observability.MetricsRegistry) *cacheMetrics {
+	if omr == nil {
+		return nil
+	}
+
+	factory := omr.With(measureScope.SubScope("cache"))
+	return &cacheMetrics{
+		requests: factory.NewGauge("requests_total"),
+		misses:   factory.NewGauge("misses_total"),
+		hitRatio: factory.NewGauge("hit_ratio"),
+		entries:  factory.NewGauge("entries_count"),
+		size:     factory.NewGauge("size_bytes"),
+	}
+}
