@@ -24,6 +24,7 @@ import (
 	"sort"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/bytes"
 	"github.com/apache/skywalking-banyandb/pkg/compress/zstd"
 	"github.com/apache/skywalking-banyandb/pkg/encoding"
@@ -35,6 +36,7 @@ import (
 type partIter struct {
 	err                  error
 	p                    *part
+	c                    storage.Cache
 	curBlock             *blockMetadata
 	sids                 []common.SeriesID
 	primaryBlockMetadata []primaryBlockMetadata
@@ -49,6 +51,7 @@ type partIter struct {
 func (pi *partIter) reset() {
 	pi.curBlock = nil
 	pi.p = nil
+	pi.c = nil
 	pi.sids = nil
 	pi.sidIdx = 0
 	pi.primaryBlockMetadata = nil
@@ -58,12 +61,12 @@ func (pi *partIter) reset() {
 	pi.err = nil
 }
 
-func (pi *partIter) init(bma *blockMetadataArray, p *part, sids []common.SeriesID, minTimestamp, maxTimestamp int64) {
+func (pi *partIter) init(p *part, sids []common.SeriesID, minTimestamp, maxTimestamp int64) {
 	pi.reset()
 	pi.curBlock = &blockMetadata{}
 	pi.p = p
+	pi.c = p.cache
 
-	pi.bms = bma.arr
 	pi.sids = sids
 	pi.minTimestamp = minTimestamp
 	pi.maxTimestamp = maxTimestamp
@@ -147,8 +150,7 @@ func (pi *partIter) loadNextBlockMetadata() bool {
 			continue
 		}
 
-		var err error
-		pi.bms, err = pi.readPrimaryBlock(pi.bms[:0], pbm)
+		err := pi.readPrimaryBlock(pbm)
 		if err != nil {
 			pi.err = fmt.Errorf("cannot read primary block for part %q at offset %d with size %d: %w",
 				&pi.p.partMetadata, pbm.offset, pbm.size, err)
@@ -179,20 +181,30 @@ func searchPBM(pbmIndex []primaryBlockMetadata, sid common.SeriesID) []primaryBl
 	return pbmIndex[n-1:]
 }
 
-func (pi *partIter) readPrimaryBlock(bms []blockMetadata, mr *primaryBlockMetadata) ([]blockMetadata, error) {
+func (pi *partIter) readPrimaryBlock(mr *primaryBlockMetadata) error {
+	value := pi.c.Get(storage.NewEntryKey(pi.p.partMetadata.ID, mr.offset))
+	if value != nil {
+		bma := value.(*blockMetadataArray)
+		pi.bms = bma.arr
+		return nil
+	}
+
 	pi.compressedPrimaryBuf = bytes.ResizeOver(pi.compressedPrimaryBuf, int(mr.size))
 	fs.MustReadData(pi.p.primary, int64(mr.offset), pi.compressedPrimaryBuf)
 
 	var err error
 	pi.primaryBuf, err = zstd.Decompress(pi.primaryBuf[:0], pi.compressedPrimaryBuf)
 	if err != nil {
-		return nil, fmt.Errorf("cannot decompress index block: %w", err)
+		return fmt.Errorf("cannot decompress index block: %w", err)
 	}
-	bms, err = unmarshalBlockMetadata(bms, pi.primaryBuf)
+	bma := &blockMetadataArray{}
+	bma.arr, err = unmarshalBlockMetadata(bma.arr[:0], pi.primaryBuf)
 	if err != nil {
-		return nil, fmt.Errorf("cannot unmarshal index block: %w", err)
+		return fmt.Errorf("cannot unmarshal index block: %w", err)
 	}
-	return bms, nil
+	pi.c.Put(storage.NewEntryKey(pi.p.partMetadata.ID, mr.offset), bma)
+	pi.bms = bma.arr
+	return nil
 }
 
 func (pi *partIter) findBlock() bool {
@@ -228,7 +240,7 @@ func (pi *partIter) findBlock() bool {
 			continue
 		}
 
-		pi.curBlock = bm
+		pi.curBlock.copyFrom(bm)
 
 		pi.bms = bhs[1:]
 		return true
