@@ -26,6 +26,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/encoding"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
 	"github.com/apache/skywalking-banyandb/pkg/test"
@@ -109,14 +110,12 @@ func Test_partIter_nextBlock(t *testing.T) {
 		},
 	}
 
-	bma := generateBlockMetadataArray()
-	defer releaseBlockMetadataArray(bma)
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			verifyPart := func(p *part) {
 				defer p.close()
 				pi := partIter{}
-				pi.init(bma, p, tt.sids, tt.opt.minTimestamp, tt.opt.maxTimestamp)
+				pi.init(p, tt.sids, tt.opt.minTimestamp, tt.opt.maxTimestamp)
 
 				var got []blockMetadata
 				for pi.nextBlock() {
@@ -147,6 +146,8 @@ func Test_partIter_nextBlock(t *testing.T) {
 			mp.mustInitFromDataPoints(dps)
 
 			p := openMemPart(mp)
+			shardCache := storage.NewShardCache("test-group", 0, 0)
+			p.cache = shardCache
 			verifyPart(p)
 			tmpDir, defFn := test.Space(require.New(t))
 			defer defFn()
@@ -155,6 +156,7 @@ func Test_partIter_nextBlock(t *testing.T) {
 			fileSystem := fs.NewLocalFileSystem()
 			mp.mustFlush(fileSystem, partPath)
 			p = mustOpenFilePart(epoch, tmpDir, fileSystem)
+			p.cache = shardCache
 			verifyPart(p)
 		})
 	}
