@@ -70,7 +70,7 @@ type queryOptions struct {
 	maxTimestamp int64
 }
 
-func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr model.MeasureQueryResult, err error) {
+func (m *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr model.MeasureQueryResult, err error) {
 	if mqo.TimeRange == nil {
 		return nil, errors.New("invalid query options: timeRange are required")
 	}
@@ -78,24 +78,27 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 		return nil, errors.New("invalid query options: tagProjection or fieldProjection is required")
 	}
 	var tsdb storage.TSDB[*tsTable, option]
-	db := s.tsdb.Load()
+	db := m.tsdb.Load()
 	if db == nil {
-		tsdb, err = s.schemaRepo.loadTSDB(s.group)
+		tsdb, err = m.schemaRepo.loadTSDB(m.group)
 		if err != nil {
 			return nil, err
 		}
-		s.tsdb.Store(tsdb)
+		m.tsdb.Store(tsdb)
 	} else {
 		tsdb = db.(storage.TSDB[*tsTable, option])
 	}
 
-	segments := tsdb.SelectSegments(*mqo.TimeRange)
+	segments, err := tsdb.SelectSegments(*mqo.TimeRange)
+	if err != nil {
+		return nil, err
+	}
 	if len(segments) < 1 {
 		return nilResult, nil
 	}
 
-	if s.schema.IndexMode {
-		return s.buildIndexQueryResult(ctx, mqo, segments)
+	if m.schema.IndexMode {
+		return m.buildIndexQueryResult(ctx, mqo, segments)
 	}
 
 	if len(mqo.Entities) < 1 {
@@ -110,7 +113,7 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 		}
 	}
 
-	sids, tables, storedIndexValue, newTagProjection, err := s.searchSeriesList(ctx, series, mqo, segments)
+	sids, tables, caches, storedIndexValue, newTagProjection, err := m.searchSeriesList(ctx, series, mqo, segments)
 	if err != nil {
 		return nil, err
 	}
@@ -144,7 +147,7 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 		if s == nil {
 			continue
 		}
-		parts, n = s.getParts(parts, qo.minTimestamp, qo.maxTimestamp)
+		parts, n = s.getParts(parts, caches[i], qo.minTimestamp, qo.maxTimestamp)
 		if n < 1 {
 			s.decRef()
 			continue
@@ -152,7 +155,7 @@ func (s *measure) Query(ctx context.Context, mqo model.MeasureQueryOptions) (mqr
 		result.snapshots = append(result.snapshots, s)
 	}
 
-	if err = s.searchBlocks(ctx, &result, sids, parts, qo); err != nil {
+	if err = m.searchBlocks(ctx, &result, sids, parts, qo); err != nil {
 		return nil, err
 	}
 
@@ -181,22 +184,22 @@ type tagNameWithType struct {
 	typ       pbv1.ValueType
 }
 
-func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, mqo model.MeasureQueryOptions,
+func (m *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, mqo model.MeasureQueryOptions,
 	segments []storage.Segment[*tsTable, option],
-) (sl []common.SeriesID, tables []*tsTable, storedIndexValue map[common.SeriesID]map[string]*modelv1.TagValue,
+) (sl []common.SeriesID, tables []*tsTable, caches []storage.Cache, storedIndexValue map[common.SeriesID]map[string]*modelv1.TagValue,
 	newTagProjection []model.TagProjection, err error,
 ) {
 	var indexProjection []index.FieldKey
 	fieldToValueType := make(map[string]tagNameWithType)
 	var projectedEntityOffsets map[string]int
 	newTagProjection = make([]model.TagProjection, 0)
-	is := s.indexSchema.Load().(indexSchema)
+	is := m.indexSchema.Load().(indexSchema)
 	for _, tp := range mqo.TagProjection {
 		var tagProjection model.TagProjection
 	TAG:
 		for _, n := range tp.Names {
-			for i := range s.schema.GetEntity().GetTagNames() {
-				if n == s.schema.GetEntity().GetTagNames()[i] {
+			for i := range m.schema.GetEntity().GetTagNames() {
+				if n == m.schema.GetEntity().GetTagNames()[i] {
 					if projectedEntityOffsets == nil {
 						projectedEntityOffsets = make(map[string]int)
 					}
@@ -223,58 +226,156 @@ func (s *measure) searchSeriesList(ctx context.Context, series []*pbv1.Series, m
 			newTagProjection = append(newTagProjection, tagProjection)
 		}
 	}
+
+	// Collect all search results first
+	var segResults []*segResult
+	var needsSorting bool
 	seriesFilter := roaring.NewPostingList()
+
 	for i := range segments {
-		sd, _, err := segments[i].IndexDB().Search(ctx, series, storage.IndexSearchOpts{
+		sd, sortedValues, err := segments[i].IndexDB().Search(ctx, series, storage.IndexSearchOpts{
 			Query:       mqo.Query,
 			Order:       mqo.Order,
 			PreloadSize: preloadSize,
 			Projection:  indexProjection,
 		})
 		if err != nil {
-			return nil, nil, nil, nil, err
+			return nil, nil, nil, nil, nil, err
 		}
 		if len(sd.SeriesList) > 0 {
-			tables = append(tables, segments[i].Tables()...)
-		}
-		for j := range sd.SeriesList {
-			if seriesFilter.Contains(uint64(sd.SeriesList[j].ID)) {
-				continue
+			tt, cc := segments[i].Tables()
+			tables = append(tables, tt...)
+			caches = append(caches, cc...)
+
+			// Create segResult for this segment
+			sr := &segResult{
+				SeriesData:   sd,
+				sortedValues: sortedValues,
+				i:            0,
 			}
-			seriesFilter.Insert(uint64(sd.SeriesList[j].ID))
-			sl = append(sl, sd.SeriesList[j].ID)
-			if projectedEntityOffsets == nil && sd.Fields == nil {
-				continue
-			}
-			if storedIndexValue == nil {
-				storedIndexValue = make(map[common.SeriesID]map[string]*modelv1.TagValue)
-			}
-			tagValues := make(map[string]*modelv1.TagValue)
-			storedIndexValue[sd.SeriesList[j].ID] = tagValues
-			for name, offset := range projectedEntityOffsets {
-				if offset < 0 || offset >= len(sd.SeriesList[j].EntityValues) {
-					logger.Warningf("offset %d for tag %s is out of range for series ID %v", offset, name, sd.SeriesList[j].ID)
-					tagValues[name] = pbv1.NullTagValue
-					continue
-				}
-				tagValues[name] = sd.SeriesList[j].EntityValues[offset]
-			}
-			if sd.Fields == nil {
-				continue
-			}
-			for f, v := range sd.Fields[j] {
-				if tnt, ok := fieldToValueType[f]; ok {
-					tagValues[tnt.fieldName] = mustDecodeTagValue(tnt.typ, v)
-				} else {
-					logger.Panicf("unknown field %s not found in fieldToValueType", f)
-				}
+			segResults = append(segResults, sr)
+
+			// Check if we need sorting
+			if mqo.Order != nil && sortedValues != nil {
+				needsSorting = true
 			}
 		}
 	}
-	return sl, tables, storedIndexValue, newTagProjection, nil
+
+	// Sort if needed, otherwise use original order
+	if needsSorting && len(segResults) > 0 {
+		// Use segResultHeap to sort
+		segHeap := &segResultHeap{
+			results:  segResults,
+			sortDesc: mqo.Order.Sort == modelv1.Sort_SORT_DESC,
+		}
+		heap.Init(segHeap)
+
+		// Extract sorted series IDs
+		for segHeap.Len() > 0 {
+			top := heap.Pop(segHeap).(*segResult)
+			series := top.SeriesList[top.i]
+
+			if seriesFilter.Contains(uint64(series.ID)) {
+				// Move to next in this segment
+				top.i++
+				if top.i < len(top.SeriesList) {
+					heap.Push(segHeap, top)
+				}
+				continue
+			}
+
+			seriesFilter.Insert(uint64(series.ID))
+			sl = append(sl, series.ID)
+
+			// Build storedIndexValue for this series
+			var fieldResult map[string][]byte
+			if top.Fields != nil && top.i < len(top.Fields) {
+				fieldResult = top.Fields[top.i]
+			}
+			storedIndexValue = m.buildStoredIndexValue(
+				series.ID,
+				series.EntityValues,
+				fieldResult,
+				projectedEntityOffsets,
+				fieldToValueType,
+				storedIndexValue,
+			)
+
+			// Move to next in this segment
+			top.i++
+			if top.i < len(top.SeriesList) {
+				heap.Push(segHeap, top)
+			}
+		}
+	} else {
+		// Original logic when no sorting is needed
+		for _, sr := range segResults {
+			for j := range sr.SeriesList {
+				if seriesFilter.Contains(uint64(sr.SeriesList[j].ID)) {
+					continue
+				}
+				seriesFilter.Insert(uint64(sr.SeriesList[j].ID))
+				sl = append(sl, sr.SeriesList[j].ID)
+
+				var fieldResult map[string][]byte
+				if sr.Fields != nil && j < len(sr.Fields) {
+					fieldResult = sr.Fields[j]
+				}
+				storedIndexValue = m.buildStoredIndexValue(
+					sr.SeriesList[j].ID,
+					sr.SeriesList[j].EntityValues,
+					fieldResult,
+					projectedEntityOffsets,
+					fieldToValueType,
+					storedIndexValue,
+				)
+			}
+		}
+	}
+
+	return sl, tables, caches, storedIndexValue, newTagProjection, nil
 }
 
-func (s *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQueryOptions,
+func (m *measure) buildStoredIndexValue(
+	seriesID common.SeriesID,
+	entityValues []*modelv1.TagValue,
+	fieldResult map[string][]byte,
+	projectedEntityOffsets map[string]int,
+	fieldToValueType map[string]tagNameWithType,
+	storedIndexValue map[common.SeriesID]map[string]*modelv1.TagValue,
+) map[common.SeriesID]map[string]*modelv1.TagValue {
+	if projectedEntityOffsets == nil && fieldResult == nil {
+		return storedIndexValue
+	}
+
+	if storedIndexValue == nil {
+		storedIndexValue = make(map[common.SeriesID]map[string]*modelv1.TagValue)
+	}
+	tagValues := make(map[string]*modelv1.TagValue)
+	storedIndexValue[seriesID] = tagValues
+
+	for name, offset := range projectedEntityOffsets {
+		if offset < 0 || offset >= len(entityValues) {
+			logger.Warningf("offset %d for tag %s is out of range for series ID %v", offset, name, seriesID)
+			tagValues[name] = pbv1.NullTagValue
+			continue
+		}
+		tagValues[name] = entityValues[offset]
+	}
+
+	for f, v := range fieldResult {
+		if tnt, ok := fieldToValueType[f]; ok {
+			tagValues[tnt.fieldName] = mustDecodeTagValue(tnt.typ, v)
+		} else {
+			logger.Panicf("unknown field %s not found in fieldToValueType", f)
+		}
+	}
+
+	return storedIndexValue
+}
+
+func (m *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQueryOptions,
 	segments []storage.Segment[*tsTable, option],
 ) (model.MeasureQueryResult, error) {
 	defer func() {
@@ -282,7 +383,7 @@ func (s *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQu
 			segments[i].DecRef()
 		}
 	}()
-	is := s.indexSchema.Load().(indexSchema)
+	is := m.indexSchema.Load().(indexSchema)
 	r := &indexSortResult{}
 	var indexProjection []index.FieldKey
 	for _, tp := range mqo.TagProjection {
@@ -294,8 +395,8 @@ func (s *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQu
 	TAG:
 		for _, n := range tp.Names {
 			tagFamilyLocation.tagNames = append(tagFamilyLocation.tagNames, n)
-			for i := range s.schema.GetEntity().GetTagNames() {
-				if n == s.schema.GetEntity().GetTagNames()[i] {
+			for i := range m.schema.GetEntity().GetTagNames() {
+				if n == m.schema.GetEntity().GetTagNames()[i] {
 					tagFamilyLocation.projectedEntityOffsets[n] = i
 					continue TAG
 				}
@@ -346,18 +447,22 @@ func (s *measure) buildIndexQueryResult(ctx context.Context, mqo model.MeasureQu
 		if len(sr.SeriesList) < 1 {
 			continue
 		}
-		r.segResults = append(r.segResults, sr)
+		r.segResults.results = append(r.segResults.results, sr)
 	}
-	if len(r.segResults) < 1 {
+	if len(r.segResults.results) < 1 {
 		return nilResult, nil
 	}
+
+	// Set sort order based on mqo.Order.Sort
+	if mqo.Order != nil && mqo.Order.Sort == modelv1.Sort_SORT_DESC {
+		r.segResults.sortDesc = true
+	}
+
 	heap.Init(&r.segResults)
 	return r, nil
 }
 
-func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []common.SeriesID, parts []*part, qo queryOptions) error {
-	bma := generateBlockMetadataArray()
-	defer releaseBlockMetadataArray(bma)
+func (m *measure) searchBlocks(ctx context.Context, result *queryResult, sids []common.SeriesID, parts []*part, qo queryOptions) error {
 	defFn := startBlockScanSpan(ctx, len(sids), parts, result)
 	defer defFn()
 	tstIter := generateTstIter()
@@ -365,12 +470,13 @@ func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []
 	originalSids := make([]common.SeriesID, len(sids))
 	copy(originalSids, sids)
 	sort.Slice(sids, func(i, j int) bool { return sids[i] < sids[j] })
-	tstIter.init(bma, parts, sids, qo.minTimestamp, qo.maxTimestamp)
+	tstIter.init(parts, sids, qo.minTimestamp, qo.maxTimestamp)
 	if tstIter.Error() != nil {
 		return fmt.Errorf("cannot init tstIter: %w", tstIter.Error())
 	}
 	var hit int
 	var totalBlockBytes uint64
+	quota := m.pm.AvailableBytes()
 	for tstIter.nextBlock() {
 		if hit%checkDoneEvery == 0 {
 			select {
@@ -385,11 +491,14 @@ func (s *measure) searchBlocks(ctx context.Context, result *queryResult, sids []
 		bc.init(p.p, p.curBlock, qo)
 		result.data = append(result.data, bc)
 		totalBlockBytes += bc.bm.uncompressedSizeBytes
+		if quota >= 0 && totalBlockBytes > uint64(quota) {
+			return fmt.Errorf("block scan quota exceeded: used %d bytes, quota is %d bytes", totalBlockBytes, quota)
+		}
 	}
 	if tstIter.Error() != nil {
 		return fmt.Errorf("cannot iterate tstIter: %w", tstIter.Error())
 	}
-	if err := s.pm.AcquireResource(ctx, totalBlockBytes); err != nil {
+	if err := m.pm.AcquireResource(ctx, totalBlockBytes); err != nil {
 		return err
 	}
 	result.sidToIndex = make(map[common.SeriesID]int)
@@ -763,18 +872,18 @@ type indexSortResult struct {
 
 // Pull implements model.MeasureQueryResult.
 func (iqr *indexSortResult) Pull() *model.MeasureResult {
-	if len(iqr.segResults) < 1 {
+	if len(iqr.segResults.results) < 1 {
 		return nil
 	}
-	if len(iqr.segResults) == 1 {
-		if iqr.segResults[0].i >= len(iqr.segResults[0].SeriesList) {
+	if len(iqr.segResults.results) == 1 {
+		if iqr.segResults.results[0].i >= len(iqr.segResults.results[0].SeriesList) {
 			return nil
 		}
-		sr := iqr.segResults[0]
+		sr := iqr.segResults.results[0]
 		r := iqr.copyTo(sr)
 		sr.i++
 		if sr.i >= len(sr.SeriesList) {
-			iqr.segResults = iqr.segResults[:0]
+			iqr.segResults.results = iqr.segResults.results[:0]
 		}
 		return r
 	}
@@ -854,25 +963,56 @@ func (sr *segResult) remove(i int) {
 	}
 }
 
-type segResultHeap []*segResult
-
-func (h segResultHeap) Len() int { return len(h) }
-func (h segResultHeap) Less(i, j int) bool {
-	if h[i].sortedValues == nil {
-		return h[i].SeriesList[h[i].i].ID < h[j].SeriesList[h[j].i].ID
-	}
-	return bytes.Compare(h[i].sortedValues[h[i].i], h[j].sortedValues[h[j].i]) < 0
+type segResultHeap struct {
+	results  []*segResult
+	sortDesc bool
 }
-func (h segResultHeap) Swap(i, j int) { h[i], h[j] = h[j], h[i] }
+
+func (h *segResultHeap) Len() int { return len(h.results) }
+func (h *segResultHeap) Less(i, j int) bool {
+	// Handle NPE - check for nil results or invalid indices
+	if i >= len(h.results) || j >= len(h.results) {
+		return false
+	}
+	if h.results[i] == nil || h.results[j] == nil {
+		return false
+	}
+	if h.results[i].i >= len(h.results[i].SeriesList) || h.results[j].i >= len(h.results[j].SeriesList) {
+		return false
+	}
+
+	// If no sortedValues, compare by SeriesID
+	if h.results[i].sortedValues == nil || h.results[j].sortedValues == nil {
+		if h.sortDesc {
+			return h.results[i].SeriesList[h.results[i].i].ID > h.results[j].SeriesList[h.results[j].i].ID
+		}
+		return h.results[i].SeriesList[h.results[i].i].ID < h.results[j].SeriesList[h.results[j].i].ID
+	}
+
+	// Handle potential index out of bounds for sortedValues
+	if h.results[i].i >= len(h.results[i].sortedValues) || h.results[j].i >= len(h.results[j].sortedValues) {
+		if h.sortDesc {
+			return h.results[i].SeriesList[h.results[i].i].ID > h.results[j].SeriesList[h.results[j].i].ID
+		}
+		return h.results[i].SeriesList[h.results[i].i].ID < h.results[j].SeriesList[h.results[j].i].ID
+	}
+
+	cmp := bytes.Compare(h.results[i].sortedValues[h.results[i].i], h.results[j].sortedValues[h.results[j].i])
+	if h.sortDesc {
+		return cmp > 0
+	}
+	return cmp < 0
+}
+func (h *segResultHeap) Swap(i, j int) { h.results[i], h.results[j] = h.results[j], h.results[i] }
 
 func (h *segResultHeap) Push(x interface{}) {
-	*h = append(*h, x.(*segResult))
+	h.results = append(h.results, x.(*segResult))
 }
 
 func (h *segResultHeap) Pop() interface{} {
-	old := *h
+	old := h.results
 	n := len(old)
 	x := old[n-1]
-	*h = old[0 : n-1]
+	h.results = old[0 : n-1]
 	return x
 }
