@@ -30,7 +30,11 @@ import (
 
 	"github.com/apache/skywalking-banyandb/api/common"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
+	itest "github.com/apache/skywalking-banyandb/banyand/internal/test"
+	"github.com/apache/skywalking-banyandb/banyand/protector"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
+	"github.com/apache/skywalking-banyandb/pkg/index"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query/model"
@@ -1229,8 +1233,6 @@ func TestQueryResult(t *testing.T) {
 		},
 	}
 
-	bma := generateBlockMetadataArray()
-	defer releaseBlockMetadataArray(bma)
 	for _, tt := range tests {
 		t.Run(tt.name, func(t *testing.T) {
 			verify := func(t *testing.T, tst *tsTable) {
@@ -1242,14 +1244,15 @@ func TestQueryResult(t *testing.T) {
 				s := tst.currentSnapshot()
 				require.NotNil(t, s)
 				defer s.decRef()
-				pp, _ := s.getParts(nil, queryOpts.minTimestamp, queryOpts.maxTimestamp)
+				shardCache := storage.NewShardCache("test-group", 0, 0)
+				pp, _ := s.getParts(nil, shardCache, queryOpts.minTimestamp, queryOpts.maxTimestamp)
 				sids := make([]common.SeriesID, len(tt.sids))
 				copy(sids, tt.sids)
 				sort.Slice(sids, func(i, j int) bool {
 					return sids[i] < tt.sids[j]
 				})
 				ti := &tstIter{}
-				ti.init(bma, pp, sids, tt.minTimestamp, tt.maxTimestamp)
+				ti.init(pp, sids, tt.minTimestamp, tt.maxTimestamp)
 
 				var result queryResult
 				result.ctx = context.TODO()
@@ -1323,7 +1326,7 @@ func TestQueryResult(t *testing.T) {
 				fileSystem := fs.NewLocalFileSystem()
 				defer defFn()
 				tst, err := newTSTable(fileSystem, tmpPath, common.Position{},
-					logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: 0, mergePolicy: newDefaultMergePolicyForTesting()}, nil)
+					logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: 0, mergePolicy: newDefaultMergePolicyForTesting(), protector: protector.Nop{}}, nil)
 				require.NoError(t, err)
 				for _, dps := range tt.dpsList {
 					tst.mustAddDataPoints(dps)
@@ -1350,7 +1353,10 @@ func TestQueryResult(t *testing.T) {
 
 				// reopen the table
 				tst, err = newTSTable(fileSystem, tmpPath, common.Position{},
-					logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: defaultFlushTimeout, mergePolicy: newDefaultMergePolicyForTesting()}, nil)
+					logger.GetLogger("test"), timestamp.TimeRange{}, option{
+						flushTimeout: defaultFlushTimeout, mergePolicy: newDefaultMergePolicyForTesting(),
+						protector: protector.Nop{},
+					}, nil)
 				require.NoError(t, err)
 
 				verify(t, tst)
@@ -1358,3 +1364,494 @@ func TestQueryResult(t *testing.T) {
 		})
 	}
 }
+
+func TestQueryResult_QuotaExceeded(t *testing.T) {
+	tests := []struct {
+		wantErr             error
+		name                string
+		dpsList             []*dataPoints
+		sids                []common.SeriesID
+		want                []model.MeasureResult
+		minTimestamp        int64
+		maxTimestamp        int64
+		orderBySeries       bool
+		ascTS               bool
+		expectQuotaExceeded bool
+	}{
+		{
+			name:                "TestQuotaNotExceeded_ExpectSuccess",
+			dpsList:             []*dataPoints{dpsTS1, dpsTS1},
+			sids:                []common.SeriesID{1},
+			minTimestamp:        1,
+			maxTimestamp:        1,
+			expectQuotaExceeded: false,
+			want: []model.MeasureResult{{
+				SID:        1,
+				Timestamps: []int64{1},
+				Versions:   []int64{1},
+				TagFamilies: []model.TagFamily{
+					{Name: "arrTag", Tags: []model.Tag{
+						{Name: "strArrTag", Values: []*modelv1.TagValue{strArrTagValue([]string{"value1", "value2"})}},
+						{Name: "intArrTag", Values: []*modelv1.TagValue{int64ArrTagValue([]int64{25, 30})}},
+					}},
+					{Name: "binaryTag", Tags: []model.Tag{
+						{Name: "binaryTag", Values: []*modelv1.TagValue{binaryDataTagValue(longText)}},
+					}},
+					{Name: "singleTag", Tags: []model.Tag{
+						{Name: "strTag", Values: []*modelv1.TagValue{strTagValue("value1")}},
+						{Name: "intTag", Values: []*modelv1.TagValue{int64TagValue(10)}},
+						{Name: "strTag1", Values: []*modelv1.TagValue{pbv1.NullTagValue}},
+						{Name: "strTag2", Values: []*modelv1.TagValue{pbv1.NullTagValue}},
+					}},
+				},
+				Fields: []model.Field{
+					{Name: "strField", Values: []*modelv1.FieldValue{strFieldValue("field1")}},
+					{Name: "intField", Values: []*modelv1.FieldValue{int64FieldValue(1110)}},
+					{Name: "floatField", Values: []*modelv1.FieldValue{float64FieldValue(1.221233343e+06)}},
+					{Name: "binaryField", Values: []*modelv1.FieldValue{binaryDataFieldValue(longText)}},
+				},
+			}},
+		},
+		{
+			name:                "TestQuotaExceeded_ExpectError",
+			dpsList:             []*dataPoints{dpsTS1, dpsTS1},
+			sids:                []common.SeriesID{1},
+			minTimestamp:        1,
+			maxTimestamp:        1,
+			expectQuotaExceeded: true,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			// Initialize a tstIter object.
+			tmpPath, defFn := test.Space(require.New(t))
+			fileSystem := fs.NewLocalFileSystem()
+			defer defFn()
+			tst, err := newTSTable(fileSystem, tmpPath, common.Position{},
+				logger.GetLogger("test"), timestamp.TimeRange{}, option{flushTimeout: 0, mergePolicy: newDefaultMergePolicyForTesting(), protector: protector.Nop{}}, nil)
+			require.NoError(t, err)
+			for _, dps := range tt.dpsList {
+				tst.mustAddDataPoints(dps)
+				time.Sleep(100 * time.Millisecond)
+			}
+			// wait until the introducer is done
+			if len(tt.dpsList) > 0 {
+				for {
+					snp := tst.currentSnapshot()
+					if snp == nil {
+						time.Sleep(100 * time.Millisecond)
+						continue
+					}
+					if snp.creator == snapshotCreatorMemPart {
+						snp.decRef()
+						time.Sleep(100 * time.Millisecond)
+						continue
+					}
+					snp.decRef()
+					tst.Close()
+					break
+				}
+			}
+
+			// reopen the table
+			tst, err = newTSTable(fileSystem, tmpPath, common.Position{},
+				logger.GetLogger("test"), timestamp.TimeRange{}, option{
+					flushTimeout: defaultFlushTimeout, mergePolicy: newDefaultMergePolicyForTesting(),
+					protector: protector.Nop{},
+				}, nil)
+			require.NoError(t, err)
+
+			m := &measure{
+				pm: &itest.MockMemoryProtector{ExpectQuotaExceeded: tt.expectQuotaExceeded},
+			}
+			defer tst.Close()
+			queryOpts := queryOptions{
+				minTimestamp: tt.minTimestamp,
+				maxTimestamp: tt.maxTimestamp,
+			}
+			queryOpts.TagProjection = tagProjections[1]
+			queryOpts.FieldProjection = fieldProjections[1]
+			s := tst.currentSnapshot()
+			require.NotNil(t, s)
+			defer s.decRef()
+			shardCache := storage.NewShardCache("test-group", 0, 0)
+			pp, _ := s.getParts(nil, shardCache, queryOpts.minTimestamp, queryOpts.maxTimestamp)
+			var result queryResult
+			result.ctx = context.TODO()
+			// Query all tags
+			result.tagProjection = allTagProjections
+			err = m.searchBlocks(context.TODO(), &result, tt.sids, pp, queryOpts)
+			if tt.expectQuotaExceeded {
+				require.Error(t, err)
+				require.Contains(t, err.Error(), "quota exceeded", "expected quota to be exceeded but got: %v", err)
+				return
+			}
+			require.NoError(t, err)
+			defer result.Release()
+			if tt.orderBySeries {
+				result.sidToIndex = make(map[common.SeriesID]int)
+				for i, si := range tt.sids {
+					result.sidToIndex[si] = i
+				}
+			} else {
+				result.orderByTS = true
+				result.ascTS = tt.ascTS
+			}
+			var got []model.MeasureResult
+			for {
+				r := result.Pull()
+				if r == nil {
+					break
+				}
+				sort.Slice(r.TagFamilies, func(i, j int) bool {
+					return r.TagFamilies[i].Name < r.TagFamilies[j].Name
+				})
+				got = append(got, *r)
+			}
+			if diff := cmp.Diff(got, tt.want,
+				protocmp.IgnoreUnknown(), protocmp.Transform()); diff != "" {
+				t.Errorf("Unexpected []pbv1.Result (-got +want):\n%s", diff)
+			}
+		})
+	}
+}
+
+func TestSegResultHeap_Sorting(t *testing.T) {
+	tests := []struct {
+		name        string
+		segResults  []*segResult
+		expectOrder []int
+		sortDesc    bool
+	}{
+		{
+			name:     "Sort ascending by SeriesID (no sortedValues)",
+			sortDesc: false,
+			segResults: []*segResult{
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(30)},
+						},
+					},
+					i: 0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(10)},
+						},
+					},
+					i: 0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(20)},
+						},
+					},
+					i: 0,
+				},
+			},
+			expectOrder: []int{1, 0, 2}, // SeriesID order: 10, 30, 20
+		},
+		{
+			name:     "Sort descending by SeriesID (no sortedValues)",
+			sortDesc: true,
+			segResults: []*segResult{
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(10)},
+						},
+					},
+					i: 0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(30)},
+						},
+					},
+					i: 0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(20)},
+						},
+					},
+					i: 0,
+				},
+			},
+			expectOrder: []int{1, 0, 2}, // SeriesID order: 30, 10, 20
+		},
+		{
+			name:     "Sort ascending by sortedValues",
+			sortDesc: false,
+			segResults: []*segResult{
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(1)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("charlie")},
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(2)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("alpha")},
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(3)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("beta")},
+					i:            0,
+				},
+			},
+			expectOrder: []int{1, 0, 2}, // alpha, charlie, beta
+		},
+		{
+			name:     "Sort descending by sortedValues",
+			sortDesc: true,
+			segResults: []*segResult{
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(1)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("alpha")},
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(2)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("charlie")},
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(3)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("beta")},
+					i:            0,
+				},
+			},
+			expectOrder: []int{1, 0, 2}, // charlie, alpha, beta
+		},
+		{
+			name:     "Mixed sortedValues and nil sortedValues ascending",
+			sortDesc: false,
+			segResults: []*segResult{
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(30)},
+						},
+					},
+					sortedValues: nil, // Will use SeriesID for sorting
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(10)},
+						},
+					},
+					sortedValues: [][]byte{[]byte("zzz")}, // Should come after nil sortedValues when sorted by SeriesID
+					i:            0,
+				},
+				{
+					SeriesData: storage.SeriesData{
+						SeriesList: pbv1.SeriesList{
+							&pbv1.Series{ID: common.SeriesID(20)},
+						},
+					},
+					sortedValues: nil, // Will use SeriesID for sorting
+					i:            0,
+				},
+			},
+			expectOrder: []int{1, 0, 2}, // SeriesID 10, 30, 20 (nil sortedValues use SeriesID)
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			// Create and initialize heap
+			heap := segResultHeap{
+				results:  make([]*segResult, 0),
+				sortDesc: tt.sortDesc,
+			}
+
+			// Add all results to heap
+			heap.results = append(heap.results, tt.segResults...)
+
+			// Initialize heap
+			require.Equal(t, len(tt.segResults), heap.Len())
+
+			// Sort using Go's heap
+			heapImpl := &heap
+			heap2 := make([]*segResult, len(tt.segResults))
+			copy(heap2, heap.results)
+
+			// Sort manually to get expected order
+			sort.Slice(heap2, func(i, j int) bool {
+				return heapImpl.Less(i, j)
+			})
+
+			// Verify the order matches expectation
+			for i, expectedIdx := range tt.expectOrder {
+				actual := heap2[i]
+				expected := tt.segResults[expectedIdx]
+				require.Equal(t, expected.SeriesList[expected.i].ID, actual.SeriesList[actual.i].ID,
+					"Position %d: expected SeriesID %d, got %d", i, expected.SeriesList[expected.i].ID, actual.SeriesList[actual.i].ID)
+			}
+		})
+	}
+}
+
+func TestSegResultHeap_NPE_Prevention(t *testing.T) {
+	tests := []struct {
+		name       string
+		segResults []*segResult
+		i, j       int
+		expectLess bool
+	}{
+		{
+			name:       "Out of bounds indices",
+			segResults: []*segResult{{SeriesData: storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 1}}}, i: 0}},
+			i:          0,
+			j:          5, // Out of bounds
+			expectLess: false,
+		},
+		{
+			name:       "Nil segResult",
+			segResults: []*segResult{nil, {SeriesData: storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 1}}}, i: 0}},
+			i:          0,
+			j:          1,
+			expectLess: false,
+		},
+		{
+			name: "Index out of bounds for SeriesList",
+			segResults: []*segResult{
+				{SeriesData: storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 1}}}, i: 5}, // i is out of bounds
+				{SeriesData: storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 2}}}, i: 0},
+			},
+			i:          0,
+			j:          1,
+			expectLess: false,
+		},
+		{
+			name: "Index out of bounds for sortedValues",
+			segResults: []*segResult{
+				{
+					SeriesData:   storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 1}}},
+					sortedValues: [][]byte{[]byte("test")},
+					i:            5, // i is out of bounds for sortedValues
+				},
+				{
+					SeriesData:   storage.SeriesData{SeriesList: pbv1.SeriesList{{ID: 2}}},
+					sortedValues: [][]byte{[]byte("test2")},
+					i:            0,
+				},
+			},
+			i:          0,
+			j:          1,
+			expectLess: false, // Should fallback to SeriesID comparison
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			heap := segResultHeap{
+				results:  tt.segResults,
+				sortDesc: false,
+			}
+
+			// This should not panic due to NPE prevention
+			result := heap.Less(tt.i, tt.j)
+			require.Equal(t, tt.expectLess, result)
+		})
+	}
+}
+
+func TestIndexSortResult_OrderBySortDesc(t *testing.T) {
+	tests := []struct {
+		name       string
+		sortOrder  modelv1.Sort
+		expectDesc bool
+	}{
+		{
+			name:       "SORT_ASC should be ascending",
+			sortOrder:  modelv1.Sort_SORT_ASC,
+			expectDesc: false,
+		},
+		{
+			name:       "SORT_UNSPECIFIED should be ascending",
+			sortOrder:  modelv1.Sort_SORT_UNSPECIFIED,
+			expectDesc: false,
+		},
+		{
+			name:       "SORT_DESC should be descending",
+			sortOrder:  modelv1.Sort_SORT_DESC,
+			expectDesc: true,
+		},
+	}
+
+	for _, tt := range tests {
+		t.Run(tt.name, func(t *testing.T) {
+			// Create mock segments and measure data
+			mqo := model.MeasureQueryOptions{
+				Order: &index.OrderBy{
+					Sort: tt.sortOrder,
+				},
+			}
+
+			// Create a simple segResult
+			sr := &segResult{
+				SeriesData: storage.SeriesData{
+					SeriesList: pbv1.SeriesList{
+						&pbv1.Series{ID: common.SeriesID(1)},
+						&pbv1.Series{ID: common.SeriesID(2)},
+					},
+					Timestamps: []int64{1000, 2000},
+					Versions:   []int64{1, 2},
+				},
+				sortedValues: [][]byte{[]byte("test1"), []byte("test2")},
+				i:            0,
+			}
+
+			// Create index sort result
+			r := &indexSortResult{
+				tfl: []tagFamilyLocation{},
+				segResults: segResultHeap{
+					results:  []*segResult{sr},
+					sortDesc: false, // This should be set by buildIndexQueryResult
+				},
+			}
+
+			// Simulate the logic from buildIndexQueryResult
+			if mqo.Order != nil && mqo.Order.Sort == modelv1.Sort_SORT_DESC {
+				r.segResults.sortDesc = true
+			}
+
+			// Verify the sort order was set correctly
+			require.Equal(t, tt.expectDesc, r.segResults.sortDesc)
+		})
+	}
+}
