@@ -55,6 +55,7 @@ const (
 	snapshotCreatorFlusher
 	snapshotCreatorMerger
 	snapshotCreatorMergedFlusher
+	snapshotCreatorSyncer
 )
 
 type snapshot struct {
@@ -65,13 +66,14 @@ type snapshot struct {
 	ref int32
 }
 
-func (s *snapshot) getParts(dst []*part, minTimestamp, maxTimestamp int64) ([]*part, int) {
+func (s *snapshot) getParts(dst []*part, cache storage.Cache, minTimestamp, maxTimestamp int64) ([]*part, int) {
 	var count int
 	for _, p := range s.parts {
 		pm := p.p.partMetadata
 		if maxTimestamp < pm.MinTimestamp || minTimestamp > pm.MaxTimestamp {
 			continue
 		}
+		p.p.cache = cache
 		dst = append(dst, p.p)
 		count++
 	}
@@ -197,7 +199,7 @@ func (tst *tsTable) createMetadata(dst string, snapshot *snapshot) {
 	}
 }
 
-func (s *service) takeGroupSnapshot(dstDir string, groupName string) error {
+func (s *standalone) takeGroupSnapshot(dstDir string, groupName string) error {
 	group, ok := s.schemaRepo.LoadGroup(groupName)
 	if !ok {
 		return errors.Errorf("group %s not found", groupName)
@@ -215,7 +217,7 @@ func (s *service) takeGroupSnapshot(dstDir string, groupName string) error {
 
 type snapshotListener struct {
 	*bus.UnImplementedHealthyListener
-	s           *service
+	s           *standalone
 	snapshotSeq uint64
 	snapshotMux sync.Mutex
 }
