@@ -51,6 +51,7 @@ import (
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 	"github.com/apache/skywalking-banyandb/pkg/pool"
 	"github.com/apache/skywalking-banyandb/pkg/query/logical"
+	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 )
 
 const (
@@ -65,13 +66,31 @@ var (
 	_ flow.Sink = (*topNStreamingProcessor)(nil)
 )
 
-func (sr *schemaRepo) getSteamingManager(source *commonv1.Metadata, pipeline queue.Queue) (manager *topNProcessorManager) {
+func (sr *schemaRepo) inFlow(stm *databasev1.Measure, seriesID uint64, shardID uint32, entityValues []*modelv1.TagValue, dp *measurev1.DataPointValue) {
+	if p, _ := sr.topNProcessorMap.Load(getKey(stm.GetMetadata())); p != nil {
+		p.(*topNProcessorManager).onMeasureWrite(seriesID, shardID, &measurev1.InternalWriteRequest{
+			Request: &measurev1.WriteRequest{
+				Metadata:  stm.GetMetadata(),
+				DataPoint: dp,
+				MessageId: uint64(time.Now().UnixNano()),
+			},
+			EntityValues: entityValues,
+		}, stm)
+	}
+}
+
+func (sr *schemaRepo) getSteamingManager(source *commonv1.Metadata, pipeline queue.Client) (manager *topNProcessorManager) {
 	key := getKey(source)
+	// avoid creating a new manager if the source group is closing
+	if sr.isGroupClosing(source.GetGroup()) {
+		return nil
+	}
 	sourceMeasure, ok := sr.loadMeasure(source)
 	if !ok {
 		m, _ := sr.topNProcessorMap.LoadOrStore(key, &topNProcessorManager{
 			l:        sr.l,
 			pipeline: pipeline,
+			nodeID:   sr.nodeID,
 		})
 		manager = m.(*topNProcessorManager)
 		return manager
@@ -79,12 +98,17 @@ func (sr *schemaRepo) getSteamingManager(source *commonv1.Metadata, pipeline que
 
 	if v, ok := sr.topNProcessorMap.Load(key); ok {
 		pre := v.(*topNProcessorManager)
-		pre.init(sourceMeasure)
-		if pre.m.schema.GetMetadata().GetModRevision() < sourceMeasure.schema.GetMetadata().GetModRevision() {
+		pre.init(sourceMeasure.GetSchema())
+		var modRevision int64
+		pre.RLock()
+		modRevision = pre.m.GetMetadata().GetModRevision()
+		pre.RUnlock()
+		if modRevision < sourceMeasure.schema.GetMetadata().GetModRevision() {
 			defer pre.Close()
 			manager = &topNProcessorManager{
 				l:        sr.l,
 				pipeline: pipeline,
+				nodeID:   sr.nodeID,
 			}
 			manager.registeredTasks = append(manager.registeredTasks, pre.registeredTasks...)
 		} else {
@@ -95,9 +119,10 @@ func (sr *schemaRepo) getSteamingManager(source *commonv1.Metadata, pipeline que
 		manager = &topNProcessorManager{
 			l:        sr.l,
 			pipeline: pipeline,
+			nodeID:   sr.nodeID,
 		}
 	}
-	manager.init(sourceMeasure)
+	manager.init(sourceMeasure.GetSchema())
 	sr.topNProcessorMap.Store(key, manager)
 	return manager
 }
@@ -118,15 +143,16 @@ type dataPointWithEntityValues struct {
 }
 
 type topNStreamingProcessor struct {
-	pipeline      queue.Queue
+	pipeline      queue.Client
 	streamingFlow flow.Flow
 	in            chan flow.StreamRecord
 	l             *logger.Logger
 	topNSchema    *databasev1.TopNAggregation
 	src           chan interface{}
-	m             *measure
+	m             *databasev1.Measure
 	errCh         <-chan error
 	stopCh        chan struct{}
+	nodeID        string
 	flow.ComponentState
 	interval      time.Duration
 	sortDirection modelv1.Sort
@@ -204,7 +230,7 @@ func (t *topNStreamingProcessor) writeStreamRecord(record flow.StreamRecord, buf
 			}
 		}
 		topNValue.Reset()
-		topNValue.setMetadata(t.topNSchema.GetFieldName(), t.m.schema.Entity.TagNames)
+		topNValue.setMetadata(t.topNSchema.GetFieldName(), t.m.Entity.TagNames)
 		var shardID uint32
 		for _, tuple := range tuples {
 			data := tuple.V2.(flow.StreamRecord).Data().(flow.Data)
@@ -236,6 +262,13 @@ func (t *topNStreamingProcessor) writeStreamRecord(record flow.StreamRecord, buf
 					},
 				},
 			},
+			{
+				Value: &modelv1.TagValue_Str{
+					Str: &modelv1.Str{
+						Value: t.nodeID,
+					},
+				},
+			},
 		}
 		buf = buf[:0]
 		if buf, err = topNValue.marshal(buf); err != nil {
@@ -310,29 +343,36 @@ func (t *topNStreamingProcessor) handleError() {
 		t.l.Err(err).Str("topN", t.topNSchema.GetMetadata().GetName()).
 			Msg("error occurred during flow setup or process")
 	}
-	t.stopCh <- struct{}{}
+	// signal Close() that flow exited
+	close(t.stopCh)
 }
 
 // topNProcessorManager manages multiple topNStreamingProcessor(s) belonging to a single measure.
 type topNProcessorManager struct {
-	l               *logger.Logger
-	pipeline        queue.Queue
-	m               *measure
+	pipeline        queue.Client
 	s               logical.TagSpecRegistry
+	l               *logger.Logger
+	m               *databasev1.Measure
+	nodeID          string
 	registeredTasks []*databasev1.TopNAggregation
 	processorList   []*topNStreamingProcessor
 	sync.RWMutex
+	closed bool
 }
 
-func (manager *topNProcessorManager) init(m *measure) {
+func (manager *topNProcessorManager) init(m *databasev1.Measure) {
 	manager.Lock()
 	defer manager.Unlock()
+	if manager.closed {
+		return
+	}
 	if manager.m != nil {
 		return
 	}
 	manager.m = m
 	tagMapSpec := logical.TagSpecMap{}
-	tagMapSpec.RegisterTagFamilies(m.schema.GetTagFamilies())
+	tagMapSpec.RegisterTagFamilies(m.GetTagFamilies())
+	manager.s = tagMapSpec
 	for i := range manager.registeredTasks {
 		if err := manager.start(manager.registeredTasks[i]); err != nil {
 			manager.l.Err(err).Msg("fail to start processor")
@@ -343,19 +383,28 @@ func (manager *topNProcessorManager) init(m *measure) {
 func (manager *topNProcessorManager) Close() error {
 	manager.Lock()
 	defer manager.Unlock()
+	if manager.closed {
+		return nil
+	}
+	manager.closed = true
 	var err error
 	for _, processor := range manager.processorList {
 		err = multierr.Append(err, processor.Close())
 	}
 	manager.processorList = nil
 	manager.registeredTasks = nil
+	manager.s = nil
+	manager.m = nil
 	return err
 }
 
-func (manager *topNProcessorManager) onMeasureWrite(seriesID uint64, shardID uint32, request *measurev1.InternalWriteRequest, measure *measure) {
+func (manager *topNProcessorManager) onMeasureWrite(seriesID uint64, shardID uint32, request *measurev1.InternalWriteRequest, measure *databasev1.Measure) {
 	go func() {
 		manager.RLock()
 		defer manager.RUnlock()
+		if manager.closed {
+			return
+		}
 		if manager.m == nil {
 			manager.RUnlock()
 			manager.init(measure)
@@ -375,6 +424,9 @@ func (manager *topNProcessorManager) onMeasureWrite(seriesID uint64, shardID uin
 func (manager *topNProcessorManager) register(topNSchema *databasev1.TopNAggregation) {
 	manager.Lock()
 	defer manager.Unlock()
+	if manager.closed {
+		return
+	}
 	exist := false
 	for i := range manager.registeredTasks {
 		if manager.registeredTasks[i].GetMetadata().GetName() == topNSchema.GetMetadata().GetName() {
@@ -408,7 +460,10 @@ func (manager *topNProcessorManager) start(topNSchema *databasev1.TopNAggregatio
 	if manager.m == nil {
 		return nil
 	}
-	interval := manager.m.interval
+	interval, err := timestamp.ParseDuration(manager.m.Interval)
+	if err != nil {
+		return errors.Wrapf(err, "invalid interval %s for measure %s", manager.m.Interval, manager.m.GetMetadata().GetName())
+	}
 	sortDirections := make([]modelv1.Sort, 0, 2)
 	if topNSchema.GetFieldValueSort() == modelv1.Sort_SORT_UNSPECIFIED {
 		sortDirections = append(sortDirections, modelv1.Sort_SORT_ASC, modelv1.Sort_SORT_DESC)
@@ -445,6 +500,7 @@ func (manager *topNProcessorManager) start(topNSchema *databasev1.TopNAggregatio
 			stopCh:        make(chan struct{}),
 			streamingFlow: streamingFlow,
 			pipeline:      manager.pipeline,
+			nodeID:        manager.nodeID,
 		}
 		processorList[i] = processor.start()
 	}
@@ -454,12 +510,15 @@ func (manager *topNProcessorManager) start(topNSchema *databasev1.TopNAggregatio
 
 func (manager *topNProcessorManager) removeProcessors(topNSchema *databasev1.TopNAggregation) []*topNStreamingProcessor {
 	var processors []*topNStreamingProcessor
+	var newList []*topNStreamingProcessor
 	for i := range manager.processorList {
 		if manager.processorList[i].topNSchema.GetMetadata().GetName() == topNSchema.GetMetadata().GetName() {
 			processors = append(processors, manager.processorList[i])
-			manager.processorList = append(manager.processorList[:i], manager.processorList[i+1:]...)
+		} else {
+			newList = append(newList, manager.processorList[i])
 		}
 	}
+	manager.processorList = newList
 	return processors
 }
 
@@ -488,11 +547,11 @@ func (manager *topNProcessorManager) buildFilter(criteria *modelv1.Criteria) (fl
 }
 
 func (manager *topNProcessorManager) buildMapper(fieldName string, groupByNames ...string) (flow.UnaryFunc[any], error) {
-	fieldIdx := slices.IndexFunc(manager.m.GetSchema().GetFields(), func(spec *databasev1.FieldSpec) bool {
+	fieldIdx := slices.IndexFunc(manager.m.GetFields(), func(spec *databasev1.FieldSpec) bool {
 		return spec.GetName() == fieldName
 	})
 	if fieldIdx == -1 {
-		return nil, fmt.Errorf("field %s is not found in %s schema", fieldName, manager.m.GetSchema().GetMetadata().GetName())
+		return nil, fmt.Errorf("field %s is not found in %s schema", fieldName, manager.m.Metadata.GetName())
 	}
 	if len(groupByNames) == 0 {
 		return func(_ context.Context, request any) any {
@@ -518,7 +577,7 @@ func (manager *topNProcessorManager) buildMapper(fieldName string, groupByNames
 			}
 		}, nil
 	}
-	groupLocator, err := newGroupLocator(manager.m.GetSchema(), groupByNames)
+	groupLocator, err := newGroupLocator(manager.m, groupByNames)
 	if err != nil {
 		return nil, err
 	}
@@ -635,8 +694,10 @@ func (t *TopNValue) setMetadata(valueName string, entityTagNames []string) {
 }
 
 func (t *TopNValue) addValue(value int64, entityValues []*modelv1.TagValue) {
+	entityValuesCopy := make([]*modelv1.TagValue, len(entityValues))
+	copy(entityValuesCopy, entityValues)
 	t.values = append(t.values, value)
-	t.entities = append(t.entities, entityValues)
+	t.entities = append(t.entities, entityValuesCopy)
 }
 
 // Values returns the valueName, entityTagNames, values, and entities.
