@@ -28,6 +28,7 @@ import (
 	"github.com/stretchr/testify/require"
 
 	"github.com/apache/skywalking-banyandb/api/common"
+	"github.com/apache/skywalking-banyandb/banyand/internal/storage"
 	"github.com/apache/skywalking-banyandb/pkg/convert"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
@@ -137,8 +138,6 @@ func Test_tstIter(t *testing.T) {
 		minTimestamp int64
 		maxTimestamp int64
 	}
-	bma := generateBlockMetadataArray()
-	defer releaseBlockMetadataArray(bma)
 
 	verify := func(t *testing.T, tt testCtx, tst *tsTable) uint64 {
 		defer tst.Close()
@@ -147,10 +146,11 @@ func Test_tstIter(t *testing.T) {
 			s = new(snapshot)
 		}
 		defer s.decRef()
-		pp, n := s.getParts(nil, tt.minTimestamp, tt.maxTimestamp)
+		shardCache := storage.NewShardCache("test-group", 0, 0)
+		pp, n := s.getParts(nil, shardCache, tt.minTimestamp, tt.maxTimestamp)
 		require.Equal(t, len(s.parts), n)
 		ti := &tstIter{}
-		ti.init(bma, pp, tt.sids, tt.minTimestamp, tt.maxTimestamp)
+		ti.init(pp, tt.sids, tt.minTimestamp, tt.maxTimestamp)
 		var got []blockMetadata
 		for ti.nextBlock() {
 			if ti.piHeap[0].curBlock.seriesID == 0 {
@@ -192,8 +192,8 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 1,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1654},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 47},
 					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 				},
 			},
@@ -204,10 +204,10 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 2,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1654},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1654},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 47},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 47},
 					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 				},
@@ -231,10 +231,10 @@ func Test_tstIter(t *testing.T) {
 				minTimestamp: 1,
 				maxTimestamp: 2,
 				want: []blockMetadata{
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
-					{seriesID: 1, count: 1, uncompressedSizeBytes: 1684},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
-					{seriesID: 2, count: 1, uncompressedSizeBytes: 63},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1654},
+					{seriesID: 1, count: 1, uncompressedSizeBytes: 1654},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 47},
+					{seriesID: 2, count: 1, uncompressedSizeBytes: 47},
 					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 					{seriesID: 3, count: 1, uncompressedSizeBytes: 32},
 				},
@@ -523,8 +523,8 @@ var duplicatedDps1 = &dataPoints{
 	},
 }
 
-func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
-	hugeDps := &dataPoints{
+func generateHugeDatapoints(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
+	dataPoints := &dataPoints{
 		seriesIDs:   []common.SeriesID{},
 		timestamps:  []int64{},
 		versions:    []int64{},
@@ -533,10 +533,10 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 	}
 	now := time.Now().UnixNano()
 	for i := startTimestamp; i <= endTimestamp; i++ {
-		hugeDps.seriesIDs = append(hugeDps.seriesIDs, 1)
-		hugeDps.timestamps = append(hugeDps.timestamps, i)
-		hugeDps.versions = append(hugeDps.versions, now+i)
-		hugeDps.tagFamilies = append(hugeDps.tagFamilies, []nameValues{
+		dataPoints.seriesIDs = append(dataPoints.seriesIDs, 1)
+		dataPoints.timestamps = append(dataPoints.timestamps, i)
+		dataPoints.versions = append(dataPoints.versions, now+i)
+		dataPoints.tagFamilies = append(dataPoints.tagFamilies, []nameValues{
 			{
 				name: "arrTag", values: []*nameValue{
 					{name: "strArrTag", valueType: pbv1.ValueTypeStrArr, value: nil, valueArr: [][]byte{[]byte("value5"), []byte("value6")}},
@@ -555,7 +555,7 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 				},
 			},
 		})
-		hugeDps.fields = append(hugeDps.fields, nameValues{
+		dataPoints.fields = append(dataPoints.fields, nameValues{
 			name: "skipped", values: []*nameValue{
 				{name: "strField", valueType: pbv1.ValueTypeStr, value: []byte("field3"), valueArr: nil},
 				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(3330), valueArr: nil},
@@ -564,10 +564,10 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 			},
 		})
 	}
-	hugeDps.seriesIDs = append(hugeDps.seriesIDs, []common.SeriesID{2, 3}...)
-	hugeDps.timestamps = append(hugeDps.timestamps, []int64{timestamp, timestamp}...)
-	hugeDps.versions = append(hugeDps.versions, []int64{now + timestamp, now + timestamp}...)
-	hugeDps.tagFamilies = append(hugeDps.tagFamilies, [][]nameValues{{
+	dataPoints.seriesIDs = append(dataPoints.seriesIDs, []common.SeriesID{2, 3}...)
+	dataPoints.timestamps = append(dataPoints.timestamps, []int64{timestamp, timestamp}...)
+	dataPoints.versions = append(dataPoints.versions, []int64{now + timestamp, now + timestamp}...)
+	dataPoints.tagFamilies = append(dataPoints.tagFamilies, [][]nameValues{{
 		{
 			name: "singleTag", values: []*nameValue{
 				{name: "strTag1", valueType: pbv1.ValueTypeStr, value: []byte("tag3"), valueArr: nil},
@@ -575,16 +575,16 @@ func generateHugeDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints
 			},
 		},
 	}, {}}...)
-	hugeDps.fields = append(hugeDps.fields, []nameValues{{}, {
+	dataPoints.fields = append(dataPoints.fields, []nameValues{{}, {
 		name: "onlyFields", values: []*nameValue{
 			{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(4440), valueArr: nil},
 		},
 	}}...)
-	return hugeDps
+	return dataPoints
 }
 
-func generateHugeSmallDps(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
-	hugeDps := &dataPoints{
+func generateSmallDatapoints(startTimestamp, endTimestamp, timestamp int64) *dataPoints {
+	dataPoints := &dataPoints{
 		seriesIDs:   []common.SeriesID{},
 		timestamps:  []int64{},
 		versions:    []int64{},
@@ -593,24 +593,24 @@ func generateHugeSmallDps(startTimestamp, endTimestamp, timestamp int64) *dataPo
 	}
 	now := time.Now().UnixNano()
 	for i := startTimestamp; i <= endTimestamp; i++ {
-		hugeDps.seriesIDs = append(hugeDps.seriesIDs, 1)
-		hugeDps.timestamps = append(hugeDps.timestamps, i)
-		hugeDps.versions = append(hugeDps.versions, now+i)
-		hugeDps.tagFamilies = append(hugeDps.tagFamilies, []nameValues{})
-		hugeDps.fields = append(hugeDps.fields, nameValues{
+		dataPoints.seriesIDs = append(dataPoints.seriesIDs, 1)
+		dataPoints.timestamps = append(dataPoints.timestamps, i)
+		dataPoints.versions = append(dataPoints.versions, now+i)
+		dataPoints.tagFamilies = append(dataPoints.tagFamilies, []nameValues{})
+		dataPoints.fields = append(dataPoints.fields, nameValues{
 			name: "skipped", values: []*nameValue{
 				{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(3330), valueArr: nil},
 			},
 		})
 	}
-	hugeDps.seriesIDs = append(hugeDps.seriesIDs, []common.SeriesID{2, 3}...)
-	hugeDps.timestamps = append(hugeDps.timestamps, []int64{timestamp, timestamp}...)
-	hugeDps.versions = append(hugeDps.versions, []int64{now + timestamp, now + timestamp}...)
-	hugeDps.tagFamilies = append(hugeDps.tagFamilies, [][]nameValues{{}, {}}...)
-	hugeDps.fields = append(hugeDps.fields, []nameValues{{}, {
+	dataPoints.seriesIDs = append(dataPoints.seriesIDs, []common.SeriesID{2, 3}...)
+	dataPoints.timestamps = append(dataPoints.timestamps, []int64{timestamp, timestamp}...)
+	dataPoints.versions = append(dataPoints.versions, []int64{now + timestamp, now + timestamp}...)
+	dataPoints.tagFamilies = append(dataPoints.tagFamilies, [][]nameValues{{}, {}}...)
+	dataPoints.fields = append(dataPoints.fields, []nameValues{{}, {
 		name: "onlyFields", values: []*nameValue{
 			{name: "intField", valueType: pbv1.ValueTypeInt64, value: convert.Int64ToBytes(4440), valueArr: nil},
 		},
 	}}...)
-	return hugeDps
+	return dataPoints
 }
