@@ -63,13 +63,40 @@ func ConfigureListener(lcs, lps []string) Option {
 	}
 }
 
+// AutoCompactionMode sets the auto compaction mode.
+func AutoCompactionMode(mode string) Option {
+	return func(config *config) {
+		config.autoCompactionMode = mode
+	}
+}
+
+// AutoCompactionRetention sets the auto compaction retention.
+func AutoCompactionRetention(retention string) Option {
+	return func(config *config) {
+		config.autoCompactionRetention = retention
+	}
+}
+
+// QuotaBackendBytes sets the quota for backend storage.
+func QuotaBackendBytes(quota int64) Option {
+	return func(config *config) {
+		config.quotaBackendBytes = quota
+	}
+}
+
 type config struct {
 	// rootDir is the root directory for etcd storage
 	rootDir string
+	// autoCompactionMode is the auto compaction mode
+	autoCompactionMode string
+	// autoCompactionRetention is the auto compaction retention
+	autoCompactionRetention string
 	// listenerClientURLs is the listener for client
 	listenerClientURLs []string
 	// listenerPeerURLs is the listener for peer
 	listenerPeerURLs []string
+	// quotaBackendBytes is the quota for backend storage
+	quotaBackendBytes int64
 }
 
 func (e *server) ReadyNotify() <-chan struct{} {
@@ -93,9 +120,12 @@ func (e *server) Close() error {
 // NewServer returns a new etcd server.
 func NewServer(options ...Option) (Server, error) {
 	conf := &config{
-		rootDir:            os.TempDir(),
-		listenerClientURLs: []string{embed.DefaultListenClientURLs},
-		listenerPeerURLs:   []string{embed.DefaultListenPeerURLs},
+		rootDir:                 os.TempDir(),
+		listenerClientURLs:      []string{embed.DefaultListenClientURLs},
+		listenerPeerURLs:        []string{embed.DefaultListenPeerURLs},
+		autoCompactionMode:      "periodic",
+		autoCompactionRetention: "1h",
+		quotaBackendBytes:       2 * 1024 * 1024 * 1024,
 	}
 	for _, opt := range options {
 		opt(conf)
@@ -152,6 +182,9 @@ func newEmbedEtcdConfig(config *config, logger *zap.Logger) (*embed.Config, erro
 	cfg.ListenClientUrls, cfg.AdvertiseClientUrls = cURLs, cURLs
 	cfg.ListenPeerUrls, cfg.AdvertisePeerUrls = pURLs, pURLs
 	cfg.InitialCluster = cfg.InitialClusterFromName(cfg.Name)
+	cfg.AutoCompactionMode = config.autoCompactionMode
+	cfg.AutoCompactionRetention = config.autoCompactionRetention
+	cfg.QuotaBackendBytes = config.quotaBackendBytes
 
 	cfg.BackendBatchInterval = 500 * time.Millisecond
 	cfg.BackendBatchLimit = 10000
