@@ -27,15 +27,18 @@ import (
 var (
 	// ErrGRPCResourceNotFound indicates the resource doesn't exist.
 	ErrGRPCResourceNotFound = statusGRPCResourceNotFound.Err()
+	// ErrClosed indicates the registry is closed.
+	ErrClosed = errors.New("metadata registry is closed")
+	// ErrGRPCAlreadyExists indicates the resource already exists.
+	ErrGRPCAlreadyExists = statusGRPCAlreadyExists.Err()
+	// ErrInputInvalid indicates the input is invalid.
+	ErrInputInvalid = statusGRPCInvalidArgument.Err()
 
 	statusGRPCInvalidArgument  = status.New(codes.InvalidArgument, "banyandb: input is invalid")
 	statusGRPCResourceNotFound = status.New(codes.NotFound, "banyandb: resource not found")
 	statusGRPCAlreadyExists    = status.New(codes.AlreadyExists, "banyandb: resource already exists")
-	errGRPCAlreadyExists       = statusGRPCAlreadyExists.Err()
 	statusDataLoss             = status.New(codes.DataLoss, "banyandb: resource corrupts.")
 	errGRPCDataLoss            = statusDataLoss.Err()
-
-	errClosed = errors.New("metadata registry is closed")
 )
 
 // BadRequest creates a gRPC error with error details with type BadRequest,
@@ -50,7 +53,3 @@ func BadRequest(field, desc string) error {
 	st, _ := statusGRPCInvalidArgument.WithDetails(br)
 	return st.Err()
 }
-
-func isNotFound(err error) bool {
-	return errors.Is(err, ErrGRPCResourceNotFound)
-}
