@@ -66,7 +66,7 @@ func (e *etcdSchemaRegistry) UpdateStream(ctx context.Context, stream *databasev
 	if err != nil {
 		return 0, err
 	}
-	if err = validate.GroupForStreamOrMeasure(g); err != nil {
+	if err = validate.GroupForNonProperty(g); err != nil {
 		return 0, err
 	}
 	prev, err := e.GetStream(ctx, stream.GetMetadata())
@@ -125,7 +125,7 @@ func (e *etcdSchemaRegistry) CreateStream(ctx context.Context, stream *databasev
 	if err != nil {
 		return 0, err
 	}
-	if err := validate.GroupForStreamOrMeasure(g); err != nil {
+	if err := validate.GroupForNonProperty(g); err != nil {
 		return 0, err
 	}
 	return e.create(ctx, Metadata{
