@@ -47,19 +47,27 @@ var (
 	// RootScope is the root scope for all metrics.
 	RootScope = meter.NewHierarchicalScope("banyandb", "_")
 	// SystemScope is the system scope for all metrics.
-	SystemScope     = RootScope.SubScope("system")
-	cpuStateGauge   meter.Gauge
-	cpuNumGauge     meter.Gauge
-	memorySateGauge meter.Gauge
-	netStateGauge   meter.Gauge
-	upTimeGauge     meter.Gauge
-	diskStateGauge  meter.Gauge
-	diskMap         = sync.Map{}
+	SystemScope      = RootScope.SubScope("system")
+	cpuStateGauge    meter.Gauge
+	cpuNumGauge      meter.Gauge
+	memoryStateGauge meter.Gauge
+	netStateGauge    meter.Gauge
+	upTimeGauge      meter.Gauge
+	diskStateGauge   meter.Gauge
+	diskMap          = sync.Map{}
 )
 
 // UpdatePath updates a path to monitoring its disk usage.
 func UpdatePath(path string) {
-	diskMap.Store(path, nil)
+	diskMap.Store(path, 0)
+}
+
+// GetPathUsedPercent returns the used percent of the path.
+func GetPathUsedPercent(path string) int {
+	if v, ok := diskMap.Load(path); ok {
+		return v.(int)
+	}
+	return 0
 }
 
 func getPath() (paths []string) {
@@ -82,7 +90,7 @@ func (p *metricService) initMetrics() {
 	factory := p.With(SystemScope)
 	cpuStateGauge = factory.NewGauge("cpu_state", "kind")
 	cpuNumGauge = factory.NewGauge("cpu_num")
-	memorySateGauge = factory.NewGauge("memory_state", "kind")
+	memoryStateGauge = factory.NewGauge("memory_state", "kind")
 	netStateGauge = factory.NewGauge("net_state", "kind", "name")
 	upTimeGauge = factory.NewGauge("up_time")
 	diskStateGauge = factory.NewGauge("disk", "path", "kind")
@@ -100,9 +108,11 @@ func collectCPU() {
 	s, err := cpuTimesFunc(false)
 	if err != nil {
 		log.Error().Err(err).Msg("cannot get cpu stat")
+		return
 	}
 	if len(s) == 0 {
 		log.Error().Msg("cannot get cpu stat")
+		return
 	}
 	allStat := s[0]
 	total := allStat.User + allStat.System + allStat.Idle + allStat.Nice + allStat.Iowait + allStat.Irq +
@@ -118,25 +128,29 @@ func collectCPU() {
 }
 
 func collectMemory() {
-	if memorySateGauge == nil {
-		log.Error().Msg("memorySateGauge is not registered")
+	if memoryStateGauge == nil {
+		log.Error().Msg("memoryStateGauge is not registered")
+		return
 	}
 	m, err := mem.VirtualMemory()
 	if err != nil {
 		log.Error().Err(err).Msg("cannot get memory stat")
+		return
 	}
-	memorySateGauge.Set(m.UsedPercent/100, "used_percent")
-	memorySateGauge.Set(float64(m.Used), "used")
-	memorySateGauge.Set(float64(m.Total), "total")
+	memoryStateGauge.Set(m.UsedPercent/100, "used_percent")
+	memoryStateGauge.Set(float64(m.Used), "used")
+	memoryStateGauge.Set(float64(m.Total), "total")
 }
 
 func collectNet() {
 	if netStateGauge == nil {
 		log.Error().Msg("netStateGauge is not registered")
+		return
 	}
 	stats, err := getNetStat(context.Background())
 	if err != nil {
 		log.Error().Err(err).Msg("cannot get net stat")
+		return
 	}
 	for _, stat := range stats {
 		netStateGauge.Set(float64(stat.BytesRecv), "bytes_recv", stat.Name)
@@ -191,6 +205,7 @@ func collectDisk() {
 			}
 			return
 		}
+		diskMap.Store(path, int(usage.UsedPercent))
 		diskStateGauge.Set(usage.UsedPercent/100, path, "used_percent")
 		diskStateGauge.Set(float64(usage.Used), path, "used")
 		diskStateGauge.Set(float64(usage.Total), path, "total")
