@@ -18,6 +18,25 @@
 // Package observability provides metrics, profiling, and etc.
 package observability
 
-import "errors"
+import (
+	"errors"
 
-var errNoAddr = errors.New("no address")
+	"github.com/apache/skywalking-banyandb/pkg/meter"
+	"github.com/apache/skywalking-banyandb/pkg/run"
+)
+
+var (
+	errNoAddr         = errors.New("no address")
+	errNoMode         = errors.New("no observability mode")
+	errInvalidMode    = errors.New("invalid observability mode")
+	errDuplicatedMode = errors.New("duplicated observability mode")
+)
+
+// MetricsRegistry is the interface for metrics registry.
+type MetricsRegistry interface {
+	run.Service
+	// With returns a factory with the given scope.
+	With(scope meter.Scope) *Factory
+	// NativeEnabled returns whether the native mode is enabled.
+	NativeEnabled() bool
+}
