@@ -18,11 +18,30 @@
 // Package property provides the property service interface.
 package property
 
-import "github.com/apache/skywalking-banyandb/pkg/run"
+import (
+	"strconv"
+	"strings"
+
+	propertyv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/property/v1"
+	"github.com/apache/skywalking-banyandb/pkg/convert"
+	"github.com/apache/skywalking-banyandb/pkg/run"
+)
 
 // Service is the interface for property service.
 type Service interface {
 	run.PreRunner
 	run.Config
 	run.Service
+
+	GetGossIPGrpcPort() *uint32
+}
+
+// GetPropertyID returns the property ID based on the property metadata and revision.
+func GetPropertyID(prop *propertyv1.Property) []byte {
+	return convert.StringToBytes(GetEntity(prop) + "/" + strconv.FormatInt(prop.Metadata.ModRevision, 10))
+}
+
+// GetEntity returns the entity string for the property.
+func GetEntity(prop *propertyv1.Property) string {
+	return strings.Join([]string{prop.Metadata.Group, prop.Metadata.Name, prop.Id}, "/")
 }
