@@ -21,27 +21,8 @@ import (
 	"strings"
 )
 
-func Format(p Plan) string {
-	return formatWithIndent(p, 0)
-}
-
-func formatWithIndent(p Plan, indent int) string {
-	res := ""
-	if indent > 1 {
-		res += strings.Repeat(" ", 5*(indent-1))
-	}
-	if indent > 0 {
-		res += "+"
-		res += strings.Repeat("-", 4)
-	}
-	res += p.String() + "\n"
-	for _, child := range p.Children() {
-		res += formatWithIndent(child, indent+1)
-	}
-	return res
-}
-
-func formatTagRefs(sep string, exprGroup ...[]*TagRef) string {
+// FormatTagRefs outputs formatted tag refs.
+func FormatTagRefs(sep string, exprGroup ...[]*TagRef) string {
 	var exprsStr []string
 	for _, exprs := range exprGroup {
 		for i := 0; i < len(exprs); i++ {
