@@ -19,6 +19,8 @@ package logical
 
 import (
 	"fmt"
+
+	"github.com/apache/skywalking-banyandb/pkg/index"
 )
 
 // UnresolvedPlan denotes an logical expression.
@@ -37,7 +39,7 @@ type Plan interface {
 // Expr represents a predicate in criteria.
 type Expr interface {
 	fmt.Stringer
-	DataType() int32
+	Elements() []string
 	Equal(Expr) bool
 }
 
@@ -45,6 +47,9 @@ type Expr interface {
 type LiteralExpr interface {
 	Expr
 	Bytes() [][]byte
+	Field(key index.FieldKey) index.Field
+	RangeOpts(isUpper bool, includeLower, includeUpper bool) index.RangeOpts
+	SubExprs() []LiteralExpr
 }
 
 // ComparableExpr allows comparing Expr and Expr arrays.
