@@ -18,6 +18,7 @@
 package measure
 
 import (
+	"context"
 	"fmt"
 
 	"github.com/pkg/errors"
@@ -121,7 +122,7 @@ func (g *aggregationPlan[N]) Schema() logical.Schema {
 	return g.schema.ProjFields(g.aggregationFieldRef)
 }
 
-func (g *aggregationPlan[N]) Execute(ec executor.MeasureExecutionContext) (executor.MIterator, error) {
+func (g *aggregationPlan[N]) Execute(ec context.Context) (executor.MIterator, error) {
 	iter, err := g.Parent.Input.(executor.MeasureExecutable).Execute(ec)
 	if err != nil {
 		return nil, err
@@ -228,7 +229,6 @@ func (ami *aggAllIterator[N]) Next() bool {
 	if ami.result != nil || ami.err != nil {
 		return false
 	}
-	defer ami.prev.Close()
 	var resultDp *measurev1.DataPoint
 	for ami.prev.Next() {
 		group := ami.prev.Current()
@@ -275,5 +275,5 @@ func (ami *aggAllIterator[N]) Current() []*measurev1.DataPoint {
 }
 
 func (ami *aggAllIterator[N]) Close() error {
-	return nil
+	return ami.prev.Close()
 }
