@@ -18,11 +18,13 @@
 package measure
 
 import (
+	"context"
 	"fmt"
 	"math"
 
-	"github.com/cespare/xxhash"
+	"github.com/cespare/xxhash/v2"
 	"github.com/pkg/errors"
+	"go.uber.org/multierr"
 
 	measurev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/measure/v1"
 	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
@@ -100,14 +102,14 @@ func (g *groupBy) Schema() logical.Schema {
 	return g.schema.ProjTags(g.groupByTagsRefs...)
 }
 
-func (g *groupBy) Execute(ec executor.MeasureExecutionContext) (executor.MIterator, error) {
+func (g *groupBy) Execute(ec context.Context) (executor.MIterator, error) {
 	if g.groupByEntity {
 		return g.sort(ec)
 	}
 	return g.hash(ec)
 }
 
-func (g *groupBy) sort(ec executor.MeasureExecutionContext) (executor.MIterator, error) {
+func (g *groupBy) sort(ec context.Context) (executor.MIterator, error) {
 	iter, err := g.Parent.Input.(executor.MeasureExecutable).Execute(ec)
 	if err != nil {
 		return nil, err
@@ -115,11 +117,15 @@ func (g *groupBy) sort(ec executor.MeasureExecutionContext) (executor.MIterator,
 	return newGroupSortIterator(iter, g.groupByTagsRefs), nil
 }
 
-func (g *groupBy) hash(ec executor.MeasureExecutionContext) (executor.MIterator, error) {
+func (g *groupBy) hash(ec context.Context) (mit executor.MIterator, err error) {
 	iter, err := g.Parent.Input.(executor.MeasureExecutable).Execute(ec)
 	if err != nil {
 		return nil, err
 	}
+	defer func() {
+		err = multierr.Append(err, iter.Close())
+	}()
+
 	groupMap := make(map[uint64][]*measurev1.DataPoint)
 	groupLst := make([]uint64, 0)
 	for iter.Next() {
@@ -141,9 +147,6 @@ func (g *groupBy) hash(ec executor.MeasureExecutionContext) (executor.MIterator,
 			groupMap[key] = group
 		}
 	}
-	if err = iter.Close(); err != nil {
-		return nil, err
-	}
 	return newGroupIterator(groupMap, groupLst), nil
 }
 
@@ -151,6 +154,12 @@ func formatGroupByKey(point *measurev1.DataPoint, groupByTagsRefs [][]*logical.T
 	hash := xxhash.New()
 	for _, tagFamilyRef := range groupByTagsRefs {
 		for _, tagRef := range tagFamilyRef {
+			if tagRef.Spec.TagFamilyIdx >= len(point.GetTagFamilies()) {
+				return 0, errors.New("tag family index out of range")
+			}
+			if tagRef.Spec.TagIdx >= len(point.GetTagFamilies()[tagRef.Spec.TagFamilyIdx].GetTags()) {
+				return 0, errors.New("tag index out of range")
+			}
 			tag := point.GetTagFamilies()[tagRef.Spec.TagFamilyIdx].GetTags()[tagRef.Spec.TagIdx]
 			switch v := tag.GetValue().GetValue().(type) {
 			case *modelv1.TagValue_Str:
@@ -262,7 +271,7 @@ func (gmi *groupSortIterator) Current() []*measurev1.DataPoint {
 
 func (gmi *groupSortIterator) Close() error {
 	gmi.closed = true
-	return gmi.err
+	return multierr.Combine(gmi.err, gmi.iter.Close())
 }
 
 func (gmi *groupSortIterator) nextDP() (*measurev1.DataPoint, bool) {
