@@ -50,7 +50,7 @@ func ApplyRules(plan Plan, rules ...OptimizeRule) error {
 // VolumeLimiter controls the volume of a Plan's output.
 type VolumeLimiter interface {
 	Plan
-	Limit(max int)
+	Limit(maxVal int)
 }
 
 // Sorter sorts a Plan's output.
@@ -74,10 +74,11 @@ func NewPushDownOrder(order *modelv1.QueryOrder) PushDownOrder {
 // Optimize a Plan by pushing down the query order.
 func (pdo PushDownOrder) Optimize(plan Plan) (Plan, error) {
 	if v, ok := plan.(Sorter); ok {
-		if order, err := ParseOrderBy(v.Schema(), pdo.order.GetIndexRuleName(), pdo.order.GetSort()); err == nil {
+		if order, err := ParseOrderBy(v.Schema(),
+			pdo.order.GetIndexRuleName(), pdo.order.GetSort()); err == nil && order != nil {
 			v.Sort(order)
 		} else {
-			return nil, err
+			return plan, err
 		}
 	}
 	return plan, nil
