@@ -34,6 +34,7 @@ import (
 
 func buildLocalFilter(criteria *modelv1.Criteria, schema logical.Schema,
 	entityDict map[string]int, entity []*modelv1.TagValue,
+	indexRuleType databasev1.IndexRule_Type,
 ) (index.Filter, [][]*modelv1.TagValue, error) {
 	if criteria == nil {
 		return nil, [][]*modelv1.TagValue{entity}, nil
@@ -48,8 +49,8 @@ func buildLocalFilter(criteria *modelv1.Criteria, schema logical.Schema,
 		if parsedEntity != nil {
 			return nil, parsedEntity, nil
 		}
-		if ok, indexRule := schema.IndexDefined(cond.Name); ok {
-			return parseConditionToFilter(cond, indexRule, expr, entity)
+		if ok, indexRule := schema.IndexDefined(cond.Name); ok && indexRule.Type == indexRuleType {
+			return parseConditionToFilter(cond, indexRule, expr, entity, schema)
 		}
 		return ENode, [][]*modelv1.TagValue{entity}, nil
 	case *modelv1.Criteria_Le:
@@ -58,16 +59,16 @@ func buildLocalFilter(criteria *modelv1.Criteria, schema logical.Schema,
 			return nil, nil, errors.WithMessagef(logical.ErrInvalidLogicalExpression, "both sides(left and right) of [%v] are empty", criteria)
 		}
 		if le.GetLeft() == nil {
-			return buildLocalFilter(le.Right, schema, entityDict, entity)
+			return buildLocalFilter(le.Right, schema, entityDict, entity, indexRuleType)
 		}
 		if le.GetRight() == nil {
-			return buildLocalFilter(le.Left, schema, entityDict, entity)
+			return buildLocalFilter(le.Left, schema, entityDict, entity, indexRuleType)
 		}
-		left, leftEntities, err := buildLocalFilter(le.Left, schema, entityDict, entity)
+		left, leftEntities, err := buildLocalFilter(le.Left, schema, entityDict, entity, indexRuleType)
 		if err != nil {
 			return nil, nil, err
 		}
-		right, rightEntities, err := buildLocalFilter(le.Right, schema, entityDict, entity)
+		right, rightEntities, err := buildLocalFilter(le.Right, schema, entityDict, entity, indexRuleType)
 		if err != nil {
 			return nil, nil, err
 		}
@@ -75,8 +76,11 @@ func buildLocalFilter(criteria *modelv1.Criteria, schema logical.Schema,
 		if entities == nil {
 			return nil, nil, nil
 		}
-		if left == nil && right == nil {
-			return nil, entities, nil
+		if left == nil {
+			return right, entities, nil
+		}
+		if right == nil {
+			return left, entities, nil
 		}
 		if left == ENode && right == ENode {
 			return ENode, entities, nil
@@ -99,7 +103,7 @@ func buildLocalFilter(criteria *modelv1.Criteria, schema logical.Schema,
 }
 
 func parseConditionToFilter(cond *modelv1.Condition, indexRule *databasev1.IndexRule,
-	expr logical.LiteralExpr, entity []*modelv1.TagValue,
+	expr logical.LiteralExpr, entity []*modelv1.TagValue, schema logical.Schema,
 ) (index.Filter, [][]*modelv1.TagValue, error) {
 	switch cond.Op {
 	case modelv1.Condition_BINARY_OP_GT:
@@ -113,10 +117,17 @@ func parseConditionToFilter(cond *modelv1.Condition, indexRule *databasev1.Index
 	case modelv1.Condition_BINARY_OP_EQ:
 		return newEq(indexRule, expr), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_MATCH:
-		return newMatch(indexRule, expr, cond.MatchOption), [][]*modelv1.TagValue{entity}, nil
+		if indexRule.Type == databasev1.IndexRule_TYPE_INVERTED {
+			return newMatch(indexRule, expr, cond.MatchOption), [][]*modelv1.TagValue{entity}, nil
+		}
+		return nil, nil, errors.WithMessagef(logical.ErrUnsupportedConditionOp, "index filter parses %v for skipping index", cond)
 	case modelv1.Condition_BINARY_OP_NE:
 		return newNot(indexRule, newEq(indexRule, expr)), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_HAVING:
+		tagSpec := schema.FindTagSpecByName(cond.Name)
+		if tagSpec == nil {
+			return nil, nil, errors.WithMessagef(logical.ErrUnsupportedConditionOp, "index filter parses %v for skipping index", cond)
+		}
 		ee := expr.SubExprs()
 		l := len(ee)
 		if l < 1 {
@@ -128,6 +139,10 @@ func parseConditionToFilter(cond *modelv1.Condition, indexRule *databasev1.Index
 		}
 		return and, [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_NOT_HAVING:
+		tagSpec := schema.FindTagSpecByName(cond.Name)
+		if tagSpec == nil {
+			return nil, nil, errors.WithMessagef(logical.ErrUnsupportedConditionOp, "index filter parses %v for skipping index", cond)
+		}
 		ee := expr.SubExprs()
 		l := len(ee)
 		if l < 1 {
@@ -139,6 +154,10 @@ func parseConditionToFilter(cond *modelv1.Condition, indexRule *databasev1.Index
 		}
 		return newNot(indexRule, and), [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_IN:
+		tagSpec := schema.FindTagSpecByName(cond.Name)
+		if tagSpec != nil && (tagSpec.Spec.GetType() == databasev1.TagType_TAG_TYPE_STRING_ARRAY || tagSpec.Spec.GetType() == databasev1.TagType_TAG_TYPE_INT_ARRAY) {
+			return nil, nil, errors.WithMessagef(logical.ErrUnsupportedConditionOp, "in condition is not supported for array type")
+		}
 		ee := expr.SubExprs()
 		l := len(ee)
 		if l < 1 {
@@ -150,6 +169,10 @@ func parseConditionToFilter(cond *modelv1.Condition, indexRule *databasev1.Index
 		}
 		return or, [][]*modelv1.TagValue{entity}, nil
 	case modelv1.Condition_BINARY_OP_NOT_IN:
+		tagSpec := schema.FindTagSpecByName(cond.Name)
+		if tagSpec != nil && (tagSpec.Spec.GetType() == databasev1.TagType_TAG_TYPE_STRING_ARRAY || tagSpec.Spec.GetType() == databasev1.TagType_TAG_TYPE_INT_ARRAY) {
+			return nil, nil, errors.WithMessagef(logical.ErrUnsupportedConditionOp, "not in condition is not supported for array type")
+		}
 		ee := expr.SubExprs()
 		l := len(ee)
 		if l < 1 {
@@ -264,6 +287,19 @@ func (an *andNode) Execute(searcher index.GetSearcher, seriesID common.SeriesID,
 	return execute(searcher, seriesID, an.node, an, tr)
 }
 
+func (an *andNode) ShouldSkip(tagFamilyFilters index.FilterOp) (bool, error) {
+	for _, sn := range an.node.SubNodes {
+		shouldSkip, err := sn.ShouldSkip(tagFamilyFilters)
+		if err != nil {
+			return shouldSkip, err
+		}
+		if shouldSkip {
+			return true, nil
+		}
+	}
+	return false, nil
+}
+
 func (an *andNode) MarshalJSON() ([]byte, error) {
 	data := make(map[string]interface{}, 1)
 	data["and"] = an.node.SubNodes
@@ -309,6 +345,19 @@ func (on *orNode) Execute(searcher index.GetSearcher, seriesID common.SeriesID,
 	return execute(searcher, seriesID, on.node, on, tr)
 }
 
+func (on *orNode) ShouldSkip(tagFamilyFilters index.FilterOp) (bool, error) {
+	for _, sn := range on.node.SubNodes {
+		shouldSkip, err := sn.ShouldSkip(tagFamilyFilters)
+		if err != nil {
+			return shouldSkip, err
+		}
+		if !shouldSkip {
+			return false, nil
+		}
+	}
+	return true, nil
+}
+
 func (on *orNode) MarshalJSON() ([]byte, error) {
 	data := make(map[string]interface{}, 1)
 	data["or"] = on.node.SubNodes
@@ -400,6 +449,10 @@ func (eq *eq) Execute(searcher index.GetSearcher, seriesID common.SeriesID, tr *
 	return s.MatchTerms(eq.Expr.Field(eq.Key.toIndex(seriesID, tr)))
 }
 
+func (eq *eq) ShouldSkip(tagFamilyFilters index.FilterOp) (bool, error) {
+	return !tagFamilyFilters.Eq(eq.Key.Tags[0], eq.Expr.String()), nil
+}
+
 func (eq *eq) MarshalJSON() ([]byte, error) {
 	data := make(map[string]interface{}, 1)
 	data["eq"] = eq.leaf
@@ -442,6 +495,10 @@ func (match *match) Execute(searcher index.GetSearcher, seriesID common.SeriesID
 	)
 }
 
+func (match *match) ShouldSkip(_ index.FilterOp) (bool, error) {
+	return false, nil
+}
+
 func (match *match) MarshalJSON() ([]byte, error) {
 	data := make(map[string]interface{}, 1)
 	data["match"] = match.leaf
@@ -474,6 +531,10 @@ func (r *rangeOp) Execute(searcher index.GetSearcher, seriesID common.SeriesID,
 	return s.Range(r.Key.toIndex(seriesID, tr), r.Opts)
 }
 
+func (r *rangeOp) ShouldSkip(tagFamilyFilters index.FilterOp) (bool, error) {
+	return tagFamilyFilters.Range(r.Key.Tags[0], r.Opts)
+}
+
 func (r *rangeOp) MarshalJSON() ([]byte, error) {
 	data := make(map[string]interface{}, 1)
 	var builder strings.Builder
@@ -521,6 +582,10 @@ func (an emptyNode) String() string {
 	return "empty"
 }
 
+func (an emptyNode) ShouldSkip(_ index.FilterOp) (bool, error) {
+	return false, nil
+}
+
 type bypassList struct{}
 
 func (bl bypassList) Contains(_ uint64) bool {
