@@ -19,8 +19,6 @@
 package stream
 
 import (
-	"github.com/google/go-cmp/cmp"
-
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/pkg/query/logical"
 )
@@ -28,8 +26,9 @@ import (
 var _ logical.Schema = (*schema)(nil)
 
 type schema struct {
-	stream *databasev1.Stream
-	common *logical.CommonSchema
+	stream   *databasev1.Stream
+	common   *logical.CommonSchema
+	children []logical.Schema
 }
 
 func (s *schema) FindTagSpecByName(name string) *logical.TagSpec {
@@ -53,13 +52,6 @@ func (s *schema) IndexDefined(tagName string) (bool, *databasev1.IndexRule) {
 	return s.common.IndexDefined(tagName)
 }
 
-func (s *schema) Equal(s2 logical.Schema) bool {
-	if other, ok := s2.(*schema); ok {
-		return cmp.Equal(other.common.TagSpecMap, s.common.TagSpecMap)
-	}
-	return false
-}
-
 // CreateTagRef create TagRef to the given tags.
 // The family name of the tag is actually not used
 // since the uniqueness of the tag names can be guaranteed across families.
@@ -83,3 +75,39 @@ func (s *schema) ProjTags(refs ...[]*logical.TagRef) logical.Schema {
 func (s *schema) ProjFields(...*logical.FieldRef) logical.Schema {
 	panic("stream does not support field")
 }
+
+func (s *schema) Children() []logical.Schema {
+	return s.children
+}
+
+func mergeSchema(schemas []logical.Schema) (logical.Schema, error) {
+	if len(schemas) == 0 {
+		return nil, nil
+	}
+	if len(schemas) == 1 {
+		return schemas[0], nil
+	}
+	var commonSchemas []*logical.CommonSchema
+	var tagFamilies []*databasev1.TagFamilySpec
+	for _, sm := range schemas {
+		if sm == nil {
+			continue
+		}
+		s := sm.(*schema)
+		if s == nil {
+			continue
+		}
+		tagFamilies = logical.MergeTagFamilySpecs(tagFamilies, s.stream.GetTagFamilies())
+		commonSchemas = append(commonSchemas, s.common)
+	}
+	merged, err := logical.MergeSchemas(commonSchemas)
+	if err != nil {
+		return nil, err
+	}
+	ret := &schema{
+		common:   merged,
+		children: schemas,
+	}
+	ret.common.RegisterTagFamilies(tagFamilies)
+	return ret, nil
+}
