@@ -152,12 +152,14 @@ func (t *distributedPlan) Execute(ctx context.Context) (ee []*streamv1.Element,
 			}
 		}()
 	}
-	ff, err := dctx.Broadcast(defaultQueryTimeout, data.TopicStreamQuery, bus.NewMessage(bus.MessageID(dctx.TimeRange().Begin.Nanos), queryRequest))
+	ff, err := dctx.Broadcast(defaultQueryTimeout, data.TopicStreamQuery,
+		bus.NewMessageWithNodeSelectors(bus.MessageID(dctx.TimeRange().Begin.Nanos), dctx.NodeSelectors(), dctx.TimeRange(), queryRequest))
 	if err != nil {
 		return nil, err
 	}
 	var allErr error
 	var see []sort.Iterator[*comparableElement]
+	var responseCount int
 	for _, f := range ff {
 		if m, getErr := f.Get(); getErr != nil {
 			allErr = multierr.Append(allErr, getErr)
@@ -167,6 +169,7 @@ func (t *distributedPlan) Execute(ctx context.Context) (ee []*streamv1.Element,
 				continue
 			}
 			resp := d.(*streamv1.QueryResponse)
+			responseCount++
 			if span != nil {
 				span.AddSubTrace(resp.Trace)
 			}
@@ -174,11 +177,21 @@ func (t *distributedPlan) Execute(ctx context.Context) (ee []*streamv1.Element,
 				newSortableElements(resp.Elements, t.sortByTime, t.sortTagSpec))
 		}
 	}
-	iter := sort.NewItemIter[*comparableElement](see, t.desc)
+	iter := sort.NewItemIter(see, t.desc)
 	var result []*streamv1.Element
+	seen := make(map[string]bool)
 	for iter.Next() {
-		result = append(result, iter.Val().Element)
+		element := iter.Val().Element
+		if !seen[element.ElementId] {
+			seen[element.ElementId] = true
+			result = append(result, element)
+		}
 	}
+	if span != nil {
+		span.Tagf("response_count", "%d", responseCount)
+		span.Tagf("element_id_count", "%d", len(seen))
+	}
+
 	return result, allErr
 }
 
