@@ -87,7 +87,8 @@ type StreamQueryOptions struct {
 	Name           string
 	TimeRange      *timestamp.TimeRange
 	Entities       [][]*modelv1.TagValue
-	Filter         index.Filter
+	InvertedFilter index.Filter
+	SkippingFilter index.Filter
 	Order          *index.OrderBy
 	TagProjection  []TagProjection
 	MaxElementSize int
@@ -98,7 +99,8 @@ func (s *StreamQueryOptions) Reset() {
 	s.Name = ""
 	s.TimeRange = nil
 	s.Entities = nil
-	s.Filter = nil
+	s.InvertedFilter = nil
+	s.SkippingFilter = nil
 	s.Order = nil
 	s.TagProjection = nil
 	s.MaxElementSize = 0
@@ -117,7 +119,8 @@ func (s *StreamQueryOptions) CopyFrom(other *StreamQueryOptions) {
 		s.Entities = nil
 	}
 
-	s.Filter = other.Filter
+	s.InvertedFilter = other.InvertedFilter
+	s.SkippingFilter = other.SkippingFilter
 	s.Order = other.Order
 
 	// Deep copy if TagProjection is a slice
@@ -311,3 +314,60 @@ type StreamQueryResult interface {
 	Pull(context.Context) *StreamResult
 	Release()
 }
+
+// TraceQueryOptions is the options of a trace query.
+type TraceQueryOptions struct {
+	SkippingFilter index.Filter
+	TagFilter      TagFilterMatcher
+	TimeRange      *timestamp.TimeRange
+	Order          *index.OrderBy
+	TagProjection  *TagProjection
+	Name           string
+	Entities       [][]*modelv1.TagValue
+	TraceIDs       []string
+	MaxTraceSize   int
+	MinVal         int64
+	MaxVal         int64
+}
+
+// Reset resets the TraceQueryOptions.
+func (t *TraceQueryOptions) Reset() {
+	t.Name = ""
+	t.TimeRange = nil
+	t.SkippingFilter = nil
+	t.TagFilter = nil
+	t.Order = nil
+	t.TagProjection = nil
+	t.Entities = nil
+	t.TraceIDs = nil
+	t.MaxTraceSize = 0
+}
+
+// CopyFrom copies the TraceQueryOptions from other to t.
+func (t *TraceQueryOptions) CopyFrom(other *TraceQueryOptions) {
+	t.Name = other.Name
+	t.TimeRange = other.TimeRange
+	t.SkippingFilter = other.SkippingFilter
+	t.TagFilter = other.TagFilter
+	t.Order = other.Order
+	t.TagProjection = other.TagProjection
+	t.Entities = other.Entities
+	t.MaxTraceSize = other.MaxTraceSize
+}
+
+// TraceResult is the result of a query.
+type TraceResult struct {
+	Error      error
+	TID        string
+	Spans      [][]byte
+	SpanIDs    []string
+	Tags       []Tag
+	Key        int64
+	GroupIndex int
+}
+
+// TraceQueryResult is the result of a trace query.
+type TraceQueryResult interface {
+	Pull() *TraceResult
+	Release()
+}
