@@ -31,6 +31,7 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/banyand/stream"
+	"github.com/apache/skywalking-banyandb/banyand/trace"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
 )
@@ -41,13 +42,14 @@ type queryService struct {
 	log         *logger.Logger
 	sqp         *streamQueryProcessor
 	mqp         *measureQueryProcessor
-	tqp         *topNQueryProcessor
+	nqp         *topNQueryProcessor
+	tqp         *traceQueryProcessor
 	nodeID      string
 	slowQuery   time.Duration
 }
 
 // NewService return a new query service.
-func NewService(_ context.Context, streamService stream.Service, measureService measure.Service,
+func NewService(_ context.Context, streamService stream.Service, measureService measure.Service, traceService trace.Service,
 	metaService metadata.Repo, pipeline queue.Server,
 ) (run.Unit, error) {
 	svc := &queryService{
@@ -65,10 +67,15 @@ func NewService(_ context.Context, streamService stream.Service, measureService
 		queryService:  svc,
 	}
 	// topN query processor
-	svc.tqp = &topNQueryProcessor{
+	svc.nqp = &topNQueryProcessor{
 		measureService: measureService,
 		queryService:   svc,
 	}
+	// trace query processor
+	svc.tqp = &traceQueryProcessor{
+		traceService: traceService,
+		queryService: svc,
+	}
 	return svc, nil
 }
 
@@ -87,7 +94,8 @@ func (q *queryService) PreRun(ctx context.Context) error {
 	return multierr.Combine(
 		q.pipeline.Subscribe(data.TopicStreamQuery, q.sqp),
 		q.pipeline.Subscribe(data.TopicMeasureQuery, q.mqp),
-		q.pipeline.Subscribe(data.TopicTopNQuery, q.tqp),
+		q.pipeline.Subscribe(data.TopicTopNQuery, q.nqp),
+		q.pipeline.Subscribe(data.TopicTraceQuery, q.tqp),
 	)
 }
 
