@@ -99,6 +99,10 @@ func (*local) GetPort() *uint32 {
 func (*local) Register(bus.Topic, schema.EventHandler) {
 }
 
+func (*local) HealthyNodes() []string {
+	return nil
+}
+
 type localBatchPublisher struct {
 	ctx      context.Context
 	local    *bus.Bus
@@ -148,3 +152,46 @@ func (l *localBatchPublisher) Close() (map[string]*common.Error, error) {
 	}
 	return nil, nil
 }
+
+func (l *local) RegisterChunkedSyncHandler(_ bus.Topic, _ ChunkedSyncHandler) {
+}
+
+func (l *local) NewChunkedSyncClient(node string, _ uint32) (ChunkedSyncClient, error) {
+	return &localChunkedSyncClient{local: l.local, node: node}, nil
+}
+
+type localChunkedSyncClient struct {
+	local *bus.Bus
+	node  string
+}
+
+func (l *localChunkedSyncClient) Close() error {
+	return nil
+}
+
+func (l *localChunkedSyncClient) SyncStreamingParts(_ context.Context, parts []StreamingPartData) (*SyncResult, error) {
+	// Check for test failure injector
+	if injector := GetChunkedSyncFailureInjector(); injector != nil {
+		shouldFail, failedParts, err := injector.BeforeSync(parts)
+		if err != nil {
+			return nil, err
+		}
+		if shouldFail {
+			return &SyncResult{
+				Success:     false,
+				SessionID:   "local-session",
+				PartsCount:  uint32(len(parts)),
+				FailedParts: failedParts,
+			}, nil
+		}
+	}
+
+	return &SyncResult{
+		Success:     true,
+		SessionID:   "local-session",
+		TotalBytes:  0,
+		DurationMs:  0,
+		ChunksCount: 0,
+		PartsCount:  uint32(len(parts)),
+	}, nil
+}
