@@ -33,7 +33,7 @@ import (
 	"google.golang.org/grpc/health"
 	"google.golang.org/grpc/health/grpc_health_v1"
 	"google.golang.org/grpc/status"
-	"google.golang.org/protobuf/types/known/anypb"
+	"google.golang.org/protobuf/proto"
 
 	"github.com/apache/skywalking-banyandb/api/data"
 	clusterv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/cluster/v1"
@@ -82,14 +82,14 @@ func (s *mockServer) Send(stream clusterv1.Service_SendServer) (err error) {
 
 		f := data.TopicResponseMap[topic]
 
-		var body *anypb.Any
+		var body []byte
+		var errMarshal error
 		if f == nil {
 			body = first.Body
 		} else {
-			var errAny error
-			body, errAny = anypb.New(f())
-			if errAny != nil {
-				panic(errAny)
+			body, errMarshal = proto.Marshal(f())
+			if errMarshal != nil {
+				panic(errMarshal)
 			}
 		}
 		res := &clusterv1.SendResponse{
@@ -214,11 +214,40 @@ func (m *mockHandler) OnDelete(_ schema.Metadata) {
 	m.deleteCount++
 }
 
-func newPub() *pub {
-	p := NewWithoutMetadata().(*pub)
+func newPub(roles ...databasev1.Role) *pub {
+	p := New(nil, roles...)
+	p.(*pub).log = logger.GetLogger("queue-client")
 	p.Register(data.TopicStreamWrite, &mockHandler{})
 	p.Register(data.TopicMeasureWrite, &mockHandler{})
-	return p
+	return p.(*pub)
+}
+
+// newPubWithNoRetry creates a pub with a retry policy that doesn't retry Unavailable errors.
+func newPubWithNoRetry(roles ...databasev1.Role) *pub {
+	p := New(nil, roles...)
+	p.(*pub).log = logger.GetLogger("queue-client")
+	p.Register(data.TopicStreamWrite, &mockHandler{})
+	p.Register(data.TopicMeasureWrite, &mockHandler{})
+
+	// Override the retry policy to not retry Unavailable errors
+	noRetryPolicy := `{
+		"methodConfig": [
+		  {
+		    "name": [{"service": "banyandb.cluster.v1.Service"}],
+		    "waitForReady": true,
+		    "retryPolicy": {
+		        "MaxAttempts": 2,
+		        "InitialBackoff": ".5s",
+		        "MaxBackoff": "10s",
+		        "BackoffMultiplier": 2.0,
+		        "RetryableStatusCodes": [ "DEADLINE_EXCEEDED", "RESOURCE_EXHAUSTED" ]
+		    }
+		  }
+		]}`
+
+	// Store the original retry policy and replace it
+	p.(*pub).retryPolicy = noRetryPolicy
+	return p.(*pub)
 }
 
 func getDataNode(name string, address string) schema.Metadata {
@@ -237,10 +266,9 @@ func getDataNode(name string, address string) schema.Metadata {
 	}
 }
 
-func getDataNodeWithLabels(name string, address string, labels map[string]string, dataBoundary map[string]*modelv1.TimeRange) schema.Metadata {
+func getDataNodeWithLabels(name string, address string, labels map[string]string) schema.Metadata {
 	node := getDataNode(name, address)
 	nodeInfo := node.Spec.(*databasev1.Node)
 	nodeInfo.Labels = labels
-	nodeInfo.DataSegmentsBoundary = dataBoundary
 	return node
 }
