@@ -43,13 +43,13 @@ type EventKind uint8
 const (
 	EventKindGroup EventKind = iota
 	EventKindResource
-	EventKindTopNAgg
+	EventKindIndexRule
+	EventKindIndexRuleBinding
 )
 
 // Group is the root node, allowing get resources from its sub nodes.
 type Group interface {
 	GetSchema() *commonv1.Group
-	LoadResource(name string) (Resource, bool)
 	SupplyTSDB() io.Closer
 }
 
@@ -67,11 +67,8 @@ type ResourceSchema interface {
 
 // Resource allows access metadata from a local cache.
 type Resource interface {
-	io.Closer
-	IndexRules() []*databasev1.IndexRule
-	TopN() []*databasev1.TopNAggregation
 	Schema() ResourceSchema
-	Delegated() io.Closer
+	Delegated() IndexListener
 }
 
 // Supplier allows open a tsdb.
@@ -79,24 +76,36 @@ type Supplier interface {
 	SupplyTSDB() io.Closer
 }
 
+// IndexListener listens to the index update.
+type IndexListener interface {
+	OnIndexUpdate(index []*databasev1.IndexRule)
+}
+
 // ResourceSchemaSupplier allows get a ResourceSchema from the metadata.
 type ResourceSchemaSupplier interface {
 	ResourceSchema(metadata *commonv1.Metadata) (ResourceSchema, error)
-	OpenResource(shardNum uint32, supplier Supplier, spec Resource) (io.Closer, error)
+	OpenResource(spec Resource) (IndexListener, error)
 }
 
 // ResourceSupplier allows open a resource and its embedded tsdb.
 type ResourceSupplier interface {
 	ResourceSchemaSupplier
-	OpenDB(groupSchema *commonv1.Group) (io.Closer, error)
+	OpenDB(groupSchema *commonv1.Group) (DB, error)
+}
+
+// DB is the interface of a tsdb.
+type DB interface {
+	io.Closer
+	UpdateOptions(opts *commonv1.ResourceOpts)
 }
 
 // Repository is the collection of several hierarchies groups by a "Group".
 type Repository interface {
 	Watcher()
-	Init(schema.Kind) []int64
+	Init(schema.Kind) ([]string, []int64)
 	SendMetadataEvent(MetadataEvent)
 	LoadGroup(name string) (Group, bool)
+	LoadAllGroups() []Group
 	LoadResource(metadata *commonv1.Metadata) (Resource, bool)
 	Close()
 	StopCh() <-chan struct{}
