@@ -48,33 +48,20 @@ func (tst *tsTable) flusherLoop(flushCh chan *flusherIntroduction, mergeCh chan
 					tst.incTotalFlushLatency(time.Since(start).Seconds())
 				}()
 				curSnapshot := tst.currentSnapshot()
-				if curSnapshot != nil {
-					flusherWatchers = tst.pauseFlusherToPileupMemParts(epoch, flusherWatcher, flusherWatchers)
-					curSnapshot.decRef()
-					curSnapshot = nil
-				}
-				tst.RLock()
-				if tst.snapshot != nil && tst.snapshot.epoch > epoch {
-					curSnapshot = tst.snapshot
-					curSnapshot.incRef()
-				}
-				tst.RUnlock()
+				var ok bool
+				var merged bool
+				curSnapshot, flusherWatchers, ok, merged = tst.pauseFlusherToPileupMemPartsWithMerge(curSnapshot, flusherWatcher, flusherWatchers, epoch, mergeCh)
 				if curSnapshot != nil {
 					defer curSnapshot.decRef()
-					merged, err := tst.mergeMemParts(curSnapshot, mergeCh)
-					if err != nil {
-						tst.l.Logger.Warn().Err(err).Msgf("cannot merge snapshot: %d", curSnapshot.epoch)
-						tst.incTotalFlushLoopErr(1)
+					if !ok {
 						return false
 					}
 					if !merged {
 						tst.flush(curSnapshot, flushCh)
 					}
-					epoch = curSnapshot.epoch
-					// Notify merger to start a new round of merge.
-					// This round might have be triggered in pauseFlusherToPileupMemParts.
 					flusherWatchers.Notify(math.MaxUint64)
 					flusherWatchers = nil
+					epoch = curSnapshot.epoch
 					if tst.currentEpoch() != epoch {
 						tst.incTotalFlushLoopProgress(1)
 						return false
@@ -89,6 +76,36 @@ func (tst *tsTable) flusherLoop(flushCh chan *flusherIntroduction, mergeCh chan
 	}
 }
 
+func (tst *tsTable) pauseFlusherToPileupMemPartsWithMerge(
+	curSnapshot *snapshot, flusherWatcher watcher.Channel, flusherWatchers watcher.Epochs,
+	epoch uint64, mergeCh chan *mergerIntroduction,
+) (*snapshot, watcher.Epochs, bool, bool) {
+	if tst.option.flushTimeout < 1 {
+		return curSnapshot, flusherWatchers, true, false
+	}
+	if curSnapshot != nil {
+		flusherWatchers = tst.pauseFlusherToPileupMemParts(epoch, flusherWatcher, flusherWatchers)
+		curSnapshot.decRef()
+		curSnapshot = nil
+	}
+	tst.RLock()
+	if tst.snapshot != nil && tst.snapshot.epoch > epoch {
+		curSnapshot = tst.snapshot
+		curSnapshot.incRef()
+	}
+	tst.RUnlock()
+	if curSnapshot == nil {
+		return nil, flusherWatchers, false, false
+	}
+	merged, err := tst.mergeMemParts(curSnapshot, mergeCh)
+	if err != nil {
+		tst.l.Logger.Warn().Err(err).Msgf("cannot merge snapshot: %d", curSnapshot.epoch)
+		tst.incTotalFlushLoopErr(1)
+		return curSnapshot, flusherWatchers, false, false
+	}
+	return curSnapshot, flusherWatchers, true, merged
+}
+
 // pauseFlusherToPileupMemParts takes a pause to wait for in-memory parts to pile up.
 // If there is no in-memory part, we can skip the pause.
 // When a merging is finished, we can skip the pause.
@@ -112,33 +129,80 @@ func (tst *tsTable) pauseFlusherToPileupMemParts(epoch uint64, flushWatcher watc
 }
 
 func (tst *tsTable) mergeMemParts(snp *snapshot, mergeCh chan *mergerIntroduction) (bool, error) {
+	var merged bool
+	var currentSegmentID int64
 	var memParts []*partWrapper
 	mergedIDs := make(map[uint64]struct{})
+
+	// Helper function to merge current segment's parts
+	mergeCurrentSegment := func() (bool, error) {
+		if len(memParts) < 2 {
+			return false, nil
+		}
+
+		// Create a copy of mergedIDs for this merge operation
+		currentMergedIDs := make(map[uint64]struct{}, len(mergedIDs))
+		for id := range mergedIDs {
+			currentMergedIDs[id] = struct{}{}
+		}
+
+		// merge memory must not be closed by the tsTable.close
+		closeCh := make(chan struct{})
+		newPart, err := tst.mergePartsThenSendIntroduction(snapshotCreatorMergedFlusher, memParts,
+			currentMergedIDs, mergeCh, closeCh, "mem")
+		close(closeCh)
+		if err != nil {
+			if errors.Is(err, errClosed) {
+				return true, nil
+			}
+			return false, err
+		}
+		return newPart != nil, nil
+	}
+
+	// Process parts grouped by segmentID
 	for i := range snp.parts {
-		if snp.parts[i].mp != nil {
-			memParts = append(memParts, snp.parts[i])
-			mergedIDs[snp.parts[i].ID()] = struct{}{}
+		if snp.parts[i].mp == nil {
 			continue
 		}
-	}
-	if len(memParts) < 2 {
-		return false, nil
-	}
-	// merge memory must not be closed by the tsTable.close
-	closeCh := make(chan struct{})
-	newPart, err := tst.mergePartsThenSendIntroduction(snapshotCreatorMergedFlusher, memParts,
-		mergedIDs, mergeCh, closeCh, "mem")
-	close(closeCh)
-	if err != nil {
-		if errors.Is(err, errClosed) {
-			return true, nil
+
+		segID := snp.parts[i].mp.segmentID
+
+		// If this is a new segment, merge the previous segment first
+		if currentSegmentID != 0 && currentSegmentID != segID {
+			m, err := mergeCurrentSegment()
+			if err != nil {
+				return false, err
+			}
+			if m {
+				merged = true
+			}
+
+			// Reset for next segment
+			memParts = memParts[:0]
+			for id := range mergedIDs {
+				delete(mergedIDs, id)
+			}
 		}
-		return false, err
+
+		// Add part to current segment
+		currentSegmentID = segID
+		memParts = append(memParts, snp.parts[i])
+		mergedIDs[snp.parts[i].ID()] = struct{}{}
 	}
-	if newPart == nil {
-		return false, nil
+
+	// Merge the last segment if it has parts
+	if len(memParts) >= 2 {
+		m, err := mergeCurrentSegment()
+		if err != nil {
+			return false, err
+		}
+		if m {
+			merged = true
+		}
 	}
-	return true, nil
+
+	return merged, nil
 }
 
 func (tst *tsTable) flush(snapshot *snapshot, flushCh chan *flusherIntroduction) {
