@@ -56,6 +56,7 @@ const (
 	snapshotCreatorFlusher
 	snapshotCreatorMerger
 	snapshotCreatorMergedFlusher
+	snapshotCreatorSyncer
 )
 
 type snapshot struct {
@@ -193,6 +194,9 @@ func parseSnapshot(name string) (uint64, error) {
 }
 
 func (tst *tsTable) TakeFileSnapshot(dst string) error {
+	if tst.index == nil {
+		return fmt.Errorf("cannot take file snapshot: index is not initialized for this tsTable")
+	}
 	indexDir := filepath.Join(dst, filepath.Base(tst.index.location))
 	tst.fileSystem.MkdirPanicIfExist(indexDir, storage.DirPerm)
 	if err := tst.index.store.TakeFileSnapshot(indexDir); err != nil {
@@ -247,7 +251,7 @@ func (tst *tsTable) createMetadata(dst string, snapshot *snapshot) {
 	}
 }
 
-func (s *service) takeGroupSnapshot(dstDir string, groupName string) error {
+func (s *standalone) takeGroupSnapshot(dstDir string, groupName string) error {
 	group, ok := s.schemaRepo.LoadGroup(groupName)
 	if !ok {
 		return errors.Errorf("group %s not found", groupName)
@@ -265,7 +269,7 @@ func (s *service) takeGroupSnapshot(dstDir string, groupName string) error {
 
 type snapshotListener struct {
 	*bus.UnImplementedHealthyListener
-	s           *service
+	s           *standalone
 	snapshotSeq uint64
 	snapshotMux sync.Mutex
 }
