@@ -27,6 +27,7 @@ import (
 	commonv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/common/v1"
 	databasev1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/database/v1"
 	"github.com/apache/skywalking-banyandb/banyand/protector"
+	"github.com/apache/skywalking-banyandb/banyand/queue"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/partition"
 	"github.com/apache/skywalking-banyandb/pkg/query/model"
@@ -37,19 +38,23 @@ import (
 
 const (
 	maxValuesBlockSize              = 8 * 1024 * 1024
-	maxElementIDsBlockSize          = 8 * 1024 * 1024
 	maxTagFamiliesMetadataSize      = 8 * 1024 * 1024
 	maxUncompressedBlockSize        = 2 * 1024 * 1024
 	maxUncompressedPrimaryBlockSize = 128 * 1024
 
 	defaultFlushTimeout = time.Second
+	defaultSyncInterval = 30 * time.Second
 )
 
 type option struct {
-	mergePolicy              *mergePolicy
-	flushTimeout             time.Duration
-	elementIndexFlushTimeout time.Duration
-	seriesCacheMaxSize       run.Bytes
+	mergePolicy                  *mergePolicy
+	protector                    protector.Memory
+	tire2Client                  queue.Client
+	seriesCacheMaxSize           run.Bytes
+	flushTimeout                 time.Duration
+	elementIndexFlushTimeout     time.Duration
+	syncInterval                 time.Duration
+	failedPartsMaxTotalSizeBytes uint64
 }
 
 // Query allow to retrieve elements in a series of streams.
@@ -85,11 +90,11 @@ func (i *indexSchema) parse(schema *databasev1.Stream) {
 var _ Stream = (*stream)(nil)
 
 type stream struct {
+	pm          protector.Memory
 	indexSchema atomic.Value
 	tsdb        atomic.Value
 	l           *logger.Logger
 	schema      *databasev1.Stream
-	pm          *protector.Memory
 	schemaRepo  *schemaRepo
 	name        string
 	group       string
@@ -126,7 +131,7 @@ type streamSpec struct {
 }
 
 func openStream(spec streamSpec,
-	l *logger.Logger, pm *protector.Memory, schemaRepo *schemaRepo,
+	l *logger.Logger, pm protector.Memory, schemaRepo *schemaRepo,
 ) *stream {
 	s := &stream{
 		schema:     spec.schema,
