@@ -18,14 +18,16 @@
 package stream
 
 import (
+	internalencoding "github.com/apache/skywalking-banyandb/banyand/internal/encoding"
 	"github.com/apache/skywalking-banyandb/pkg/bytes"
-	"github.com/apache/skywalking-banyandb/pkg/encoding"
+	pkgencoding "github.com/apache/skywalking-banyandb/pkg/encoding"
 	"github.com/apache/skywalking-banyandb/pkg/fs"
 	"github.com/apache/skywalking-banyandb/pkg/logger"
 	pbv1 "github.com/apache/skywalking-banyandb/pkg/pb/v1"
 )
 
 type tag struct {
+	tagFilter
 	name      string
 	values    [][]byte
 	valueType pbv1.ValueType
@@ -39,6 +41,8 @@ func (t *tag) reset() {
 		values[i] = nil
 	}
 	t.values = values[:0]
+
+	t.tagFilter.reset()
 }
 
 func (t *tag) resizeValues(valuesLen int) [][]byte {
@@ -51,7 +55,7 @@ func (t *tag) resizeValues(valuesLen int) [][]byte {
 	return values
 }
 
-func (t *tag) mustWriteTo(tm *tagMetadata, tagWriter *writer) {
+func (t *tag) mustWriteTo(tm *tagMetadata, tagWriter *writer, tagFilterWriter *writer) {
 	tm.reset()
 
 	tm.name = t.name
@@ -60,42 +64,62 @@ func (t *tag) mustWriteTo(tm *tagMetadata, tagWriter *writer) {
 	bb := bigValuePool.Generate()
 	defer bigValuePool.Release(bb)
 
-	// marshal values
-	bb.Buf = encoding.EncodeBytesBlock(bb.Buf[:0], t.values)
+	// Use shared encoding module
+	err := internalencoding.EncodeTagValues(bb, t.values, t.valueType)
+	if err != nil {
+		logger.Panicf("failed to encode tag values: %v", err)
+	}
+
 	tm.size = uint64(len(bb.Buf))
 	if tm.size > maxValuesBlockSize {
-		logger.Panicf("too valuesSize: %d bytes; mustn't exceed %d bytes", tm.size, maxValuesBlockSize)
+		logger.Panicf("too large valuesSize: %d bytes; mustn't exceed %d bytes", tm.size, maxValuesBlockSize)
 	}
 	tm.offset = tagWriter.bytesWritten
 	tagWriter.MustWrite(bb.Buf)
+
+	if t.filter != nil {
+		bb := bigValuePool.Generate()
+		defer bigValuePool.Release(bb)
+		bb.Buf = encodeBloomFilter(bb.Buf[:0], t.filter)
+		if tm.valueType == pbv1.ValueTypeInt64 {
+			tm.min = t.min
+			tm.max = t.max
+		}
+		tm.filterBlock.size = uint64(len(bb.Buf))
+		tm.filterBlock.offset = tagFilterWriter.bytesWritten
+		tagFilterWriter.MustWrite(bb.Buf)
+	}
 }
 
-func (t *tag) mustReadValues(decoder *encoding.BytesBlockDecoder, reader fs.Reader, cm tagMetadata, count uint64) {
+func (t *tag) mustReadValues(decoder *pkgencoding.BytesBlockDecoder, reader fs.Reader, cm tagMetadata, count uint64) {
 	t.name = cm.name
 	t.valueType = cm.valueType
 	if t.valueType == pbv1.ValueTypeUnknown {
-		for i := uint64(0); i < count; i++ {
+		for range count {
 			t.values = append(t.values, nil)
 		}
 		return
 	}
 
-	bb := bigValuePool.Generate()
-	defer bigValuePool.Release(bb)
 	valuesSize := cm.size
 	if valuesSize > maxValuesBlockSize {
 		logger.Panicf("%s: block size cannot exceed %d bytes; got %d bytes", reader.Path(), maxValuesBlockSize, valuesSize)
 	}
-	bb.Buf = bytes.ResizeOver(bb.Buf, int(valuesSize))
+
+	// Use shared decoding module
+	bb := bigValuePool.Generate()
+	defer bigValuePool.Release(bb)
+	bb.Buf = bytes.ResizeOver(bb.Buf[:0], int(valuesSize))
 	fs.MustReadData(reader, int64(cm.offset), bb.Buf)
+
 	var err error
-	t.values, err = decoder.Decode(t.values[:0], bb.Buf, count)
+	t.values, err = internalencoding.DecodeTagValues(t.values, decoder, bb, t.valueType, int(count))
 	if err != nil {
-		logger.Panicf("%s: cannot decode values: %v", reader.Path(), err)
+		logger.Panicf("%s: failed to decode tag values: %v", reader.Path(), err)
 	}
 }
 
-func (t *tag) mustSeqReadValues(decoder *encoding.BytesBlockDecoder, reader *seqReader, cm tagMetadata, count uint64) {
+func (t *tag) mustSeqReadValues(decoder *pkgencoding.BytesBlockDecoder, reader *seqReader, cm tagMetadata, count uint64) {
 	t.name = cm.name
 	t.valueType = cm.valueType
 	if cm.offset != reader.bytesRead {
@@ -108,13 +132,14 @@ func (t *tag) mustSeqReadValues(decoder *encoding.BytesBlockDecoder, reader *seq
 
 	bb := bigValuePool.Generate()
 	defer bigValuePool.Release(bb)
-
-	bb.Buf = bytes.ResizeOver(bb.Buf, int(valuesSize))
+	bb.Buf = bytes.ResizeOver(bb.Buf[:0], int(valuesSize))
 	reader.mustReadFull(bb.Buf)
+
+	// Use shared decoding module
 	var err error
-	t.values, err = decoder.Decode(t.values[:0], bb.Buf, count)
+	t.values, err = internalencoding.DecodeTagValues(t.values, decoder, bb, t.valueType, int(count))
 	if err != nil {
-		logger.Panicf("%s: cannot decode values: %v", reader.Path(), err)
+		logger.Panicf("%s: failed to decode tag values: %v", reader.Path(), err)
 	}
 }
 
