@@ -27,7 +27,9 @@ import (
 
 	"github.com/apache/skywalking-banyandb/pkg/grpchelper"
 	casesmeasuredata "github.com/apache/skywalking-banyandb/test/cases/measure/data"
+	caseproperty "github.com/apache/skywalking-banyandb/test/cases/property/data"
 	casesstreamdata "github.com/apache/skywalking-banyandb/test/cases/stream/data"
+	casestrace "github.com/apache/skywalking-banyandb/test/cases/trace/data"
 )
 
 // Initialize test data.
@@ -39,7 +41,8 @@ func Initialize(addr string, now time.Time) {
 	// stream
 	casesstreamdata.Write(conn, "sw", now, interval)
 	casesstreamdata.Write(conn, "duplicated", now, 0)
-	// // measure
+	casesstreamdata.WriteToGroup(conn, "sw", "updated", "sw_updated", now.Add(time.Minute), interval)
+	// measure
 	interval = time.Minute
 	casesmeasuredata.Write(conn, "service_traffic", "index_mode", "service_traffic_data_old.json", now.AddDate(0, 0, -2), interval)
 	casesmeasuredata.Write(conn, "service_traffic", "index_mode", "service_traffic_data.json", now, interval)
@@ -57,4 +60,16 @@ func Initialize(addr string, now time.Time) {
 	casesmeasuredata.Write(conn, "service_instance_latency_minute", "sw_metric", "service_instance_latency_minute_data1.json", now.Add(1*time.Minute), interval)
 	casesmeasuredata.Write(conn, "endpoint_traffic", "sw_metric", "endpoint_traffic.json", now, interval)
 	casesmeasuredata.Write(conn, "duplicated", "exception", "duplicated.json", now, 0)
+	casesmeasuredata.Write(conn, "service_cpm_minute", "sw_updated", "service_cpm_minute_updated_data.json", now.Add(10*time.Minute), interval)
+	time.Sleep(5 * time.Second)
+	// trace
+	interval = 500 * time.Millisecond
+	casestrace.WriteToGroup(conn, "sw", "test-trace-group", "sw", now, interval)
+	casestrace.WriteToGroup(conn, "zipkin", "zipkinTrace", "zipkin", now, interval)
+	casestrace.WriteToGroup(conn, "sw", "test-trace-updated", "sw_updated", now.Add(time.Minute), interval)
+	time.Sleep(5 * time.Second)
+	casestrace.WriteToGroup(conn, "sw", "test-trace-group", "sw_mixed_traces", now.Add(time.Minute), interval)
+	// property
+	caseproperty.Write(conn, "sw1")
+	caseproperty.Write(conn, "sw2")
 }
