@@ -41,13 +41,17 @@ import (
 	"github.com/apache/skywalking-banyandb/pkg/test/gmatcher"
 	"github.com/apache/skywalking-banyandb/pkg/test/helpers"
 	test_measure "github.com/apache/skywalking-banyandb/pkg/test/measure"
+	test_property "github.com/apache/skywalking-banyandb/pkg/test/property"
 	"github.com/apache/skywalking-banyandb/pkg/test/setup"
 	test_stream "github.com/apache/skywalking-banyandb/pkg/test/stream"
+	test_trace "github.com/apache/skywalking-banyandb/pkg/test/trace"
 	"github.com/apache/skywalking-banyandb/pkg/timestamp"
 	test_cases "github.com/apache/skywalking-banyandb/test/cases"
 	casesmeasure "github.com/apache/skywalking-banyandb/test/cases/measure"
+	casesproperty "github.com/apache/skywalking-banyandb/test/cases/property"
 	casesstream "github.com/apache/skywalking-banyandb/test/cases/stream"
 	casestopn "github.com/apache/skywalking-banyandb/test/cases/topn"
+	casestrace "github.com/apache/skywalking-banyandb/test/cases/trace"
 )
 
 func TestQuery(t *testing.T) {
@@ -67,6 +71,7 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 		Env:   "dev",
 		Level: flags.LogLevel,
 	})).To(Succeed())
+	pool.EnableStackTracking(true)
 	goods = gleak.Goroutines()
 	By("Starting etcd server")
 	ports, err := test.AllocateFreePorts(2)
@@ -76,7 +81,11 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 	ep := fmt.Sprintf("http://127.0.0.1:%d", ports[0])
 	server, err := embeddedetcd.NewServer(
 		embeddedetcd.ConfigureListener([]string{ep}, []string{fmt.Sprintf("http://127.0.0.1:%d", ports[1])}),
-		embeddedetcd.RootDir(dir))
+		embeddedetcd.RootDir(dir),
+		embeddedetcd.AutoCompactionMode("periodic"),
+		embeddedetcd.AutoCompactionRetention("1h"),
+		embeddedetcd.QuotaBackendBytes(2*1024*1024*1024),
+	)
 	Expect(err).ShouldNot(HaveOccurred())
 	<-server.ReadyNotify()
 	By("Loading schema")
@@ -89,6 +98,8 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 	ctx := context.Background()
 	test_stream.PreloadSchema(ctx, schemaRegistry)
 	test_measure.PreloadSchema(ctx, schemaRegistry)
+	test_trace.PreloadSchema(ctx, schemaRegistry)
+	test_property.PreloadSchema(ctx, schemaRegistry)
 	By("Starting data node 0")
 	closeDataNode0 := setup.DataNode(ep)
 	By("Starting data node 1")
@@ -124,6 +135,14 @@ var _ = SynchronizedBeforeSuite(func() []byte {
 		Connection: connection,
 		BaseTime:   now,
 	}
+	casestrace.SharedContext = helpers.SharedContext{
+		Connection: connection,
+		BaseTime:   now,
+	}
+	casesproperty.SharedContext = helpers.SharedContext{
+		Connection: connection,
+		BaseTime:   now,
+	}
 	Expect(err).NotTo(HaveOccurred())
 })
 
@@ -131,8 +150,14 @@ var _ = SynchronizedAfterSuite(func() {
 	if connection != nil {
 		Expect(connection.Close()).To(Succeed())
 	}
-}, func() {
-	deferFunc()
-	Eventually(gleak.Goroutines, flags.EventuallyTimeout).ShouldNot(gleak.HaveLeaked(goods))
-	Eventually(pool.AllRefsCount, flags.EventuallyTimeout).Should(gmatcher.HaveZeroRef())
+}, func() {})
+
+var _ = ReportAfterSuite("Distributed Query Suite", func(report Report) {
+	if report.SuiteSucceeded {
+		if deferFunc != nil {
+			deferFunc()
+		}
+		Eventually(gleak.Goroutines, flags.EventuallyTimeout).ShouldNot(gleak.HaveLeaked(goods))
+		Eventually(pool.AllRefsCount, flags.EventuallyTimeout).Should(gmatcher.HaveZeroRef())
+	}
 })
