@@ -21,6 +21,8 @@ package setup
 import (
 	"context"
 	"fmt"
+	"os"
+	"path/filepath"
 	"sync"
 	"time"
 
@@ -33,12 +35,15 @@ import (
 	"github.com/apache/skywalking-banyandb/banyand/metadata"
 	"github.com/apache/skywalking-banyandb/banyand/metadata/schema"
 	"github.com/apache/skywalking-banyandb/pkg/cmdsetup"
+	"github.com/apache/skywalking-banyandb/pkg/logger"
 	"github.com/apache/skywalking-banyandb/pkg/run"
 	"github.com/apache/skywalking-banyandb/pkg/test"
 	testflags "github.com/apache/skywalking-banyandb/pkg/test/flags"
 	"github.com/apache/skywalking-banyandb/pkg/test/helpers"
 	test_measure "github.com/apache/skywalking-banyandb/pkg/test/measure"
+	test_property "github.com/apache/skywalking-banyandb/pkg/test/property"
 	test_stream "github.com/apache/skywalking-banyandb/pkg/test/stream"
+	test_trace "github.com/apache/skywalking-banyandb/pkg/test/trace"
 )
 
 const host = "localhost"
@@ -48,7 +53,19 @@ func Standalone(flags ...string) (string, string, func()) {
 	return StandaloneWithSchemaLoaders([]SchemaLoader{
 		&preloadService{name: "stream"},
 		&preloadService{name: "measure"},
-	}, "", "", flags...)
+		&preloadService{name: "trace"},
+		&preloadService{name: "property"},
+	}, "", "", "", "", flags...)
+}
+
+// StandaloneWithAuth wires standalone modules to build a testing ready runtime with Auth.
+func StandaloneWithAuth(username, password string, flags ...string) (string, string, func()) {
+	return StandaloneWithSchemaLoaders([]SchemaLoader{
+		&preloadService{name: "stream"},
+		&preloadService{name: "measure"},
+		&preloadService{name: "trace"},
+		&preloadService{name: "property"},
+	}, "", "", username, password, flags...)
 }
 
 // StandaloneWithTLS wires standalone modules to build a testing ready runtime with TLS enabled.
@@ -56,22 +73,29 @@ func StandaloneWithTLS(certFile, keyFile string, flags ...string) (string, strin
 	return StandaloneWithSchemaLoaders([]SchemaLoader{
 		&preloadService{name: "stream"},
 		&preloadService{name: "measure"},
-	}, certFile, keyFile, flags...)
+		&preloadService{name: "trace"},
+		&preloadService{name: "property"},
+	}, certFile, keyFile, "", "", flags...)
 }
 
 // EmptyStandalone wires standalone modules to build a testing ready runtime.
 func EmptyStandalone(flags ...string) (string, string, func()) {
-	return StandaloneWithSchemaLoaders(nil, "", "", flags...)
+	return StandaloneWithSchemaLoaders(nil, "", "", "", "", flags...)
+}
+
+// EmptyStandaloneWithAuth wires standalone modules to build a testing ready runtime with Auth.
+func EmptyStandaloneWithAuth(username, password string, flags ...string) (string, string, func()) {
+	return StandaloneWithSchemaLoaders(nil, "", "", username, password, flags...)
 }
 
 // StandaloneWithSchemaLoaders wires standalone modules to build a testing ready runtime. It also allows to preload schema.
-func StandaloneWithSchemaLoaders(schemaLoaders []SchemaLoader, certFile, keyFile string, flags ...string) (string, string, func()) {
+func StandaloneWithSchemaLoaders(schemaLoaders []SchemaLoader, certFile, keyFile string, username, password string, flags ...string) (string, string, func()) {
 	path, deferFn, err := test.NewSpace()
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	var ports []int
 	ports, err = test.AllocateFreePorts(4)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
-	addr, httpAddr, closeFn := standaloneServer(path, ports, schemaLoaders, certFile, keyFile, flags...)
+	addr, httpAddr, closeFn := standaloneServerWithAuth(path, ports, schemaLoaders, certFile, keyFile, username, password, flags...)
 	return addr, httpAddr, func() {
 		closeFn()
 		deferFn()
@@ -83,6 +107,8 @@ func ClosableStandalone(path string, ports []int, flags ...string) (string, stri
 	return standaloneServer(path, ports, []SchemaLoader{
 		&preloadService{name: "stream"},
 		&preloadService{name: "measure"},
+		&preloadService{name: "trace"},
+		&preloadService{name: "property"},
 	}, "", "", flags...)
 }
 
@@ -97,6 +123,12 @@ func EmptyClosableStandalone(path string, ports []int, flags ...string) (string,
 }
 
 func standaloneServer(path string, ports []int, schemaLoaders []SchemaLoader, certFile, keyFile string, flags ...string) (string, string, func()) {
+	return standaloneServerWithAuth(path, ports, schemaLoaders, certFile, keyFile, "", "", flags...)
+}
+
+func standaloneServerWithAuth(path string, ports []int, schemaLoaders []SchemaLoader, certFile, keyFile string,
+	username, password string, flags ...string,
+) (string, string, func()) {
 	addr := fmt.Sprintf("%s:%d", host, ports[0])
 	httpAddr := fmt.Sprintf("%s:%d", host, ports[1])
 	endpoint := fmt.Sprintf("http://%s:%d", host, ports[2])
@@ -112,6 +144,7 @@ func standaloneServer(path string, ports []int, schemaLoaders []SchemaLoader, ce
 		"--measure-root-path=" + path,
 		"--metadata-root-path=" + path,
 		"--property-root-path=" + path,
+		"--trace-root-path=" + path,
 		fmt.Sprintf("--etcd-listen-client-url=%s", endpoint), fmt.Sprintf("--etcd-listen-peer-url=http://%s:%d", host, ports[3]),
 	}
 	tlsEnabled := false
@@ -129,14 +162,16 @@ func standaloneServer(path string, ports []int, schemaLoaders []SchemaLoader, ce
 	if tlsEnabled {
 		creds, err := credentials.NewClientTLSFromFile(certFile, "localhost")
 		gomega.Expect(err).NotTo(gomega.HaveOccurred())
-		gomega.Eventually(helpers.HealthCheck(addr, 10*time.Second, 10*time.Second, grpclib.WithTransportCredentials(creds)), testflags.EventuallyTimeout).
+		gomega.Eventually(helpers.HealthCheckWithAuth(addr, 10*time.Second, 10*time.Second,
+			username, password, grpclib.WithTransportCredentials(creds)), testflags.EventuallyTimeout).
 			Should(gomega.Succeed())
-		gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, certFile), testflags.EventuallyTimeout).Should(gomega.Succeed())
+		gomega.Eventually(helpers.HTTPHealthCheckWithAuth(httpAddr, certFile, username, password), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	} else {
 		gomega.Eventually(
-			helpers.HealthCheck(addr, 10*time.Second, 10*time.Second, grpclib.WithTransportCredentials(insecure.NewCredentials())),
+			helpers.HealthCheckWithAuth(addr, 10*time.Second, 10*time.Second,
+				username, password, grpclib.WithTransportCredentials(insecure.NewCredentials())),
 			testflags.EventuallyTimeout).Should(gomega.Succeed())
-		gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, ""), testflags.EventuallyTimeout).Should(gomega.Succeed())
+		gomega.Eventually(helpers.HTTPHealthCheckWithAuth(httpAddr, "", username, password), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	}
 
 	if schemaLoaders != nil {
@@ -178,6 +213,12 @@ func (p *preloadService) PreRun(ctx context.Context) error {
 	if p.name == "stream" {
 		return test_stream.PreloadSchema(ctx, p.registry)
 	}
+	if p.name == "trace" {
+		return test_trace.PreloadSchema(ctx, p.registry)
+	}
+	if p.name == "property" {
+		return test_property.PreloadSchema(ctx, p.registry)
+	}
 	return test_measure.PreloadSchema(ctx, p.registry)
 }
 
@@ -204,11 +245,8 @@ func CMD(flags ...string) func() {
 	}
 }
 
-func startDataNode(etcdEndpoint string, flags ...string) (string, string, func(), func()) {
-	path, deferFn, err := test.NewSpace()
-	gomega.Expect(err).NotTo(gomega.HaveOccurred())
-
-	ports, err := test.AllocateFreePorts(1)
+func startDataNode(etcdEndpoint, dataDir string, flags ...string) (string, string, func()) {
+	ports, err := test.AllocateFreePorts(2)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 
 	addr := fmt.Sprintf("%s:%d", host, ports[0])
@@ -218,12 +256,16 @@ func startDataNode(etcdEndpoint string, flags ...string) (string, string, func()
 		"data",
 		"--grpc-host="+host,
 		fmt.Sprintf("--grpc-port=%d", ports[0]),
-		"--stream-root-path="+path,
-		"--measure-root-path="+path,
-		"--property-root-path="+path,
+		fmt.Sprintf("--property-repair-gossip-grpc-port=%d", ports[1]),
+		"--stream-root-path="+dataDir,
+		"--measure-root-path="+dataDir,
+		"--property-root-path="+dataDir,
+		"--trace-root-path="+dataDir,
 		"--etcd-endpoints", etcdEndpoint,
 		"--node-host-provider", "flag",
 		"--node-host", nodeHost,
+		"--logging-modules", "trace,sidx",
+		"--logging-levels", "debug,debug",
 	)
 
 	closeFn := CMD(flags...)
@@ -236,48 +278,101 @@ func startDataNode(etcdEndpoint string, flags ...string) (string, string, func()
 		return helpers.ListKeys(etcdEndpoint, fmt.Sprintf("/%s/nodes/%s:%d", metadata.DefaultNamespace, nodeHost, ports[0]))
 	}, testflags.EventuallyTimeout).Should(gomega.HaveLen(1))
 
-	return addr, path, closeFn, deferFn
+	return addr, fmt.Sprintf("%s:%d", host, ports[1]), closeFn
 }
 
 // DataNode runs a data node.
 func DataNode(etcdEndpoint string, flags ...string) func() {
-	_, _, closeFn, deferFn := startDataNode(etcdEndpoint, flags...)
+	path, deferFn, err := test.NewSpace()
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
+	_, _, closeFn := DataNodeFromDataDir(etcdEndpoint, path, flags...)
 	return func() {
+		fmt.Printf("Data tsdb path: %s\n", path)
+		_ = filepath.Walk(path, func(path string, _ os.FileInfo, err error) error {
+			if err != nil {
+				return err
+			}
+			fmt.Println(path)
+			return nil
+		})
+		fmt.Println("done")
 		closeFn()
 		deferFn()
 	}
 }
 
+// DataNodeFromDataDir runs a data node with a specific data directory.
+func DataNodeFromDataDir(etcdEndpoint, dataDir string, flags ...string) (string, string, func()) {
+	grpcAddr, propertyRepairAddr, closeFn := startDataNode(etcdEndpoint, dataDir, flags...)
+	return grpcAddr, propertyRepairAddr, closeFn
+}
+
 // DataNodeWithAddrAndDir runs a data node and returns the address and root path.
 func DataNodeWithAddrAndDir(etcdEndpoint string, flags ...string) (string, string, func()) {
-	addr, dir, closeFn, deferFn := startDataNode(etcdEndpoint, flags...)
-	return addr, dir, func() {
+	path, deferFn, err := test.NewSpace()
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
+	addr, _, closeFn := startDataNode(etcdEndpoint, path, flags...)
+	return addr, path, func() {
 		closeFn()
 		deferFn()
 	}
 }
 
 // LiaisonNode runs a liaison node.
-func LiaisonNode(etcdEndpoint string, flags ...string) (string, func()) {
-	ports, err := test.AllocateFreePorts(2)
+func LiaisonNode(etcdEndpoint string, flags ...string) (grpcAddr string, closeFn func()) {
+	grpcAddr, _, closeFn = LiaisonNodeWithHTTP(etcdEndpoint, flags...)
+	return
+}
+
+// LiaisonNodeWithHTTP runs a liaison node with HTTP enabled and returns the gRPC and HTTP addresses.
+func LiaisonNodeWithHTTP(etcdEndpoint string, flags ...string) (string, string, func()) {
+	ports, err := test.AllocateFreePorts(3)
 	gomega.Expect(err).NotTo(gomega.HaveOccurred())
-	addr := fmt.Sprintf("%s:%d", host, ports[0])
+	grpcAddr := fmt.Sprintf("%s:%d", host, ports[0])
 	httpAddr := fmt.Sprintf("%s:%d", host, ports[1])
 	nodeHost := "127.0.0.1"
+	path, deferFn, err := test.NewSpace()
+	logger.Infof("liaison test directory: %s", path)
+	gomega.Expect(err).NotTo(gomega.HaveOccurred())
 	flags = append(flags, "liaison",
 		"--grpc-host="+host,
 		fmt.Sprintf("--grpc-port=%d", ports[0]),
 		"--http-host="+host,
 		fmt.Sprintf("--http-port=%d", ports[1]),
-		"--http-grpc-addr="+addr,
+		"--liaison-server-grpc-host="+host,
+		fmt.Sprintf("--liaison-server-grpc-port=%d", ports[2]),
+		"--http-grpc-addr="+grpcAddr,
 		"--etcd-endpoints", etcdEndpoint,
 		"--node-host-provider", "flag",
 		"--node-host", nodeHost,
+		"--stream-root-path="+path,
+		"--measure-root-path="+path,
+		"--trace-root-path="+path,
+		"--stream-flush-timeout=500ms",
+		"--measure-flush-timeout=500ms",
+		"--trace-flush-timeout=500ms",
+		"--stream-sync-interval=1s",
+		"--measure-sync-interval=1s",
+		"--trace-sync-interval=1s",
+		"--logging-modules", "trace,sidx",
+		"--logging-levels", "debug,debug",
 	)
 	closeFn := CMD(flags...)
 	gomega.Eventually(helpers.HTTPHealthCheck(httpAddr, ""), testflags.EventuallyTimeout).Should(gomega.Succeed())
 	gomega.Eventually(func() (map[string]*databasev1.Node, error) {
-		return helpers.ListKeys(etcdEndpoint, fmt.Sprintf("/%s/nodes/%s:%d", metadata.DefaultNamespace, nodeHost, ports[0]))
+		return helpers.ListKeys(etcdEndpoint, fmt.Sprintf("/%s/nodes/%s:%d", metadata.DefaultNamespace, nodeHost, ports[2]))
 	}, testflags.EventuallyTimeout).Should(gomega.HaveLen(1))
-	return addr, closeFn
+	return grpcAddr, httpAddr, func() {
+		fmt.Printf("Liaison %d write queue path: %s\n", ports[0], path)
+		_ = filepath.Walk(path, func(path string, _ os.FileInfo, err error) error {
+			if err != nil {
+				return err
+			}
+			fmt.Println(path)
+			return nil
+		})
+		fmt.Println("done")
+		closeFn()
+		deferFn()
+	}
 }
