@@ -19,6 +19,8 @@ package timestamp
 
 import (
 	"time"
+
+	modelv1 "github.com/apache/skywalking-banyandb/api/proto/banyandb/model/v1"
 )
 
 // TimeRange is a range of periods into which data can be written or retrieved.
@@ -60,6 +62,22 @@ func (t TimeRange) Overlapping(other TimeRange) bool {
 	return !t.Start.After(other.End) && !other.Start.After(t.End)
 }
 
+// Include returns whether the TimeRange includes the other TimeRange.
+func (t TimeRange) Include(other TimeRange) bool {
+	var start, end bool
+	if t.Start.Equal(other.Start) {
+		start = t.IncludeStart || !other.IncludeStart
+	} else {
+		start = !t.Start.After(other.Start)
+	}
+	if t.End.Equal(other.End) {
+		end = t.IncludeEnd || !other.IncludeEnd
+	} else {
+		end = !t.End.Before(other.End)
+	}
+	return start && end
+}
+
 // Duration converts TimeRange to time.Duration.
 func (t TimeRange) Duration() time.Duration {
 	return t.End.Sub(t.Start)
@@ -122,27 +140,28 @@ func NewTimeRangeDuration(start time.Time, duration time.Duration, includeStart,
 }
 
 // FindRange returns the indices of the first and last elements in a sorted 'timestamps' slice that are within the min and max range.
-func FindRange[T int64 | uint64](timestamps []T, min, max T) (int, int, bool) {
+func FindRange[T int64 | uint64](timestamps []T, minVal, maxVal T) (int, int, bool) {
 	if len(timestamps) == 0 {
 		return -1, -1, false
 	}
-	if timestamps[0] > max {
+	isAsc := timestamps[0] <= timestamps[len(timestamps)-1]
+	if isAsc && (timestamps[0] > maxVal || timestamps[len(timestamps)-1] < minVal) {
 		return -1, -1, false
 	}
-	if timestamps[len(timestamps)-1] < min {
+	if !isAsc && (timestamps[0] < minVal || timestamps[len(timestamps)-1] > maxVal) {
 		return -1, -1, false
 	}
 
 	start, end := -1, len(timestamps)
 	for start < len(timestamps)-1 {
 		start++
-		if timestamps[start] >= min {
+		if isAsc && timestamps[start] >= minVal || !isAsc && timestamps[start] <= maxVal {
 			break
 		}
 	}
 	for end > 0 {
 		end--
-		if timestamps[end] <= max {
+		if isAsc && timestamps[end] <= maxVal || !isAsc && timestamps[end] >= minVal {
 			break
 		}
 	}
@@ -171,3 +190,31 @@ func Find(timestamps []int64, target int64) int {
 	}
 	return -1
 }
+
+// PbHasOverlap checks if two TimeRanges have an overlap.
+// Returns true if they have at least one point in common, false otherwise.
+// Note: this follows [begin, end] (inclusive on both ends)
+//
+//	other follows [begin, end) (inclusive begin, exclusive end)
+func PbHasOverlap(this, other *modelv1.TimeRange) bool {
+	if this == nil || other == nil {
+		return false
+	}
+
+	if this.Begin == nil || this.End == nil || other.Begin == nil || other.End == nil {
+		return false
+	}
+
+	thisBegin := this.Begin.GetSeconds()*1e9 + int64(this.Begin.GetNanos())
+	thisEnd := this.End.GetSeconds()*1e9 + int64(this.End.GetNanos())
+	otherBegin := other.Begin.GetSeconds()*1e9 + int64(other.Begin.GetNanos())
+	otherEnd := other.End.GetSeconds()*1e9 + int64(other.End.GetNanos())
+
+	// Check for overlap:
+	// - For x [begin, end] (inclusive on both ends)
+	// - For other [begin, end) (inclusive begin, exclusive end)
+	// They overlap when:
+	// thisBegin <= otherEnd-1 && thisEnd >= otherBegin
+	// Simplified to: thisBegin < otherEnd && thisEnd >= otherBegin
+	return thisBegin < otherEnd && thisEnd >= otherBegin
+}
